@@ -4,54 +4,99 @@
 
 package org.mozilla.geckoview.test
 
-import android.app.ActivityManager
-import android.content.Context
 import android.graphics.SurfaceTexture
-import android.net.Uri
-import android.os.Process
-import org.mozilla.geckoview.GeckoSession.NavigationDelegate.LoadRequest
-import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
-import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.IgnoreCrash
-import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.WithDisplay
-import org.mozilla.geckoview.test.util.Callbacks
-
-import android.support.annotation.AnyThread
-import androidx.test.filters.MediumTest
-import androidx.test.ext.junit.runners.AndroidJUnit4
+import android.view.PointerIcon
 import android.view.Surface
-import org.hamcrest.Matchers.*
+import androidx.annotation.AnyThread
+import androidx.core.net.toUri
+import androidx.test.ext.junit.runners.AndroidJUnit4
+import androidx.test.filters.MediumTest
+import org.hamcrest.Matchers.endsWith
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.greaterThan
+import org.hamcrest.Matchers.isEmptyOrNullString
+import org.hamcrest.Matchers.not
+import org.hamcrest.Matchers.notNullValue
+import org.hamcrest.Matchers.startsWith
 import org.json.JSONObject
 import org.junit.Assume.assumeThat
 import org.junit.Test
 import org.junit.runner.RunWith
-import org.mozilla.gecko.GeckoAppShell
-import org.mozilla.geckoview.*
+import org.mozilla.geckoview.AllowOrDeny
+import org.mozilla.geckoview.ContentBlocking.CookieBannerMode
+import org.mozilla.geckoview.GeckoDisplay.SurfaceInfo
+import org.mozilla.geckoview.GeckoResult
+import org.mozilla.geckoview.GeckoSession
+import org.mozilla.geckoview.GeckoSession.ContentDelegate
+import org.mozilla.geckoview.GeckoSession.NavigationDelegate
+import org.mozilla.geckoview.GeckoSession.NavigationDelegate.LoadRequest
+import org.mozilla.geckoview.GeckoSession.ProgressDelegate
+import org.mozilla.geckoview.GeckoSessionSettings
+import org.mozilla.geckoview.SlowScriptResponse
+import org.mozilla.geckoview.WebResponse
+import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
+import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.IgnoreCrash
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.NullDelegate
-
+import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.WithDisplay
+import java.io.ByteArrayInputStream
 
 @RunWith(AndroidJUnit4::class)
 @MediumTest
 class ContentDelegateTest : BaseSessionTest() {
     @Test fun titleChange() {
-        sessionRule.session.loadTestPath(TITLE_CHANGE_HTML_PATH)
+        mainSession.loadTestPath(TITLE_CHANGE_HTML_PATH)
 
-        sessionRule.waitUntilCalled(object : Callbacks.ContentDelegate {
+        sessionRule.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 2)
             override fun onTitleChange(session: GeckoSession, title: String?) {
-                assertThat("Title should match", title,
-                           equalTo(forEachCall("Title1", "Title2")))
+                assertThat(
+                    "Title should match",
+                    title,
+                    equalTo(forEachCall("Title1", "Title2")),
+                )
             }
         })
     }
 
-    @Test fun download() {
+    @Test fun openInAppRequest() {
+        // Testing WebResponse behavior
+        val data = "Hello, World.".toByteArray()
+        val fileHeader = "attachment; filename=\"hello-world.txt\""
+        val requestExternal = true
+        val skipConfirmation = true
+        var response = WebResponse.Builder(HELLO_HTML_PATH)
+            .statusCode(200)
+            .body(ByteArrayInputStream(data))
+            .addHeader("Content-Type", "application/txt")
+            .addHeader("Content-Length", data.size.toString())
+            .addHeader("Content-Disposition", fileHeader)
+            .requestExternalApp(requestExternal)
+            .skipConfirmation(skipConfirmation)
+            .build()
+        assertThat(
+            "Filename matches as expected",
+            response.headers["Content-Disposition"],
+            equalTo(fileHeader),
+        )
+        assertThat(
+            "Request external response matches as expected.",
+            requestExternal,
+            equalTo(response.requestExternalApp),
+        )
+        assertThat(
+            "Skipping the confirmation matches as expected.",
+            skipConfirmation,
+            equalTo(response.skipConfirmation),
+        )
+    }
+
+    @Test fun downloadOneRequest() {
         // disable test on pgo for frequently failing Bug 1543355
         assumeThat(sessionRule.env.isDebugBuild, equalTo(true))
 
-        val executor = GeckoWebExecutor(sessionRule.runtime)
-        sessionRule.session.loadTestPath(DOWNLOAD_HTML_PATH)
+        mainSession.loadTestPath(DOWNLOAD_HTML_PATH)
 
-        sessionRule.waitUntilCalled(object : Callbacks.NavigationDelegate, Callbacks.ContentDelegate {
+        sessionRule.waitUntilCalled(object : NavigationDelegate, ContentDelegate {
 
             @AssertCalled(count = 2)
             override fun onLoadRequest(session: GeckoSession, request: LoadRequest): GeckoResult<AllowOrDeny>? {
@@ -64,38 +109,42 @@ class ContentDelegateTest : BaseSessionTest() {
             }
 
             @AssertCalled(count = 1)
-            override fun onExternalResponse(session: GeckoSession, response: GeckoSession.WebResponseInfo) {
+            override fun onExternalResponse(session: GeckoSession, response: WebResponse) {
                 assertThat("Uri should start with data:", response.uri, startsWith("blob:"))
-                assertThat("Content type should match", response.contentType, equalTo("text/plain"))
-                assertThat("Content length should be non-zero", response.contentLength, greaterThan(0L))
-                assertThat("Filename should match", response.filename, equalTo("download.txt"))
-
-                val req = WebRequest.Builder(response.uri).addHeader("Accept", "text/plain").build()
-
-                val res = sessionRule.waitForResult(executor.fetch(req))
-                assertThat("We should download the thing", String(res.body?.readBytes()!!), equalTo("Downloaded Data"))
+                assertThat("We should download the thing", String(response.body?.readBytes()!!), equalTo("Downloaded Data"))
+                // The headers below are special headers that we try to get for responses of any kind (http, blob, etc.)
+                // Note the case of the header keys. In the WebResponse object, all of them are lower case.
+                assertThat("Content type should match", response.headers.get("content-type"), equalTo("text/plain"))
+                assertThat("Content length should be non-zero", response.headers.get("Content-Length")!!.toLong(), greaterThan(0L))
+                assertThat("Filename should match", response.headers.get("cONTent-diSPOsiTion"), equalTo("attachment; filename=\"download.txt\""))
+                assertThat("Request external response should not be set.", response.requestExternalApp, equalTo(false))
+                assertThat("Should not skip the confirmation on a regular download.", response.skipConfirmation, equalTo(false))
             }
         })
     }
 
     @IgnoreCrash
-    @Test fun crashContent() {
-        // This test doesn't make sense without multiprocess
-        assumeThat(sessionRule.env.isMultiprocess, equalTo(true))
+    @Test
+    fun crashContent() {
+        // TODO: bug 1710940
+        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
 
         mainSession.loadUri(CONTENT_CRASH_URL)
-        mainSession.waitUntilCalled(object : Callbacks.ContentDelegate {
+        mainSession.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
             override fun onCrash(session: GeckoSession) {
-                assertThat("Session should be closed after a crash",
-                           session.isOpen, equalTo(false))
+                assertThat(
+                    "Session should be closed after a crash",
+                    session.isOpen,
+                    equalTo(false),
+                )
             }
         })
 
         // Recover immediately
         mainSession.open()
         mainSession.loadTestPath(HELLO_HTML_PATH)
-        mainSession.waitUntilCalled(object: Callbacks.ProgressDelegate {
+        mainSession.waitUntilCalled(object : ProgressDelegate {
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
                 assertThat("Page should load successfully", success, equalTo(true))
@@ -105,11 +154,12 @@ class ContentDelegateTest : BaseSessionTest() {
 
     @IgnoreCrash
     @WithDisplay(width = 10, height = 10)
-    @Test fun crashContent_tapAfterCrash() {
-        // This test doesn't make sense without multiprocess
-        assumeThat(sessionRule.env.isMultiprocess, equalTo(true))
+    @Test
+    fun crashContent_tapAfterCrash() {
+        // TODO: bug 1710940
+        assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
 
-        mainSession.delegateUntilTestEnd(object : Callbacks.ContentDelegate {
+        mainSession.delegateUntilTestEnd(object : ContentDelegate {
             override fun onCrash(session: GeckoSession) {
                 mainSession.open()
                 mainSession.loadTestPath(HELLO_HTML_PATH)
@@ -127,32 +177,30 @@ class ContentDelegateTest : BaseSessionTest() {
 
     @AnyThread
     fun killAllContentProcesses() {
-        val context = GeckoAppShell.getApplicationContext()
-        val manager = context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
-        val expr = ".*:tab\\d+$".toRegex()
-        for (info in manager.runningAppProcesses) {
-            if (info.processName.matches(expr)) {
-                Process.killProcess(info.pid)
-            }
+        val contentProcessPids = sessionRule.getAllSessionPids()
+        for (pid in contentProcessPids) {
+            sessionRule.killContentProcess(pid)
         }
     }
 
     @IgnoreCrash
-    @Test fun killContent() {
-        assumeThat(sessionRule.env.isMultiprocess, equalTo(true))
-
+    @Test
+    fun killContent() {
         killAllContentProcesses()
-        mainSession.waitUntilCalled(object : Callbacks.ContentDelegate {
+        mainSession.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
             override fun onKill(session: GeckoSession) {
-                assertThat("Session should be closed after being killed",
-                        session.isOpen, equalTo(false))
+                assertThat(
+                    "Session should be closed after being killed",
+                    session.isOpen,
+                    equalTo(false),
+                )
             }
         })
 
         mainSession.open()
         mainSession.loadTestPath(HELLO_HTML_PATH)
-        mainSession.waitUntilCalled(object : Callbacks.ProgressDelegate {
+        mainSession.waitUntilCalled(object : ProgressDelegate {
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
                 assertThat("Page should load successfully", success, equalTo(true))
@@ -164,19 +212,20 @@ class ContentDelegateTest : BaseSessionTest() {
         sessionRule.setPrefsUntilTestEnd(mapOf("full-screen-api.allow-trusted-requests-only" to false))
         mainSession.loadTestPath(FULLSCREEN_PATH)
         mainSession.waitForPageStop()
-        mainSession.evaluateJS("document.querySelector('#fullscreen').requestFullscreen(); true")
-        sessionRule.waitUntilCalled(object : Callbacks.ContentDelegate {
+        val promise = mainSession.evaluatePromiseJS("document.querySelector('#fullscreen').requestFullscreen()")
+        sessionRule.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
-            override  fun onFullScreen(session: GeckoSession, fullScreen: Boolean) {
+            override fun onFullScreen(session: GeckoSession, fullScreen: Boolean) {
                 assertThat("Div went fullscreen", fullScreen, equalTo(true))
             }
         })
+        promise.value
     }
 
     private fun waitForFullscreenExit() {
-        sessionRule.waitUntilCalled(object : Callbacks.ContentDelegate {
+        sessionRule.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
-            override  fun onFullScreen(session: GeckoSession, fullScreen: Boolean) {
+            override fun onFullScreen(session: GeckoSession, fullScreen: Boolean) {
                 assertThat("Div left fullscreen", fullScreen, equalTo(false))
             }
         })
@@ -184,8 +233,9 @@ class ContentDelegateTest : BaseSessionTest() {
 
     @Test fun fullscreen() {
         goFullscreen()
-        mainSession.evaluateJS("document.exitFullscreen(); true")
+        val promise = mainSession.evaluatePromiseJS("document.exitFullscreen()")
         waitForFullscreenExit()
+        promise.value
     }
 
     @Test fun sessionExitFullscreen() {
@@ -199,16 +249,16 @@ class ContentDelegateTest : BaseSessionTest() {
         val texture = SurfaceTexture(0)
         texture.setDefaultBufferSize(100, 100)
         val surface = Surface(texture)
-        display.surfaceChanged(surface, 100, 100)
+        display.surfaceChanged(SurfaceInfo.Builder(surface).size(100, 100).build())
         mainSession.loadTestPath(HELLO_HTML_PATH)
-        sessionRule.waitUntilCalled(object : Callbacks.ContentDelegate {
+        sessionRule.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
             override fun onFirstComposite(session: GeckoSession) {
             }
         })
         display.surfaceDestroyed()
-        display.surfaceChanged(surface, 100, 100)
-        sessionRule.waitUntilCalled(object : Callbacks.ContentDelegate {
+        display.surfaceChanged(SurfaceInfo.Builder(surface).size(100, 100).build())
+        sessionRule.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
             override fun onFirstComposite(session: GeckoSession) {
             }
@@ -218,9 +268,10 @@ class ContentDelegateTest : BaseSessionTest() {
     }
 
     @WithDisplay(width = 10, height = 10)
-    @Test fun firstContentfulPaint() {
+    @Test
+    fun firstContentfulPaint() {
         mainSession.loadTestPath(HELLO_HTML_PATH)
-        sessionRule.waitUntilCalled(object : Callbacks.ContentDelegate {
+        sessionRule.waitUntilCalled(object : ContentDelegate {
             @AssertCalled(count = 1)
             override fun onFirstContentfulPaint(session: GeckoSession) {
             }
@@ -253,7 +304,7 @@ class ContentDelegateTest : BaseSessionTest() {
 
     @Test fun webAppManifest() {
         mainSession.loadTestPath(HELLO_HTML_PATH)
-        mainSession.waitUntilCalled(object : Callbacks.All {
+        mainSession.waitUntilCalled(object : ContentDelegate, ProgressDelegate {
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
                 assertThat("Page load should succeed", success, equalTo(true))
@@ -271,20 +322,30 @@ class ContentDelegateTest : BaseSessionTest() {
                 assertThat("background_color should match", manifest.getString("background_color"), equalTo("#eec0ffee"))
                 assertThat("start_url should match", manifest.getString("start_url"), endsWith("/assets/www/start/index.html"))
 
-                val icon = manifest.getJSONArray("icons").getJSONObject(0);
+                val icon = manifest.getJSONArray("icons").getJSONObject(0)
 
-                val iconSrc = Uri.parse(icon.getString("src"))
+                val iconSrc = icon.getString("src").toUri()
                 assertThat("icon should have a valid src", iconSrc, notNullValue())
                 assertThat("icon src should be absolute", iconSrc.isAbsolute, equalTo(true))
-                assertThat("icon should have sizes", icon.getString("sizes"),  not(isEmptyOrNullString()))
+                assertThat("icon should have sizes", icon.getString("sizes"), not(isEmptyOrNullString()))
                 assertThat("icon type should match", icon.getString("type"), equalTo("image/gif"))
             }
         })
     }
 
+    @Test fun previewImage() {
+        mainSession.loadTestPath(METATAGS_PATH)
+        mainSession.waitUntilCalled(object : ContentDelegate, ProgressDelegate {
+            @AssertCalled(count = 1)
+            override fun onPreviewImage(session: GeckoSession, previewImageUrl: String) {
+                assertThat("Preview image should match", previewImageUrl, equalTo("https://test.com/og-image-url"))
+            }
+        })
+    }
+
     @Test fun viewportFit() {
         mainSession.loadTestPath(VIEWPORT_PATH)
-        mainSession.waitUntilCalled(object : Callbacks.All {
+        mainSession.waitUntilCalled(object : ContentDelegate, ProgressDelegate {
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
                 assertThat("Page load should succeed", success, equalTo(true))
@@ -297,7 +358,7 @@ class ContentDelegateTest : BaseSessionTest() {
         })
 
         mainSession.loadTestPath(HELLO_HTML_PATH)
-        mainSession.waitUntilCalled(object : Callbacks.All {
+        mainSession.waitUntilCalled(object : ContentDelegate, ProgressDelegate {
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
                 assertThat("Page load should succeed", success, equalTo(true))
@@ -310,32 +371,153 @@ class ContentDelegateTest : BaseSessionTest() {
         })
     }
 
+    @Test fun closeRequest() {
+        if (!sessionRule.env.isAutomation) {
+            sessionRule.setPrefsUntilTestEnd(mapOf("dom.allow_scripts_to_close_windows" to true))
+        }
+
+        mainSession.loadTestPath(HELLO_HTML_PATH)
+        mainSession.waitForPageStop()
+
+        mainSession.evaluateJS("window.close()")
+        mainSession.waitUntilCalled(object : ContentDelegate {
+            @AssertCalled(count = 1)
+            override fun onCloseRequest(session: GeckoSession) {
+            }
+        })
+    }
+
+    @Test fun windowOpenClose() {
+        sessionRule.setPrefsUntilTestEnd(mapOf("dom.disable_open_during_load" to false))
+
+        mainSession.loadTestPath(HELLO_HTML_PATH)
+        mainSession.waitForPageStop()
+
+        val newSession = sessionRule.createClosedSession()
+        mainSession.delegateDuringNextWait(object : NavigationDelegate {
+            @AssertCalled(count = 1)
+            override fun onNewSession(session: GeckoSession, uri: String): GeckoResult<GeckoSession>? {
+                return GeckoResult.fromValue(newSession)
+            }
+        })
+
+        mainSession.evaluateJS("const w = window.open('about:blank'); w.close()")
+
+        newSession.waitUntilCalled(object : ContentDelegate, ProgressDelegate {
+            @AssertCalled(count = 1)
+            override fun onCloseRequest(session: GeckoSession) {
+            }
+
+            @AssertCalled(count = 1)
+            override fun onPageStop(session: GeckoSession, success: Boolean) {
+            }
+        })
+    }
+
+    @Test fun cookieBannerDetectedEvent() {
+        sessionRule.setPrefsUntilTestEnd(
+            mapOf(
+                "cookiebanners.service.mode" to CookieBannerMode.COOKIE_BANNER_MODE_REJECT,
+            ),
+        )
+
+        val detectHandled = GeckoResult<Void>()
+        mainSession.delegateUntilTestEnd(object : GeckoSession.ContentDelegate {
+            override fun onCookieBannerDetected(
+                session: GeckoSession,
+            ) {
+                detectHandled.complete(null)
+            }
+        })
+
+        mainSession.loadTestPath(HELLO_HTML_PATH)
+        mainSession.waitForPageStop()
+        mainSession.triggerCookieBannerDetected()
+
+        sessionRule.waitForResult(detectHandled)
+    }
+
+    @Test fun cookieBannerHandledEvent() {
+        sessionRule.setPrefsUntilTestEnd(
+            mapOf(
+                "cookiebanners.service.mode" to CookieBannerMode.COOKIE_BANNER_MODE_REJECT,
+            ),
+        )
+
+        val handleHandled = GeckoResult<Void>()
+        mainSession.delegateUntilTestEnd(object : GeckoSession.ContentDelegate {
+            override fun onCookieBannerHandled(
+                session: GeckoSession,
+            ) {
+                handleHandled.complete(null)
+            }
+        })
+
+        mainSession.loadTestPath(HELLO_HTML_PATH)
+        mainSession.waitForPageStop()
+        mainSession.triggerCookieBannerHandled()
+
+        sessionRule.waitForResult(handleHandled)
+    }
+
+    @WithDisplay(width = 100, height = 100)
+    @Test
+    fun setCursor() {
+        mainSession.loadTestPath(HELLO_HTML_PATH)
+        mainSession.waitForPageStop()
+
+        mainSession.evaluateJS("document.body.style.cursor = 'wait'")
+        mainSession.synthesizeMouseMove(50, 50)
+
+        mainSession.waitUntilCalled(object : ContentDelegate {
+            @AssertCalled(count = 1)
+            override fun onPointerIconChange(session: GeckoSession, icon: PointerIcon) {
+                // PointerIcon has no compare method.
+            }
+        })
+
+        val delegate = mainSession.contentDelegate
+        mainSession.contentDelegate = null
+        mainSession.evaluateJS("document.body.style.cursor = 'text'")
+        for (i in 51..70) {
+            mainSession.synthesizeMouseMove(i, 50)
+            // No wait function since we remove content delegate.
+            mainSession.waitForJS("new Promise(resolve => window.setTimeout(resolve, 100))")
+        }
+        mainSession.contentDelegate = delegate
+    }
+
     /**
      * Preferences to induce wanted behaviour.
      */
     private fun setHangReportTestPrefs(timeout: Int = 20000) {
-        sessionRule.setPrefsUntilTestEnd(mapOf(
+        sessionRule.setPrefsUntilTestEnd(
+            mapOf(
                 "dom.max_script_run_time" to 1,
                 "dom.max_chrome_script_run_time" to 1,
                 "dom.max_ext_content_script_run_time" to 1,
                 "dom.ipc.cpow.timeout" to 100,
-                "browser.hangNotification.waitPeriod" to timeout
-        ))
+                "browser.hangNotification.waitPeriod" to timeout,
+            ),
+        )
     }
 
     /**
      * With no delegate set, the default behaviour is to stop hung scripts.
      */
-    @NullDelegate(GeckoSession.ContentDelegate::class)
-    @Test fun stopHungProcessDefault() {
+    @NullDelegate(ContentDelegate::class)
+    @Test
+    fun stopHungProcessDefault() {
         setHangReportTestPrefs()
         mainSession.loadTestPath(HUNG_SCRIPT)
-        sessionRule.delegateUntilTestEnd(object : Callbacks.ProgressDelegate {
+        sessionRule.delegateUntilTestEnd(object : ProgressDelegate {
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
-                assertThat("The script did not complete.",
-                        sessionRule.session.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
-                        equalTo("Started"))
+                assertThat(
+                    "The script did not complete.",
+                    mainSession.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
+                    equalTo("Started"),
+                )
             }
         })
         sessionRule.waitForPageStop(mainSession)
@@ -347,13 +529,15 @@ class ContentDelegateTest : BaseSessionTest() {
      */
     @Test fun stopHungProcessNull() {
         setHangReportTestPrefs()
-        sessionRule.delegateUntilTestEnd(object : GeckoSession.ContentDelegate, Callbacks.ProgressDelegate {
+        sessionRule.delegateUntilTestEnd(object : ContentDelegate, ProgressDelegate {
             // default onSlowScript returns null
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
-                assertThat("The script did not complete.",
-                        sessionRule.session.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
-                        equalTo("Started"))
+                assertThat(
+                    "The script did not complete.",
+                    mainSession.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
+                    equalTo("Started"),
+                )
             }
         })
         mainSession.loadTestPath(HUNG_SCRIPT)
@@ -366,18 +550,21 @@ class ContentDelegateTest : BaseSessionTest() {
     @Test fun stopHungProcessDoNothing() {
         setHangReportTestPrefs()
         var scriptHungReportCount = 0
-        sessionRule.delegateUntilTestEnd(object : GeckoSession.ContentDelegate, Callbacks.ProgressDelegate {
+        sessionRule.delegateUntilTestEnd(object : ContentDelegate, ProgressDelegate {
             @AssertCalled()
             override fun onSlowScript(geckoSession: GeckoSession, scriptFileName: String): GeckoResult<SlowScriptResponse> {
-                scriptHungReportCount += 1;
+                scriptHungReportCount += 1
                 return GeckoResult.fromValue(null)
             }
+
             @AssertCalled(count = 1)
             override fun onPageStop(session: GeckoSession, success: Boolean) {
                 assertThat("The delegate was informed of the hang repeatedly", scriptHungReportCount, greaterThan(1))
-                assertThat("The script did complete.",
-                        sessionRule.session.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
-                        equalTo("Finished"))
+                assertThat(
+                    "The script did complete.",
+                    mainSession.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
+                    equalTo("Finished"),
+                )
             }
         })
         mainSession.loadTestPath(HUNG_SCRIPT)
@@ -389,16 +576,19 @@ class ContentDelegateTest : BaseSessionTest() {
      */
     @Test fun stopHungProcess() {
         setHangReportTestPrefs()
-        sessionRule.delegateUntilTestEnd(object : GeckoSession.ContentDelegate, Callbacks.ProgressDelegate {
+        sessionRule.delegateUntilTestEnd(object : ContentDelegate, ProgressDelegate {
             @AssertCalled(count = 1, order = [1])
             override fun onSlowScript(geckoSession: GeckoSession, scriptFileName: String): GeckoResult<SlowScriptResponse> {
                 return GeckoResult.fromValue(SlowScriptResponse.STOP)
             }
+
             @AssertCalled(count = 1, order = [2])
             override fun onPageStop(session: GeckoSession, success: Boolean) {
-                assertThat("The script did not complete.",
-                        sessionRule.session.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
-                        equalTo("Started"))
+                assertThat(
+                    "The script did not complete.",
+                    mainSession.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
+                    equalTo("Started"),
+                )
             }
         })
         mainSession.loadTestPath(HUNG_SCRIPT)
@@ -410,16 +600,19 @@ class ContentDelegateTest : BaseSessionTest() {
      */
     @Test fun stopHungProcessWait() {
         setHangReportTestPrefs()
-        sessionRule.delegateUntilTestEnd(object : GeckoSession.ContentDelegate, Callbacks.ProgressDelegate {
+        sessionRule.delegateUntilTestEnd(object : ContentDelegate, ProgressDelegate {
             @AssertCalled(count = 1, order = [1])
             override fun onSlowScript(geckoSession: GeckoSession, scriptFileName: String): GeckoResult<SlowScriptResponse> {
                 return GeckoResult.fromValue(SlowScriptResponse.CONTINUE)
             }
+
             @AssertCalled(count = 1, order = [2])
             override fun onPageStop(session: GeckoSession, success: Boolean) {
-                assertThat("The script did complete.",
-                        sessionRule.session.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
-                        equalTo("Finished"))
+                assertThat(
+                    "The script did complete.",
+                    mainSession.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
+                    equalTo("Finished"),
+                )
             }
         })
         mainSession.loadTestPath(HUNG_SCRIPT)
@@ -432,24 +625,47 @@ class ContentDelegateTest : BaseSessionTest() {
     @Test fun stopHungProcessWaitThenStop() {
         setHangReportTestPrefs(500)
         var scriptWaited = false
-        sessionRule.delegateUntilTestEnd(object : GeckoSession.ContentDelegate, Callbacks.ProgressDelegate {
+        sessionRule.delegateUntilTestEnd(object : ContentDelegate, ProgressDelegate {
             @AssertCalled(count = 2, order = [1, 2])
             override fun onSlowScript(geckoSession: GeckoSession, scriptFileName: String): GeckoResult<SlowScriptResponse> {
                 return if (!scriptWaited) {
-                    scriptWaited = true;
+                    scriptWaited = true
                     GeckoResult.fromValue(SlowScriptResponse.CONTINUE)
                 } else {
                     GeckoResult.fromValue(SlowScriptResponse.STOP)
                 }
             }
+
             @AssertCalled(count = 1, order = [3])
             override fun onPageStop(session: GeckoSession, success: Boolean) {
-                assertThat("The script did not complete.",
-                        sessionRule.session.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
-                        equalTo("Started"))
+                assertThat(
+                    "The script did not complete.",
+                    mainSession.evaluateJS("document.getElementById(\"content\").innerHTML") as String,
+                    equalTo("Started"),
+                )
             }
         })
         mainSession.loadTestPath(HUNG_SCRIPT)
         sessionRule.waitForPageStop(mainSession)
     }
+
+    /**
+     * Test that the display mode is applied to CSS media query
+     */
+    @Test fun displayMode() {
+        val pwaSession = sessionRule.createOpenSession(
+            GeckoSessionSettings.Builder(mainSession.settings)
+                .displayMode(GeckoSessionSettings.DISPLAY_MODE_FULLSCREEN)
+                .build(),
+        )
+        pwaSession.loadTestPath(HELLO_HTML_PATH)
+        pwaSession.waitForPageStop()
+
+        val matches = pwaSession.evaluateJS("window.matchMedia('(display-mode: fullscreen)').matches") as Boolean
+        assertThat(
+            "display-mode should be fullscreen",
+            matches,
+            equalTo(true),
+        )
+    }
 }
