@@ -23,11 +23,44 @@ pub enum InlineBaseDirection {
     RightToLeft,
 }
 
+/// The writing-mode property (different from the WritingMode enum).
+/// https://drafts.csswg.org/css-writing-modes/#block-flow
+/// Aliases come from https://drafts.csswg.org/css-writing-modes-4/#svg-writing-mode
+#[allow(missing_docs)]
+#[derive(
+    Clone,
+    Copy,
+    Debug,
+    Eq,
+    FromPrimitive,
+    MallocSizeOf,
+    Parse,
+    PartialEq,
+    SpecifiedValueInfo,
+    ToComputedValue,
+    ToCss,
+    ToResolvedValue,
+    ToShmem,
+)]
+#[repr(u8)]
+pub enum WritingModeProperty {
+    #[parse(aliases = "lr,lr-tb,rl,rl-tb")]
+    HorizontalTb,
+    #[parse(aliases = "tb,tb-rl")]
+    VerticalRl,
+    VerticalLr,
+    #[cfg(feature = "gecko")]
+    SidewaysRl,
+    #[cfg(feature = "gecko")]
+    SidewaysLr,
+}
+
 // TODO: improve the readability of the WritingMode serialization, refer to the Debug:fmt()
+#[derive(Clone, Copy, Debug, Eq, MallocSizeOf, PartialEq, Serialize)]
+#[repr(C)]
+pub struct WritingMode(u8);
 bitflags!(
-    #[derive(Clone, Copy, Eq, MallocSizeOf, PartialEq, Serialize)]
-    #[repr(C)]
-    pub struct WritingMode: u8 {
+    impl WritingMode: u8 {
         /// A vertical writing mode; writing-mode is vertical-rl,
         /// vertical-lr, sideways-lr, or sideways-rl.
         const VERTICAL = 1 << 0;
@@ -81,7 +114,6 @@ impl WritingMode {
     /// Return a WritingMode bitflags from the relevant CSS properties.
     pub fn new(inheritedbox_style: &style_structs::InheritedBox) -> Self {
         use crate::properties::longhands::direction::computed_value::T as Direction;
-        use crate::properties::longhands::writing_mode::computed_value::T as SpecifiedWritingMode;
 
         let mut flags = WritingMode::empty();
 
@@ -96,18 +128,18 @@ impl WritingMode {
         }
 
         match writing_mode {
-            SpecifiedWritingMode::HorizontalTb => {
+            WritingModeProperty::HorizontalTb => {
                 if direction == Direction::Rtl {
                     flags.insert(WritingMode::INLINE_REVERSED);
                 }
             },
-            SpecifiedWritingMode::VerticalRl => {
+            WritingModeProperty::VerticalRl => {
                 flags.insert(WritingMode::VERTICAL);
                 if direction == Direction::Rtl {
                     flags.insert(WritingMode::INLINE_REVERSED);
                 }
             },
-            SpecifiedWritingMode::VerticalLr => {
+            WritingModeProperty::VerticalLr => {
                 flags.insert(WritingMode::VERTICAL);
                 flags.insert(WritingMode::VERTICAL_LR);
                 flags.insert(WritingMode::LINE_INVERTED);
@@ -116,7 +148,7 @@ impl WritingMode {
                 }
             },
             #[cfg(feature = "gecko")]
-            SpecifiedWritingMode::SidewaysRl => {
+            WritingModeProperty::SidewaysRl => {
                 flags.insert(WritingMode::VERTICAL);
                 flags.insert(WritingMode::VERTICAL_SIDEWAYS);
                 if direction == Direction::Rtl {
@@ -124,7 +156,7 @@ impl WritingMode {
                 }
             },
             #[cfg(feature = "gecko")]
-            SpecifiedWritingMode::SidewaysLr => {
+            WritingModeProperty::SidewaysLr => {
                 flags.insert(WritingMode::VERTICAL);
                 flags.insert(WritingMode::VERTICAL_LR);
                 flags.insert(WritingMode::VERTICAL_SIDEWAYS);
@@ -141,7 +173,7 @@ impl WritingMode {
             // text-orientation only has an effect for vertical-rl and
             // vertical-lr values of writing-mode.
             match writing_mode {
-                SpecifiedWritingMode::VerticalRl | SpecifiedWritingMode::VerticalLr => {
+                WritingModeProperty::VerticalRl | WritingModeProperty::VerticalLr => {
                     match inheritedbox_style.clone_text_orientation() {
                         TextOrientation::Mixed => {},
                         TextOrientation::Upright => {
@@ -263,29 +295,9 @@ impl WritingMode {
         }
     }
 
-    #[inline]
-    fn physical_sides_to_corner(
-        block_side: PhysicalSide,
-        inline_side: PhysicalSide,
-    ) -> PhysicalCorner {
-        match (block_side, inline_side) {
-            (PhysicalSide::Top, PhysicalSide::Left) | (PhysicalSide::Left, PhysicalSide::Top) => {
-                PhysicalCorner::TopLeft
-            },
-            (PhysicalSide::Top, PhysicalSide::Right) | (PhysicalSide::Right, PhysicalSide::Top) => {
-                PhysicalCorner::TopRight
-            },
-            (PhysicalSide::Bottom, PhysicalSide::Right) |
-            (PhysicalSide::Right, PhysicalSide::Bottom) => PhysicalCorner::BottomRight,
-            (PhysicalSide::Bottom, PhysicalSide::Left) |
-            (PhysicalSide::Left, PhysicalSide::Bottom) => PhysicalCorner::BottomLeft,
-            _ => unreachable!("block and inline sides must be orthogonal"),
-        }
-    }
-
     #[inline]
     pub fn start_start_physical_corner(&self) -> PhysicalCorner {
-        WritingMode::physical_sides_to_corner(
+        PhysicalCorner::from_sides(
             self.block_start_physical_side(),
             self.inline_start_physical_side(),
         )
@@ -293,7 +305,7 @@ impl WritingMode {
 
     #[inline]
     pub fn start_end_physical_corner(&self) -> PhysicalCorner {
-        WritingMode::physical_sides_to_corner(
+        PhysicalCorner::from_sides(
             self.block_start_physical_side(),
             self.inline_end_physical_side(),
         )
@@ -301,7 +313,7 @@ impl WritingMode {
 
     #[inline]
     pub fn end_start_physical_corner(&self) -> PhysicalCorner {
-        WritingMode::physical_sides_to_corner(
+        PhysicalCorner::from_sides(
             self.block_end_physical_side(),
             self.inline_start_physical_side(),
         )
@@ -309,7 +321,7 @@ impl WritingMode {
 
     #[inline]
     pub fn end_end_physical_corner(&self) -> PhysicalCorner {
-        WritingMode::physical_sides_to_corner(
+        PhysicalCorner::from_sides(
             self.block_end_physical_side(),
             self.inline_end_physical_side(),
         )
@@ -344,6 +356,12 @@ impl WritingMode {
             bidi::Level::rtl()
         }
     }
+
+    #[inline]
+    /// Is the text layout vertical?
+    pub fn is_text_vertical(&self) -> bool {
+        self.is_vertical() && !self.is_sideways()
+    }
 }
 
 impl fmt::Display for WritingMode {
@@ -493,14 +511,14 @@ impl<T> LogicalSize<T> {
     }
 }
 
-impl<T: Copy> LogicalSize<T> {
+impl<T: Clone> LogicalSize<T> {
     #[inline]
     pub fn width(&self, mode: WritingMode) -> T {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
-            self.block
+            self.block.clone()
         } else {
-            self.inline
+            self.inline.clone()
         }
     }
 
@@ -518,9 +536,9 @@ impl<T: Copy> LogicalSize<T> {
     pub fn height(&self, mode: WritingMode) -> T {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
-            self.inline
+            self.inline.clone()
         } else {
-            self.block
+            self.block.clone()
         }
     }
 
@@ -538,9 +556,9 @@ impl<T: Copy> LogicalSize<T> {
     pub fn to_physical(&self, mode: WritingMode) -> Size2D<T> {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
-            Size2D::new(self.block, self.inline)
+            Size2D::new(self.block.clone(), self.inline.clone())
         } else {
-            Size2D::new(self.inline, self.block)
+            Size2D::new(self.inline.clone(), self.block.clone())
         }
     }
 
@@ -548,7 +566,7 @@ impl<T: Copy> LogicalSize<T> {
     pub fn convert(&self, mode_from: WritingMode, mode_to: WritingMode) -> LogicalSize<T> {
         if mode_from == mode_to {
             self.debug_writing_mode.check(mode_from);
-            *self
+            self.clone()
         } else {
             LogicalSize::from_physical(mode_to, self.to_physical(mode_from))
         }
@@ -921,10 +939,10 @@ impl<T> LogicalMargin<T> {
     }
 }
 
-impl<T: Copy> LogicalMargin<T> {
+impl<T: Clone> LogicalMargin<T> {
     #[inline]
     pub fn new_all_same(mode: WritingMode, value: T) -> LogicalMargin<T> {
-        LogicalMargin::new(mode, value, value, value, value)
+        LogicalMargin::new(mode, value.clone(), value.clone(), value.clone(), value)
     }
 
     #[inline]
@@ -932,12 +950,12 @@ impl<T: Copy> LogicalMargin<T> {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
             if mode.is_inline_tb() {
-                self.inline_start
+                self.inline_start.clone()
             } else {
-                self.inline_end
+                self.inline_end.clone()
             }
         } else {
-            self.block_start
+            self.block_start.clone()
         }
     }
 
@@ -960,15 +978,15 @@ impl<T: Copy> LogicalMargin<T> {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
             if mode.is_vertical_lr() {
-                self.block_end
+                self.block_end.clone()
             } else {
-                self.block_start
+                self.block_start.clone()
             }
         } else {
             if mode.is_bidi_ltr() {
-                self.inline_end
+                self.inline_end.clone()
             } else {
-                self.inline_start
+                self.inline_start.clone()
             }
         }
     }
@@ -996,12 +1014,12 @@ impl<T: Copy> LogicalMargin<T> {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
             if mode.is_inline_tb() {
-                self.inline_end
+                self.inline_end.clone()
             } else {
-                self.inline_start
+                self.inline_start.clone()
             }
         } else {
-            self.block_end
+            self.block_end.clone()
         }
     }
 
@@ -1024,15 +1042,15 @@ impl<T: Copy> LogicalMargin<T> {
         self.debug_writing_mode.check(mode);
         if mode.is_vertical() {
             if mode.is_vertical_lr() {
-                self.block_start
+                self.block_start.clone()
             } else {
-                self.block_end
+                self.block_end.clone()
             }
         } else {
             if mode.is_bidi_ltr() {
-                self.inline_start
+                self.inline_start.clone()
             } else {
-                self.inline_end
+                self.inline_end.clone()
             }
         }
     }
@@ -1064,28 +1082,28 @@ impl<T: Copy> LogicalMargin<T> {
         let left;
         if mode.is_vertical() {
             if mode.is_vertical_lr() {
-                left = self.block_start;
-                right = self.block_end;
+                left = self.block_start.clone();
+                right = self.block_end.clone();
             } else {
-                right = self.block_start;
-                left = self.block_end;
+                right = self.block_start.clone();
+                left = self.block_end.clone();
             }
             if mode.is_inline_tb() {
-                top = self.inline_start;
-                bottom = self.inline_end;
+                top = self.inline_start.clone();
+                bottom = self.inline_end.clone();
             } else {
-                bottom = self.inline_start;
-                top = self.inline_end;
+                bottom = self.inline_start.clone();
+                top = self.inline_end.clone();
             }
         } else {
-            top = self.block_start;
-            bottom = self.block_end;
+            top = self.block_start.clone();
+            bottom = self.block_end.clone();
             if mode.is_bidi_ltr() {
-                left = self.inline_start;
-                right = self.inline_end;
+                left = self.inline_start.clone();
+                right = self.inline_end.clone();
             } else {
-                right = self.inline_start;
-                left = self.inline_end;
+                right = self.inline_start.clone();
+                left = self.inline_end.clone();
             }
         }
         SideOffsets2D::new(top, right, bottom, left)
@@ -1095,7 +1113,7 @@ impl<T: Copy> LogicalMargin<T> {
     pub fn convert(&self, mode_from: WritingMode, mode_to: WritingMode) -> LogicalMargin<T> {
         if mode_from == mode_to {
             self.debug_writing_mode.check(mode_from);
-            *self
+            self.clone()
         } else {
             LogicalMargin::from_physical(mode_to, self.to_physical(mode_from))
         }
@@ -1456,17 +1474,187 @@ impl<T: Copy + Add<T, Output = T> + Sub<T, Output = T>> Sub<LogicalMargin<T>> fo
 }
 
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[repr(u8)]
+pub enum LogicalAxis {
+    Block = 0,
+    Inline,
+}
+
+impl LogicalAxis {
+    #[inline]
+    pub fn to_physical(self, wm: WritingMode) -> PhysicalAxis {
+        if wm.is_horizontal() == (self == Self::Inline) {
+            PhysicalAxis::Horizontal
+        } else {
+            PhysicalAxis::Vertical
+        }
+    }
+}
+
+#[derive(Clone, Copy, Debug, PartialEq)]
+#[repr(u8)]
+pub enum LogicalSide {
+    BlockStart = 0,
+    BlockEnd,
+    InlineStart,
+    InlineEnd,
+}
+
+impl LogicalSide {
+    fn is_block(self) -> bool {
+        matches!(self, Self::BlockStart | Self::BlockEnd)
+    }
+
+    #[inline]
+    pub fn to_physical(self, wm: WritingMode) -> PhysicalSide {
+        // Block mapping depends only on vertical+vertical-lr
+        static BLOCK_MAPPING: [[PhysicalSide; 2]; 4] = [
+            [PhysicalSide::Top, PhysicalSide::Bottom], // horizontal-tb
+            [PhysicalSide::Right, PhysicalSide::Left], // vertical-rl
+            [PhysicalSide::Bottom, PhysicalSide::Top], // (horizontal-bt)
+            [PhysicalSide::Left, PhysicalSide::Right], // vertical-lr
+        ];
+
+        if self.is_block() {
+            let vertical = wm.is_vertical();
+            let lr = wm.is_vertical_lr();
+            let index = (vertical as usize) | ((lr as usize) << 1);
+            return BLOCK_MAPPING[index][self as usize];
+        }
+
+        // start = 0, end = 1
+        let edge = self as usize - 2;
+        // Inline axis sides depend on all three of writing-mode, text-orientation and direction,
+        // which are encoded in the VERTICAL, INLINE_REVERSED, VERTICAL_LR and LINE_INVERTED bits.
+        //
+        //   bit 0 = the VERTICAL value
+        //   bit 1 = the INLINE_REVERSED value
+        //   bit 2 = the VERTICAL_LR value
+        //   bit 3 = the LINE_INVERTED value
+        //
+        // Note that not all of these combinations can actually be specified via CSS: there is no
+        // horizontal-bt writing-mode, and no text-orientation value that produces "inverted"
+        // text. (The former 'sideways-left' value, no longer in the spec, would have produced
+        // this in vertical-rl mode.)
+        static INLINE_MAPPING: [[PhysicalSide; 2]; 16] = [
+            [PhysicalSide::Left, PhysicalSide::Right], // horizontal-tb               ltr
+            [PhysicalSide::Top, PhysicalSide::Bottom], // vertical-rl                 ltr
+            [PhysicalSide::Right, PhysicalSide::Left], // horizontal-tb               rtl
+            [PhysicalSide::Bottom, PhysicalSide::Top], // vertical-rl                 rtl
+            [PhysicalSide::Right, PhysicalSide::Left], // (horizontal-bt)  (inverted) ltr
+            [PhysicalSide::Top, PhysicalSide::Bottom], // sideways-lr                 rtl
+            [PhysicalSide::Left, PhysicalSide::Right], // (horizontal-bt)  (inverted) rtl
+            [PhysicalSide::Bottom, PhysicalSide::Top], // sideways-lr                 ltr
+            [PhysicalSide::Left, PhysicalSide::Right], // horizontal-tb    (inverted) rtl
+            [PhysicalSide::Top, PhysicalSide::Bottom], // vertical-rl      (inverted) rtl
+            [PhysicalSide::Right, PhysicalSide::Left], // horizontal-tb    (inverted) ltr
+            [PhysicalSide::Bottom, PhysicalSide::Top], // vertical-rl      (inverted) ltr
+            [PhysicalSide::Left, PhysicalSide::Right], // (horizontal-bt)             ltr
+            [PhysicalSide::Top, PhysicalSide::Bottom], // vertical-lr                 ltr
+            [PhysicalSide::Right, PhysicalSide::Left], // (horizontal-bt)             rtl
+            [PhysicalSide::Bottom, PhysicalSide::Top], // vertical-lr                 rtl
+        ];
+
+        debug_assert!(
+            WritingMode::VERTICAL.bits() == 0x01 &&
+                WritingMode::INLINE_REVERSED.bits() == 0x02 &&
+                WritingMode::VERTICAL_LR.bits() == 0x04 &&
+                WritingMode::LINE_INVERTED.bits() == 0x08
+        );
+        let index = (wm.bits() & 0xF) as usize;
+        INLINE_MAPPING[index][edge]
+    }
+}
+
+#[derive(Clone, Copy, Debug, PartialEq)]
+#[repr(u8)]
+pub enum LogicalCorner {
+    StartStart = 0,
+    StartEnd,
+    EndStart,
+    EndEnd,
+}
+
+impl LogicalCorner {
+    #[inline]
+    pub fn to_physical(self, wm: WritingMode) -> PhysicalCorner {
+        static CORNER_TO_SIDES: [[LogicalSide; 2]; 4] = [
+            [LogicalSide::BlockStart, LogicalSide::InlineStart],
+            [LogicalSide::BlockStart, LogicalSide::InlineEnd],
+            [LogicalSide::BlockEnd, LogicalSide::InlineStart],
+            [LogicalSide::BlockEnd, LogicalSide::InlineEnd],
+        ];
+
+        let [block, inline] = CORNER_TO_SIDES[self as usize];
+        let block = block.to_physical(wm);
+        let inline = inline.to_physical(wm);
+        PhysicalCorner::from_sides(block, inline)
+    }
+}
+
+#[derive(Clone, Copy, Debug, PartialEq)]
+#[repr(u8)]
+pub enum PhysicalAxis {
+    Vertical = 0,
+    Horizontal,
+}
+
+#[derive(Clone, Copy, Debug, PartialEq)]
+#[repr(u8)]
 pub enum PhysicalSide {
-    Top,
+    Top = 0,
     Right,
     Bottom,
     Left,
 }
 
+impl PhysicalSide {
+    fn orthogonal_to(self, other: Self) -> bool {
+        matches!(self, Self::Top | Self::Bottom) != matches!(other, Self::Top | Self::Bottom)
+    }
+}
+
 #[derive(Clone, Copy, Debug, PartialEq)]
+#[repr(u8)]
 pub enum PhysicalCorner {
-    TopLeft,
+    TopLeft = 0,
     TopRight,
     BottomRight,
     BottomLeft,
 }
+
+impl PhysicalCorner {
+    fn from_sides(a: PhysicalSide, b: PhysicalSide) -> Self {
+        debug_assert!(a.orthogonal_to(b), "Sides should be orthogonal");
+        // Only some of these are possible, since we expect only orthogonal values. If the two
+        // sides were to be parallel, we fall back to returning TopLeft.
+        const IMPOSSIBLE: PhysicalCorner = PhysicalCorner::TopLeft;
+        static SIDES_TO_CORNER: [[PhysicalCorner; 4]; 4] = [
+            [
+                IMPOSSIBLE,
+                PhysicalCorner::TopRight,
+                IMPOSSIBLE,
+                PhysicalCorner::TopLeft,
+            ],
+            [
+                PhysicalCorner::TopRight,
+                IMPOSSIBLE,
+                PhysicalCorner::BottomRight,
+                IMPOSSIBLE,
+            ],
+            [
+                IMPOSSIBLE,
+                PhysicalCorner::BottomRight,
+                IMPOSSIBLE,
+                PhysicalCorner::BottomLeft,
+            ],
+            [
+                PhysicalCorner::TopLeft,
+                IMPOSSIBLE,
+                PhysicalCorner::BottomLeft,
+                IMPOSSIBLE,
+            ],
+        ];
+        SIDES_TO_CORNER[a as usize][b as usize]
+    }
+}
