@@ -4,17 +4,15 @@
 
 <%namespace name="helpers" file="/helpers.mako.rs" />
 
-<% data.new_style_struct("List", inherited=True) %>
-
 ${helpers.single_keyword(
     "list-style-position",
     "outside inside",
-    engines="gecko servo-2013 servo-2020",
-    servo_2020_pref="layout.2020.unimplemented",
+    engines="gecko servo",
     gecko_enum_prefix="StyleListStylePosition",
-    animation_value_type="discrete",
+    animation_type="discrete",
     spec="https://drafts.csswg.org/css-lists/#propdef-list-style-position",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 // TODO(pcwalton): Implement the full set of counter styles per CSS-COUNTER-STYLES [1] 6.1:
@@ -23,20 +21,20 @@ ${helpers.single_keyword(
 //     upper-roman
 //
 // [1]: http://dev.w3.org/csswg/css-counter-styles/
-% if engine in ["servo-2013", "servo-2020"]:
+% if engine == "servo":
     ${helpers.single_keyword(
         "list-style-type",
-        "disc none circle square disclosure-open disclosure-closed",
-        extra_servo_2013_values="""
+        """disc none circle square disclosure-open disclosure-closed
             decimal lower-alpha upper-alpha arabic-indic bengali cambodian cjk-decimal devanagari
             gujarati gurmukhi kannada khmer lao malayalam mongolian myanmar oriya persian telugu
             thai tibetan cjk-earthly-branch cjk-heavenly-stem lower-greek hiragana hiragana-iroha
             katakana katakana-iroha
         """,
-        engines="servo-2013 servo-2020",
-        animation_value_type="discrete",
+        engines="servo",
+        animation_type="discrete",
         spec="https://drafts.csswg.org/css-lists/#propdef-list-style-type",
         servo_restyle_damage="rebuild_and_reflow",
+        affects="layout",
     )}
 % endif
 % if engine == "gecko":
@@ -46,30 +44,33 @@ ${helpers.single_keyword(
         "computed::ListStyleType::disc()",
         engines="gecko",
         initial_specified_value="specified::ListStyleType::disc()",
-        animation_value_type="discrete",
-        boxed=True,
+        animation_type="discrete",
         spec="https://drafts.csswg.org/css-lists/#propdef-list-style-type",
         servo_restyle_damage="rebuild_and_reflow",
+        affects="layout",
     )}
 % endif
 
 ${helpers.predefined_type(
     "list-style-image",
     "Image",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_value="computed::Image::None",
     initial_specified_value="specified::Image::None",
-    animation_value_type="discrete",
+    animation_type="discrete",
     spec="https://drafts.csswg.org/css-lists/#propdef-list-style-image",
+    boxed=engine == "servo",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "quotes",
     "Quotes",
     "computed::Quotes::get_initial_value()",
-    engines="gecko servo-2013",
-    animation_value_type="discrete",
+    engines="gecko servo",
+    animation_type="discrete",
     spec="https://drafts.csswg.org/css-content/#propdef-quotes",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
