@@ -10,7 +10,8 @@ use crate::properties::{ComputedValues, StyleBuilder};
 use crate::rule_tree::StrongRuleNode;
 use crate::selector_parser::PseudoElement;
 use crate::shared_lock::StylesheetGuards;
-use crate::values::computed::NonNegativeLength;
+use crate::values::computed::{NonNegativeLength, Zoom};
+use crate::values::specified::color::ColorSchemeFlags;
 use fxhash::FxHashMap;
 use servo_arc::Arc;
 use smallvec::SmallVec;
@@ -20,7 +21,9 @@ use smallvec::SmallVec;
 pub struct RuleCacheConditions {
     uncacheable: bool,
     font_size: Option<NonNegativeLength>,
+    line_height: Option<NonNegativeLength>,
     writing_mode: Option<WritingMode>,
+    color_scheme: Option<ColorSchemeFlags>,
 }
 
 impl RuleCacheConditions {
@@ -30,6 +33,18 @@ impl RuleCacheConditions {
         self.font_size = Some(font_size);
     }
 
+    /// Sets the style as depending in the line-height value.
+    pub fn set_line_height_dependency(&mut self, line_height: NonNegativeLength) {
+        debug_assert!(self.line_height.map_or(true, |l| l == line_height));
+        self.line_height = Some(line_height);
+    }
+
+    /// Sets the style as depending in the color-scheme property value.
+    pub fn set_color_scheme_dependency(&mut self, color_scheme: ColorSchemeFlags) {
+        debug_assert!(self.color_scheme.map_or(true, |cs| cs == color_scheme));
+        self.color_scheme = Some(color_scheme);
+    }
+
     /// Sets the style as uncacheable.
     pub fn set_uncacheable(&mut self) {
         self.uncacheable = true;
@@ -45,10 +60,21 @@ impl RuleCacheConditions {
     fn cacheable(&self) -> bool {
         !self.uncacheable
     }
+}
 
+#[derive(Debug)]
+struct CachedConditions {
+    font_size: Option<NonNegativeLength>,
+    line_height: Option<NonNegativeLength>,
+    color_scheme: Option<ColorSchemeFlags>,
+    writing_mode: Option<WritingMode>,
+    zoom: Zoom,
+}
+
+impl CachedConditions {
     /// Returns whether `style` matches the conditions.
     fn matches(&self, style: &StyleBuilder) -> bool {
-        if self.uncacheable {
+        if style.effective_zoom != self.zoom {
             return false;
         }
 
@@ -58,6 +84,22 @@ impl RuleCacheConditions {
             }
         }
 
+        if let Some(lh) = self.line_height {
+            let new_line_height =
+                style
+                    .device
+                    .calc_line_height(&style.get_font(), style.writing_mode, None);
+            if new_line_height != lh {
+                return false;
+            }
+        }
+
+        if let Some(cs) = self.color_scheme {
+            if style.get_inherited_ui().color_scheme_bits() != cs {
+                return false;
+            }
+        }
+
         if let Some(wm) = self.writing_mode {
             if style.writing_mode != wm {
                 return false;
@@ -71,7 +113,7 @@ impl RuleCacheConditions {
 /// A TLS cache from rules matched to computed values.
 pub struct RuleCache {
     // FIXME(emilio): Consider using LRUCache or something like that?
-    map: FxHashMap<StrongRuleNode, SmallVec<[(RuleCacheConditions, Arc<ComputedValues>); 1]>>,
+    map: FxHashMap<StrongRuleNode, SmallVec<[(CachedConditions, Arc<ComputedValues>); 1]>>,
 }
 
 impl RuleCache {
@@ -85,30 +127,33 @@ impl RuleCache {
     /// Walk the rule tree and return a rule node for using as the key
     /// for rule cache.
     ///
-    /// It currently skips a rule node when it is neither from a style
-    /// rule, nor containing any declaration of reset property. We don't
-    /// skip style rule so that we don't need to walk a long way in the
-    /// worst case. Skipping declarations rule nodes should be enough
-    /// to address common cases that rule cache would fail to share
-    /// when using the rule node directly, like preshint, style attrs,
-    /// and animations.
+    /// It currently skips animation / style attribute / preshint rules when they don't contain any
+    /// declaration of a reset property. We don't skip other levels because walking the whole
+    /// parent chain can be expensive.
+    ///
+    /// TODO(emilio): Measure this, this was not super-well measured for performance (this was done
+    /// for memory in bug 1427681)... Walking the rule tree might be worth it if we hit the cache
+    /// enough?
     fn get_rule_node_for_cache<'r>(
         guards: &StylesheetGuards,
         mut rule_node: Option<&'r StrongRuleNode>,
     ) -> Option<&'r StrongRuleNode> {
+        use crate::rule_tree::CascadeLevel;
         while let Some(node) = rule_node {
-            match node.style_source() {
-                Some(s) => match s.as_declarations() {
-                    Some(decls) => {
-                        let cascade_level = node.cascade_level();
-                        let decls = decls.read_with(cascade_level.guard(guards));
-                        if decls.contains_any_reset() {
-                            break;
-                        }
-                    },
-                    None => break,
-                },
-                None => {},
+            let priority = node.cascade_priority();
+            let cascade_level = priority.cascade_level();
+            let should_try_to_skip =
+                cascade_level.is_animation() ||
+                matches!(cascade_level, CascadeLevel::PresHints) ||
+                priority.layer_order().is_style_attribute_layer();
+            if !should_try_to_skip {
+                break;
+            }
+            if let Some(source) = node.style_source() {
+                let decls = source.get().read_with(cascade_level.guard(guards));
+                if decls.contains_any_reset() {
+                    break;
+                }
             }
             rule_node = node.parent();
         }
@@ -177,11 +222,17 @@ impl RuleCache {
             "Inserting cached reset style with conditions {:?}",
             conditions
         );
+        let cached_conditions = CachedConditions {
+            writing_mode: conditions.writing_mode,
+            font_size: conditions.font_size,
+            line_height: conditions.line_height,
+            color_scheme: conditions.color_scheme,
+            zoom: style.effective_zoom,
+        };
         self.map
             .entry(rules)
-            .or_insert_with(SmallVec::new)
-            .push((conditions.clone(), style.clone()));
-
+            .or_default()
+            .push((cached_conditions, style.clone()));
         true
     }
 }
