@@ -4,7 +4,6 @@
 
 //! Style sheets and their CSS rules.
 
-mod cascading_at_rule;
 pub mod container_rule;
 mod counter_style_rule;
 mod document_rule;
@@ -15,25 +14,30 @@ pub mod import_rule;
 pub mod keyframes_rule;
 pub mod layer_rule;
 mod loader;
+mod margin_rule;
 mod media_rule;
 mod namespace_rule;
+mod nested_declarations_rule;
 pub mod origin;
 mod page_rule;
+pub mod position_try_rule;
 mod property_rule;
 mod rule_list;
 mod rule_parser;
 mod rules_iterator;
+pub mod scope_rule;
+mod starting_style_rule;
 mod style_rule;
 mod stylesheet;
 pub mod supports_rule;
-pub mod viewport_rule;
 
 #[cfg(feature = "gecko")]
 use crate::gecko_bindings::sugar::refptr::RefCounted;
 #[cfg(feature = "gecko")]
 use crate::gecko_bindings::{bindings, structs};
-use crate::parser::ParserContext;
-use crate::shared_lock::{DeepCloneParams, DeepCloneWithLock, Locked};
+use crate::parser::{NestingContext, ParserContext};
+use crate::properties::{parse_property_declaration_list, PropertyDeclarationBlock};
+use crate::shared_lock::{DeepCloneWithLock, Locked};
 use crate::shared_lock::{SharedRwLock, SharedRwLockReadGuard, ToCssWithGuard};
 use crate::str::CssStringWriter;
 use cssparser::{parse_one_rule, Parser, ParserInput};
@@ -41,12 +45,11 @@ use cssparser::{parse_one_rule, Parser, ParserInput};
 use malloc_size_of::{MallocSizeOfOps, MallocUnconditionalShallowSizeOf};
 use servo_arc::Arc;
 use std::borrow::Cow;
-use std::fmt;
+use std::fmt::{self, Write};
 #[cfg(feature = "gecko")]
 use std::mem::{self, ManuallyDrop};
 use style_traits::ParsingMode;
-#[cfg(feature = "gecko")]
-use to_shmem::{self, SharedMemoryBuilder, ToShmem};
+use to_shmem::{SharedMemoryBuilder, ToShmem};
 
 pub use self::container_rule::ContainerRule;
 pub use self::counter_style_rule::CounterStyleRule;
@@ -58,10 +61,13 @@ pub use self::import_rule::ImportRule;
 pub use self::keyframes_rule::KeyframesRule;
 pub use self::layer_rule::{LayerBlockRule, LayerStatementRule};
 pub use self::loader::StylesheetLoader;
+pub use self::margin_rule::{MarginRule, MarginRuleType};
 pub use self::media_rule::MediaRule;
 pub use self::namespace_rule::NamespaceRule;
+pub use self::nested_declarations_rule::NestedDeclarationsRule;
 pub use self::origin::{Origin, OriginSet, OriginSetIterator, PerOrigin, PerOriginIter};
-pub use self::page_rule::{PageRule, PageSelector, PageSelectors};
+pub use self::page_rule::{PagePseudoClassFlags, PageRule, PageSelector, PageSelectors};
+pub use self::position_try_rule::PositionTryRule;
 pub use self::property_rule::PropertyRule;
 pub use self::rule_list::{CssRules, CssRulesHelpers};
 pub use self::rule_parser::{InsertRuleContext, State, TopLevelRuleParser};
@@ -69,12 +75,13 @@ pub use self::rules_iterator::{AllRules, EffectiveRules};
 pub use self::rules_iterator::{
     EffectiveRulesIterator, NestedRuleIterationCondition, RulesIterator,
 };
+pub use self::scope_rule::ScopeRule;
+pub use self::starting_style_rule::StartingStyleRule;
 pub use self::style_rule::StyleRule;
 pub use self::stylesheet::{AllowImportRules, SanitizationData, SanitizationKind};
 pub use self::stylesheet::{DocumentStyleSheet, Namespaces, Stylesheet};
 pub use self::stylesheet::{StylesheetContents, StylesheetInDocument, UserAgentStylesheets};
 pub use self::supports_rule::SupportsRule;
-pub use self::viewport_rule::ViewportRule;
 
 /// The CORS mode used for a CSS load.
 #[repr(u8)]
@@ -100,13 +107,43 @@ pub enum CorsMode {
 /// We use this packed representation rather than an enum so that
 /// `from_ptr_ref` can work.
 #[cfg(feature = "gecko")]
-#[derive(PartialEq)]
+// Although deriving MallocSizeOf means it always returns 0, that is fine because UrlExtraData
+// objects are reference-counted.
+#[derive(MallocSizeOf, PartialEq)]
 #[repr(C)]
 pub struct UrlExtraData(usize);
 
 /// Extra data that the backend may need to resolve url values.
+#[cfg(feature = "servo")]
+#[derive(Clone, Debug, Eq, MallocSizeOf, PartialEq)]
+pub struct UrlExtraData(#[ignore_malloc_size_of = "Arc"] pub Arc<::url::Url>);
+
+#[cfg(feature = "servo")]
+impl UrlExtraData {
+    /// True if this URL scheme is chrome.
+    pub fn chrome_rules_enabled(&self) -> bool {
+        self.0.scheme() == "chrome"
+    }
+
+    /// Get the interior Url as a string.
+    pub fn as_str(&self) -> &str {
+        self.0.as_str()
+    }
+}
+
+#[cfg(feature = "servo")]
+impl From<::url::Url> for UrlExtraData {
+    fn from(url: ::url::Url) -> Self {
+        Self(Arc::new(url))
+    }
+}
+
 #[cfg(not(feature = "gecko"))]
-pub type UrlExtraData = ::servo_url::ServoUrl;
+impl ToShmem for UrlExtraData {
+    fn to_shmem(&self, _builder: &mut SharedMemoryBuilder) -> to_shmem::Result<Self> {
+        unimplemented!("If servo wants to share stylesheets across processes, ToShmem for Url must be implemented");
+    }
+}
 
 #[cfg(feature = "gecko")]
 impl Clone for UrlExtraData {
@@ -131,7 +168,11 @@ impl Drop for UrlExtraData {
 impl ToShmem for UrlExtraData {
     fn to_shmem(&self, _builder: &mut SharedMemoryBuilder) -> to_shmem::Result<Self> {
         if self.0 & 1 == 0 {
-            let shared_extra_datas = unsafe { &structs::URLExtraData_sShared };
+            let shared_extra_datas = unsafe {
+                std::ptr::addr_of!(structs::URLExtraData_sShared)
+                    .as_ref()
+                    .unwrap()
+            };
             let self_ptr = self.as_ref() as *const _ as *mut _;
             let sheet_id = shared_extra_datas
                 .iter()
@@ -239,31 +280,75 @@ impl fmt::Debug for UrlExtraData {
 #[cfg(feature = "gecko")]
 impl Eq for UrlExtraData {}
 
+/// Serialize a page or style rule, starting with the opening brace.
+///
+/// https://drafts.csswg.org/cssom/#serialize-a-css-rule CSSStyleRule
+///
+/// This is not properly specified for page-rules, but we will apply the
+/// same process.
+fn style_or_page_rule_to_css(
+    rules: Option<&Arc<Locked<CssRules>>>,
+    block: &Locked<PropertyDeclarationBlock>,
+    guard: &SharedRwLockReadGuard,
+    dest: &mut CssStringWriter,
+) -> fmt::Result {
+    // Write the opening brace. The caller needs to serialize up to this point.
+    dest.write_char('{')?;
+
+    // Step 2
+    let declaration_block = block.read_with(guard);
+    let has_declarations = !declaration_block.declarations().is_empty();
+
+    // Step 3
+    if let Some(ref rules) = rules {
+        let rules = rules.read_with(guard);
+        // Step 6 (here because it's more convenient)
+        if !rules.is_empty() {
+            if has_declarations {
+                dest.write_str("\n  ")?;
+                declaration_block.to_css(dest)?;
+            }
+            return rules.to_css_block_without_opening(guard, dest);
+        }
+    }
+
+    // Steps 4 & 5
+    if has_declarations {
+        dest.write_char(' ')?;
+        declaration_block.to_css(dest)?;
+    }
+    dest.write_str(" }")
+}
+
 /// A CSS rule.
 ///
 /// TODO(emilio): Lots of spec links should be around.
 #[derive(Clone, Debug, ToShmem)]
 #[allow(missing_docs)]
 pub enum CssRule {
+    Style(Arc<Locked<StyleRule>>),
     // No Charset here, CSSCharsetRule has been removed from CSSOM
     // https://drafts.csswg.org/cssom/#changes-from-5-december-2013
     Namespace(Arc<NamespaceRule>),
     Import(Arc<Locked<ImportRule>>),
-    Style(Arc<Locked<StyleRule>>),
     Media(Arc<MediaRule>),
     Container(Arc<ContainerRule>),
     FontFace(Arc<Locked<FontFaceRule>>),
     FontFeatureValues(Arc<FontFeatureValuesRule>),
     FontPaletteValues(Arc<FontPaletteValuesRule>),
     CounterStyle(Arc<Locked<CounterStyleRule>>),
-    Viewport(Arc<ViewportRule>),
     Keyframes(Arc<Locked<KeyframesRule>>),
+    Margin(Arc<MarginRule>),
     Supports(Arc<SupportsRule>),
     Page(Arc<Locked<PageRule>>),
     Property(Arc<PropertyRule>),
     Document(Arc<DocumentRule>),
     LayerBlock(Arc<LayerBlockRule>),
     LayerStatement(Arc<LayerStatementRule>),
+    Scope(Arc<ScopeRule>),
+    StartingStyle(Arc<StartingStyleRule>),
+    PositionTry(Arc<Locked<PositionTryRule>>),
+    NestedDeclarations(Arc<Locked<NestedDeclarationsRule>>),
 }
 
 impl CssRule {
@@ -292,8 +377,10 @@ impl CssRule {
             CssRule::FontFeatureValues(_) => 0,
             CssRule::FontPaletteValues(_) => 0,
             CssRule::CounterStyle(_) => 0,
-            CssRule::Viewport(_) => 0,
             CssRule::Keyframes(_) => 0,
+            CssRule::Margin(ref arc) => {
+                arc.unconditional_shallow_size_of(ops) + arc.size_of(guard, ops)
+            },
             CssRule::Supports(ref arc) => {
                 arc.unconditional_shallow_size_of(ops) + arc.size_of(guard, ops)
             },
@@ -306,8 +393,20 @@ impl CssRule {
             CssRule::Document(ref arc) => {
                 arc.unconditional_shallow_size_of(ops) + arc.size_of(guard, ops)
             },
+            CssRule::StartingStyle(ref arc) => {
+                arc.unconditional_shallow_size_of(ops) + arc.size_of(guard, ops)
+            },
             // TODO(emilio): Add memory reporting for these rules.
             CssRule::LayerBlock(_) | CssRule::LayerStatement(_) => 0,
+            CssRule::Scope(ref rule) => {
+                rule.unconditional_shallow_size_of(ops) + rule.size_of(guard, ops)
+            },
+            CssRule::PositionTry(ref lock) => {
+                lock.unconditional_shallow_size_of(ops) + lock.read_with(guard).size_of(guard, ops)
+            },
+            CssRule::NestedDeclarations(ref lock) => {
+                lock.unconditional_shallow_size_of(ops) + lock.read_with(guard).size_of(guard, ops)
+            },
         }
     }
 }
@@ -328,7 +427,7 @@ pub enum CssRuleType {
     Keyframes = 7,
     Keyframe = 8,
     // https://drafts.csswg.org/cssom/#the-cssrule-interface
-    // Margin = 9, // Not implemented yet.
+    Margin = 9,
     Namespace = 10,
     // https://drafts.csswg.org/css-counter-styles-3/#extentions-to-cssrule-interface
     CounterStyle = 11,
@@ -338,8 +437,6 @@ pub enum CssRuleType {
     Document = 13,
     // https://drafts.csswg.org/css-fonts/#om-fontfeaturevalues
     FontFeatureValues = 14,
-    // https://drafts.csswg.org/css-device-adapt/#css-rule-interface
-    Viewport = 15,
     // After viewport, all rules should return 0 from the API, but we still need
     // a constant somewhere.
     LayerBlock = 16,
@@ -348,6 +445,13 @@ pub enum CssRuleType {
     FontPaletteValues = 19,
     // 20 is an arbitrary number to use for Property.
     Property = 20,
+    Scope = 21,
+    // https://drafts.csswg.org/css-transitions-2/#the-cssstartingstylerule-interface
+    StartingStyle = 22,
+    // https://drafts.csswg.org/css-anchor-position-1/#om-position-try
+    PositionTry = 23,
+    // https://drafts.csswg.org/css-nesting-1/#nested-declarations-rule
+    NestedDeclarations = 24,
 }
 
 impl CssRuleType {
@@ -369,6 +473,9 @@ impl From<CssRuleType> for CssRuleTypes {
 }
 
 impl CssRuleTypes {
+    /// Rules where !important declarations are forbidden.
+    pub const IMPORTANT_FORBIDDEN: Self = Self(CssRuleType::PositionTry.bit() | CssRuleType::Keyframe.bit());
+
     /// Returns whether the rule is in the current set.
     #[inline]
     pub fn contains(self, ty: CssRuleType) -> bool {
@@ -376,15 +483,34 @@ impl CssRuleTypes {
     }
 
     /// Returns all the rules specified in the set.
+    #[inline]
     pub fn bits(self) -> u32 {
         self.0
     }
 
+    /// Creates a raw CssRuleTypes bitfield.
+    #[inline]
+    pub fn from_bits(bits: u32) -> Self {
+        Self(bits)
+    }
+
+    /// Returns whether the rule set is empty.
+    #[inline]
+    pub fn is_empty(self) -> bool {
+        self.0 == 0
+    }
+
     /// Inserts a rule type into the set.
     #[inline]
     pub fn insert(&mut self, ty: CssRuleType) {
         self.0 |= ty.bit()
     }
+
+    /// Returns whether any of the types intersect.
+    #[inline]
+    pub fn intersects(self, other: Self) -> bool {
+        self.0 & other.0 != 0
+    }
 }
 
 #[allow(missing_docs)]
@@ -407,8 +533,8 @@ impl CssRule {
             CssRule::FontPaletteValues(_) => CssRuleType::FontPaletteValues,
             CssRule::CounterStyle(_) => CssRuleType::CounterStyle,
             CssRule::Keyframes(_) => CssRuleType::Keyframes,
+            CssRule::Margin(_) => CssRuleType::Margin,
             CssRule::Namespace(_) => CssRuleType::Namespace,
-            CssRule::Viewport(_) => CssRuleType::Viewport,
             CssRule::Supports(_) => CssRuleType::Supports,
             CssRule::Page(_) => CssRuleType::Page,
             CssRule::Property(_) => CssRuleType::Property,
@@ -416,26 +542,27 @@ impl CssRule {
             CssRule::LayerBlock(_) => CssRuleType::LayerBlock,
             CssRule::LayerStatement(_) => CssRuleType::LayerStatement,
             CssRule::Container(_) => CssRuleType::Container,
+            CssRule::Scope(_) => CssRuleType::Scope,
+            CssRule::StartingStyle(_) => CssRuleType::StartingStyle,
+            CssRule::PositionTry(_) => CssRuleType::PositionTry,
+            CssRule::NestedDeclarations(_) => CssRuleType::NestedDeclarations,
         }
     }
 
     /// Parse a CSS rule.
     ///
-    /// Returns a parsed CSS rule and the final state of the parser.
-    ///
-    /// Input state is None for a nested rule
+    /// This mostly implements steps 3..7 of https://drafts.csswg.org/cssom/#insert-a-css-rule
     pub fn parse(
         css: &str,
         insert_rule_context: InsertRuleContext,
         parent_stylesheet_contents: &StylesheetContents,
         shared_lock: &SharedRwLock,
-        state: State,
         loader: Option<&dyn StylesheetLoader>,
         allow_import_rules: AllowImportRules,
     ) -> Result<Self, RulesMutateError> {
         let url_data = parent_stylesheet_contents.url_data.read();
         let namespaces = parent_stylesheet_contents.namespaces.read();
-        let context = ParserContext::new(
+        let mut context = ParserContext::new(
             parent_stylesheet_contents.origin,
             &url_data,
             None,
@@ -445,12 +572,26 @@ impl CssRule {
             None,
             None,
         );
+        // Override the nesting context with existing data.
+        context.nesting_context = NestingContext::new(
+            insert_rule_context.containing_rule_types,
+            insert_rule_context.parse_relative_rule_type,
+        );
+
+        let state = if !insert_rule_context.containing_rule_types.is_empty() {
+            State::Body
+        } else if insert_rule_context.index == 0 {
+            State::Start
+        } else {
+            let index = insert_rule_context.index;
+            insert_rule_context.max_rule_state_at_index(index - 1)
+        };
 
         let mut input = ParserInput::new(css);
         let mut input = Parser::new(&mut input);
 
         // nested rules are in the body state
-        let mut rule_parser = TopLevelRuleParser {
+        let mut parser = TopLevelRuleParser {
             context,
             shared_lock: &shared_lock,
             loader,
@@ -459,43 +600,49 @@ impl CssRule {
             insert_rule_context: Some(insert_rule_context),
             allow_import_rules,
             declaration_parser_state: Default::default(),
+            first_declaration_block: Default::default(),
+            wants_first_declaration_block: false,
+            error_reporting_state: Default::default(),
             rules: Default::default(),
         };
 
-        match parse_one_rule(&mut input, &mut rule_parser) {
-            Ok(_) => Ok(rule_parser.rules.pop().unwrap()),
-            Err(_) => Err(rule_parser.dom_error.unwrap_or(RulesMutateError::Syntax)),
+        if input.try_parse(|input| parse_one_rule(input, &mut parser)).is_ok() {
+            return Ok(parser.rules.pop().unwrap());
         }
+
+        let error = parser.dom_error.take().unwrap_or(RulesMutateError::Syntax);
+        // If new rule is a syntax error, and nested is set, perform the following substeps:
+        if matches!(error, RulesMutateError::Syntax) && parser.can_parse_declarations() {
+            let declarations = parse_property_declaration_list(&parser.context, &mut input, &[]);
+            if !declarations.is_empty() {
+                return Ok(CssRule::NestedDeclarations(Arc::new(parser.shared_lock.wrap(NestedDeclarationsRule {
+                    block: Arc::new(parser.shared_lock.wrap(declarations)),
+                    source_location: input.current_source_location(),
+                }))));
+            }
+        }
+        Err(error)
     }
 }
 
 impl DeepCloneWithLock for CssRule {
     /// Deep clones this CssRule.
-    fn deep_clone_with_lock(
-        &self,
-        lock: &SharedRwLock,
-        guard: &SharedRwLockReadGuard,
-        params: &DeepCloneParams,
-    ) -> CssRule {
+    fn deep_clone_with_lock(&self, lock: &SharedRwLock, guard: &SharedRwLockReadGuard) -> CssRule {
         match *self {
             CssRule::Namespace(ref arc) => CssRule::Namespace(arc.clone()),
             CssRule::Import(ref arc) => {
-                let rule = arc
-                    .read_with(guard)
-                    .deep_clone_with_lock(lock, guard, params);
+                let rule = arc.read_with(guard).deep_clone_with_lock(lock, guard);
                 CssRule::Import(Arc::new(lock.wrap(rule)))
             },
             CssRule::Style(ref arc) => {
                 let rule = arc.read_with(guard);
-                CssRule::Style(Arc::new(
-                    lock.wrap(rule.deep_clone_with_lock(lock, guard, params)),
-                ))
+                CssRule::Style(Arc::new(lock.wrap(rule.deep_clone_with_lock(lock, guard))))
             },
             CssRule::Container(ref arc) => {
-                CssRule::Container(Arc::new(arc.deep_clone_with_lock(lock, guard, params)))
+                CssRule::Container(Arc::new(arc.deep_clone_with_lock(lock, guard)))
             },
             CssRule::Media(ref arc) => {
-                CssRule::Media(Arc::new(arc.deep_clone_with_lock(lock, guard, params)))
+                CssRule::Media(Arc::new(arc.deep_clone_with_lock(lock, guard)))
             },
             CssRule::FontFace(ref arc) => {
                 let rule = arc.read_with(guard);
@@ -507,21 +654,19 @@ impl DeepCloneWithLock for CssRule {
                 let rule = arc.read_with(guard);
                 CssRule::CounterStyle(Arc::new(lock.wrap(rule.clone())))
             },
-            CssRule::Viewport(ref arc) => CssRule::Viewport(arc.clone()),
             CssRule::Keyframes(ref arc) => {
                 let rule = arc.read_with(guard);
-                CssRule::Keyframes(Arc::new(
-                    lock.wrap(rule.deep_clone_with_lock(lock, guard, params)),
-                ))
+                CssRule::Keyframes(Arc::new(lock.wrap(rule.deep_clone_with_lock(lock, guard))))
+            },
+            CssRule::Margin(ref arc) => {
+                CssRule::Margin(Arc::new(arc.deep_clone_with_lock(lock, guard)))
             },
             CssRule::Supports(ref arc) => {
-                CssRule::Supports(Arc::new(arc.deep_clone_with_lock(lock, guard, params)))
+                CssRule::Supports(Arc::new(arc.deep_clone_with_lock(lock, guard)))
             },
             CssRule::Page(ref arc) => {
                 let rule = arc.read_with(guard);
-                CssRule::Page(Arc::new(
-                    lock.wrap(rule.deep_clone_with_lock(lock, guard, params)),
-                ))
+                CssRule::Page(Arc::new(lock.wrap(rule.deep_clone_with_lock(lock, guard))))
             },
             CssRule::Property(ref arc) => {
                 // @property rules are immutable, so we don't need any of the `Locked`
@@ -529,11 +674,25 @@ impl DeepCloneWithLock for CssRule {
                 CssRule::Property(arc.clone())
             },
             CssRule::Document(ref arc) => {
-                CssRule::Document(Arc::new(arc.deep_clone_with_lock(lock, guard, params)))
+                CssRule::Document(Arc::new(arc.deep_clone_with_lock(lock, guard)))
             },
             CssRule::LayerStatement(ref arc) => CssRule::LayerStatement(arc.clone()),
             CssRule::LayerBlock(ref arc) => {
-                CssRule::LayerBlock(Arc::new(arc.deep_clone_with_lock(lock, guard, params)))
+                CssRule::LayerBlock(Arc::new(arc.deep_clone_with_lock(lock, guard)))
+            },
+            CssRule::Scope(ref arc) => {
+                CssRule::Scope(Arc::new(arc.deep_clone_with_lock(lock, guard)))
+            },
+            CssRule::StartingStyle(ref arc) => {
+                CssRule::StartingStyle(Arc::new(arc.deep_clone_with_lock(lock, guard)))
+            },
+            CssRule::PositionTry(ref arc) => {
+                let rule = arc.read_with(guard);
+                CssRule::PositionTry(Arc::new(lock.wrap(rule.deep_clone_with_lock(lock, guard))))
+            },
+            CssRule::NestedDeclarations(ref arc) => {
+                let decls = arc.read_with(guard);
+                CssRule::NestedDeclarations(Arc::new(lock.wrap(decls.clone())))
             },
         }
     }
@@ -550,8 +709,8 @@ impl ToCssWithGuard for CssRule {
             CssRule::FontFeatureValues(ref rule) => rule.to_css(guard, dest),
             CssRule::FontPaletteValues(ref rule) => rule.to_css(guard, dest),
             CssRule::CounterStyle(ref lock) => lock.read_with(guard).to_css(guard, dest),
-            CssRule::Viewport(ref rule) => rule.to_css(guard, dest),
             CssRule::Keyframes(ref lock) => lock.read_with(guard).to_css(guard, dest),
+            CssRule::Margin(ref rule) => rule.to_css(guard, dest),
             CssRule::Media(ref rule) => rule.to_css(guard, dest),
             CssRule::Supports(ref rule) => rule.to_css(guard, dest),
             CssRule::Page(ref lock) => lock.read_with(guard).to_css(guard, dest),
@@ -560,6 +719,10 @@ impl ToCssWithGuard for CssRule {
             CssRule::LayerBlock(ref rule) => rule.to_css(guard, dest),
             CssRule::LayerStatement(ref rule) => rule.to_css(guard, dest),
             CssRule::Container(ref rule) => rule.to_css(guard, dest),
+            CssRule::Scope(ref rule) => rule.to_css(guard, dest),
+            CssRule::StartingStyle(ref rule) => rule.to_css(guard, dest),
+            CssRule::PositionTry(ref lock) => lock.read_with(guard).to_css(guard, dest),
+            CssRule::NestedDeclarations(ref lock) => lock.read_with(guard).to_css(guard, dest),
         }
     }
 }
