@@ -8,29 +8,31 @@ use crate::parser::{Parse, ParserContext};
 use crate::values::animated::ToAnimatedValue;
 use crate::values::computed::{
     Angle, Context, Integer, Length, NonNegativeLength, NonNegativeNumber, Number, Percentage,
-    ToComputedValue,
+    ToComputedValue, Zoom,
 };
 use crate::values::generics::font::{
     FeatureTagValue, FontSettings, TaggedFontValue, VariationValue,
 };
 use crate::values::generics::{font as generics, NonNegative};
+use crate::values::resolved::{Context as ResolvedContext, ToResolvedValue};
 use crate::values::specified::font::{
     self as specified, KeywordInfo, MAX_FONT_WEIGHT, MIN_FONT_WEIGHT,
 };
-use crate::values::specified::length::{FontBaseSize, NoCalcLength};
+use crate::values::specified::length::{FontBaseSize, LineHeightBase, NoCalcLength};
 use crate::Atom;
 use cssparser::{serialize_identifier, CssStringWriter, Parser};
-#[cfg(feature = "gecko")]
 use malloc_size_of::{MallocSizeOf, MallocSizeOfOps};
+use num_traits::abs;
+use num_traits::cast::AsPrimitive;
 use std::fmt::{self, Write};
 use style_traits::{CssWriter, ParseError, ToCss};
 
 pub use crate::values::computed::Length as MozScriptMinSize;
 pub use crate::values::specified::font::MozScriptSizeMultiplier;
-pub use crate::values::specified::font::{FontPalette, FontSynthesis};
+pub use crate::values::specified::font::{FontPalette, FontSynthesis, FontSynthesisStyle};
 pub use crate::values::specified::font::{
-    FontVariantAlternates, FontVariantEastAsian, FontVariantLigatures, FontVariantNumeric, XLang,
-    XTextScale,
+    FontVariantAlternates, FontVariantEastAsian, FontVariantLigatures, FontVariantNumeric,
+    QueryFontMetricsFlags, XLang, XTextScale,
 };
 pub use crate::values::specified::Integer as SpecifiedInteger;
 pub use crate::values::specified::Number as SpecifiedNumber;
@@ -59,38 +61,60 @@ pub use crate::values::specified::Number as SpecifiedNumber;
     ComputeSquaredDistance,
     Copy,
     Debug,
+    Eq,
     Hash,
     MallocSizeOf,
     PartialEq,
     PartialOrd,
     ToResolvedValue,
 )]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
 pub struct FixedPoint<T, const FRACTION_BITS: u16> {
-    value: T,
+    /// The actual representation.
+    pub value: T,
 }
 
 impl<T, const FRACTION_BITS: u16> FixedPoint<T, FRACTION_BITS>
 where
-    T: num_traits::cast::AsPrimitive<f32>,
-    f32: num_traits::cast::AsPrimitive<T>,
+    T: AsPrimitive<f32>,
+    f32: AsPrimitive<T>,
+    u16: AsPrimitive<T>,
 {
     const SCALE: u16 = 1 << FRACTION_BITS;
     const INVERSE_SCALE: f32 = 1.0 / Self::SCALE as f32;
 
     /// Returns a fixed-point bit from a floating-point context.
-    fn from_float(v: f32) -> Self {
-        use num_traits::cast::AsPrimitive;
+    pub fn from_float(v: f32) -> Self {
         Self {
             value: (v * Self::SCALE as f32).round().as_(),
         }
     }
 
     /// Returns the floating-point representation.
-    fn to_float(&self) -> f32 {
+    pub fn to_float(&self) -> f32 {
         self.value.as_() * Self::INVERSE_SCALE
     }
 }
 
+// We implement this and mul below only for u16 types, because u32 types might need more care about
+// overflow. But it's not hard to implement in either case.
+impl<const FRACTION_BITS: u16> std::ops::Div for FixedPoint<u16, FRACTION_BITS> {
+    type Output = Self;
+    fn div(self, rhs: Self) -> Self {
+        Self {
+            value: (((self.value as u32) << (FRACTION_BITS as u32)) / (rhs.value as u32)) as u16,
+        }
+    }
+}
+impl<const FRACTION_BITS: u16> std::ops::Mul for FixedPoint<u16, FRACTION_BITS> {
+    type Output = Self;
+    fn mul(self, rhs: Self) -> Self {
+        Self {
+            value: (((self.value as u32) * (rhs.value as u32)) >> (FRACTION_BITS as u32)) as u16,
+        }
+    }
+}
+
 /// font-weight: range 1..1000, fractional values permitted; keywords
 /// 'normal', 'bold' aliased to 400, 700 respectively.
 ///
@@ -127,7 +151,7 @@ impl ToAnimatedValue for FontWeight {
     type AnimatedValue = Number;
 
     #[inline]
-    fn to_animated_value(self) -> Self::AnimatedValue {
+    fn to_animated_value(self, _: &crate::values::animated::Context) -> Self::AnimatedValue {
         self.value()
     }
 
@@ -225,7 +249,6 @@ impl FontWeight {
     PartialEq,
     ToAnimatedZero,
     ToCss,
-    ToResolvedValue,
 )]
 #[cfg_attr(feature = "servo", derive(Serialize, Deserialize))]
 /// The computed value of font-size
@@ -255,6 +278,16 @@ impl FontSize {
         self.used_size.0
     }
 
+    /// Apply zoom to the font-size. This is usually done by ToComputedValue.
+    #[inline]
+    pub fn zoom(&self, zoom: Zoom) -> Self {
+        Self {
+            computed_size: NonNegative(Length::new(zoom.zoom(self.computed_size.0.px()))),
+            used_size: NonNegative(Length::new(zoom.zoom(self.used_size.0.px()))),
+            keyword_info: self.keyword_info,
+        }
+    }
+
     #[inline]
     /// Get default value of font size.
     pub fn medium() -> Self {
@@ -270,8 +303,8 @@ impl ToAnimatedValue for FontSize {
     type AnimatedValue = Length;
 
     #[inline]
-    fn to_animated_value(self) -> Self::AnimatedValue {
-        self.computed_size.0
+    fn to_animated_value(self, context: &crate::values::animated::Context) -> Self::AnimatedValue {
+        self.computed_size.0.to_animated_value(context)
     }
 
     #[inline]
@@ -284,8 +317,27 @@ impl ToAnimatedValue for FontSize {
     }
 }
 
+impl ToResolvedValue for FontSize {
+    type ResolvedValue = NonNegativeLength;
+
+    #[inline]
+    fn to_resolved_value(self, context: &ResolvedContext) -> Self::ResolvedValue {
+        self.computed_size.to_resolved_value(context)
+    }
+
+    #[inline]
+    fn from_resolved_value(resolved: Self::ResolvedValue) -> Self {
+        let computed_size = NonNegativeLength::from_resolved_value(resolved);
+        Self {
+            computed_size,
+            used_size: computed_size,
+            keyword_info: KeywordInfo::none(),
+        }
+    }
+}
+
 #[derive(Clone, Debug, Eq, PartialEq, ToComputedValue, ToResolvedValue)]
-#[cfg_attr(feature = "servo", derive(Hash, MallocSizeOf, Serialize, Deserialize))]
+#[cfg_attr(feature = "servo", derive(Hash, Serialize, Deserialize))]
 /// Specifies a prioritized list of font family names or generic family names.
 #[repr(C)]
 pub struct FontFamily {
@@ -320,6 +372,7 @@ impl FontFamily {
     }
 
     /// Returns the font family for `-moz-bullet-font`.
+    #[cfg(feature = "gecko")]
     pub(crate) fn moz_bullet() -> &'static Self {
         static_font_family!(
             MOZ_BULLET,
@@ -333,6 +386,7 @@ impl FontFamily {
     }
 
     /// Returns a font family for a single system font.
+    #[cfg(feature = "gecko")]
     pub fn for_system_font(name: &str) -> Self {
         Self {
             families: FontFamilyList {
@@ -364,6 +418,7 @@ impl FontFamily {
         generic_font_family!(MONOSPACE, Monospace);
         generic_font_family!(CURSIVE, Cursive);
         generic_font_family!(FANTASY, Fantasy);
+        #[cfg(feature = "gecko")]
         generic_font_family!(MOZ_EMOJI, MozEmoji);
         generic_font_family!(SYSTEM_UI, SystemUi);
 
@@ -377,15 +432,18 @@ impl FontFamily {
             GenericFontFamily::Monospace => &*MONOSPACE,
             GenericFontFamily::Cursive => &*CURSIVE,
             GenericFontFamily::Fantasy => &*FANTASY,
+            #[cfg(feature = "gecko")]
             GenericFontFamily::MozEmoji => &*MOZ_EMOJI,
             GenericFontFamily::SystemUi => &*SYSTEM_UI,
         };
-        debug_assert_eq!(*family.families.iter().next().unwrap(), SingleFontFamily::Generic(generic));
+        debug_assert_eq!(
+            *family.families.iter().next().unwrap(),
+            SingleFontFamily::Generic(generic)
+        );
         family
     }
 }
 
-#[cfg(feature = "gecko")]
 impl MallocSizeOf for FontFamily {
     fn size_of(&self, ops: &mut MallocSizeOfOps) -> usize {
         use malloc_size_of::MallocUnconditionalSizeOf;
@@ -432,6 +490,7 @@ pub struct FamilyName {
     pub syntax: FontFamilyNameSyntax,
 }
 
+#[cfg(feature = "gecko")]
 impl FamilyName {
     fn is_known_icon_font_family(&self) -> bool {
         use crate::gecko_bindings::bindings;
@@ -439,6 +498,13 @@ impl FamilyName {
     }
 }
 
+#[cfg(feature = "servo")]
+impl FamilyName {
+    fn is_known_icon_font_family(&self) -> bool {
+        false
+    }
+}
+
 impl ToCss for FamilyName {
     fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
     where
@@ -559,8 +625,9 @@ impl GenericFontFamily {
     /// the user. See bug 789788 and bug 1730098.
     pub(crate) fn valid_for_user_font_prioritization(self) -> bool {
         match self {
-            Self::None | Self::Fantasy | Self::Cursive | Self::SystemUi | Self::MozEmoji => false,
-
+            Self::None | Self::Fantasy | Self::Cursive | Self::SystemUi => false,
+            #[cfg(feature = "gecko")]
+            Self::MozEmoji => false,
             Self::Serif | Self::SansSerif | Self::Monospace => true,
         }
     }
@@ -628,39 +695,9 @@ impl Parse for SingleFontFamily {
     }
 }
 
-#[cfg(feature = "servo")]
-impl SingleFontFamily {
-    /// Get the corresponding font-family with Atom
-    pub fn from_atom(input: Atom) -> SingleFontFamily {
-        match input {
-            atom!("serif") => return SingleFontFamily::Generic(GenericFontFamily::Serif),
-            atom!("sans-serif") => return SingleFontFamily::Generic(GenericFontFamily::SansSerif),
-            atom!("cursive") => return SingleFontFamily::Generic(GenericFontFamily::Cursive),
-            atom!("fantasy") => return SingleFontFamily::Generic(GenericFontFamily::Fantasy),
-            atom!("monospace") => return SingleFontFamily::Generic(GenericFontFamily::Monospace),
-            _ => {},
-        }
-
-        match_ignore_ascii_case! { &input,
-            "serif" => return SingleFontFamily::Generic(GenericFontFamily::Serif),
-            "sans-serif" => return SingleFontFamily::Generic(GenericFontFamily::SansSerif),
-            "cursive" => return SingleFontFamily::Generic(GenericFontFamily::Cursive),
-            "fantasy" => return SingleFontFamily::Generic(GenericFontFamily::Fantasy),
-            "monospace" => return SingleFontFamily::Generic(GenericFontFamily::Monospace),
-            _ => {}
-        }
-
-        // We don't know if it's quoted or not. So we set it to
-        // quoted by default.
-        SingleFontFamily::FamilyName(FamilyName {
-            name: input,
-            syntax: FontFamilyNameSyntax::Quoted,
-        })
-    }
-}
-
 /// A list of font families.
 #[derive(Clone, Debug, ToComputedValue, ToResolvedValue, ToShmem, PartialEq, Eq)]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize, Hash))]
 #[repr(C)]
 pub struct FontFamilyList {
     /// The actual list of font families specified.
@@ -679,6 +716,7 @@ impl FontFamilyList {
     /// generic instead of the site's specified font may cause substantial breakage.
     /// If no suitable generic is found in the list, insert the default generic ahead
     /// of all the listed families except for known ligature-based icon fonts.
+    #[cfg_attr(feature = "servo", allow(unused))]
     pub(crate) fn prioritize_first_generic_or_prepend(&mut self, generic: GenericFontFamily) {
         let mut index_of_first_generic = None;
         let mut target_index = None;
@@ -727,6 +765,7 @@ impl FontFamilyList {
     }
 
     /// Returns whether we need to prioritize user fonts.
+    #[cfg_attr(feature = "servo", allow(unused))]
     pub(crate) fn needs_user_font_prioritization(&self) -> bool {
         self.iter().next().map_or(true, |f| match f {
             SingleFontFamily::Generic(f) => !f.valid_for_user_font_prioritization(),
@@ -746,7 +785,7 @@ impl FontFamilyList {
     }
 }
 
-/// Preserve the readability of text when font fallback occurs
+/// Preserve the readability of text when font fallback occurs.
 pub type FontSizeAdjust = generics::GenericFontSizeAdjust<NonNegativeNumber>;
 
 impl FontSizeAdjust {
@@ -757,6 +796,90 @@ impl FontSizeAdjust {
     }
 }
 
+impl ToComputedValue for specified::FontSizeAdjust {
+    type ComputedValue = FontSizeAdjust;
+
+    fn to_computed_value(&self, context: &Context) -> Self::ComputedValue {
+        use crate::font_metrics::FontMetricsOrientation;
+
+        let font_metrics = |vertical, flags| {
+            let orient = if vertical {
+                FontMetricsOrientation::MatchContextPreferVertical
+            } else {
+                FontMetricsOrientation::Horizontal
+            };
+            let metrics = context.query_font_metrics(FontBaseSize::CurrentStyle, orient, flags);
+            let font_size = context.style().get_font().clone_font_size().used_size.0;
+            (metrics, font_size)
+        };
+
+        // Macro to resolve a from-font value using the given metric field. If not present,
+        // returns the fallback value, or if that is negative, resolves using ascent instead
+        // of the missing field (this is the fallback for cap-height).
+        macro_rules! resolve {
+            ($basis:ident, $value:expr, $vertical:expr, $field:ident, $fallback:expr, $flags:expr) => {{
+                match $value {
+                    specified::FontSizeAdjustFactor::Number(f) => {
+                        FontSizeAdjust::$basis(f.to_computed_value(context))
+                    },
+                    specified::FontSizeAdjustFactor::FromFont => {
+                        let (metrics, font_size) = font_metrics($vertical, $flags);
+                        let ratio = if let Some(metric) = metrics.$field {
+                            metric / font_size
+                        } else if $fallback >= 0.0 {
+                            $fallback
+                        } else {
+                            metrics.ascent / font_size
+                        };
+                        if ratio.is_nan() {
+                            FontSizeAdjust::$basis(NonNegative(abs($fallback)))
+                        } else {
+                            FontSizeAdjust::$basis(NonNegative(ratio))
+                        }
+                    },
+                }
+            }};
+        }
+
+        match *self {
+            Self::None => FontSizeAdjust::None,
+            Self::ExHeight(val) => {
+                resolve!(ExHeight, val, false, x_height, 0.5, QueryFontMetricsFlags::empty())
+            },
+            Self::CapHeight(val) => {
+                resolve!(CapHeight, val, false, cap_height, -1.0 /* fall back to ascent */, QueryFontMetricsFlags::empty())
+            },
+            Self::ChWidth(val) => {
+                resolve!(ChWidth, val, false, zero_advance_measure, 0.5, QueryFontMetricsFlags::NEEDS_CH)
+            },
+            Self::IcWidth(val) => {
+                resolve!(IcWidth, val, false, ic_width, 1.0, QueryFontMetricsFlags::NEEDS_IC)
+            },
+            Self::IcHeight(val) => {
+                resolve!(IcHeight, val, true, ic_width, 1.0, QueryFontMetricsFlags::NEEDS_IC)
+            },
+        }
+    }
+
+    fn from_computed_value(computed: &Self::ComputedValue) -> Self {
+        macro_rules! case {
+            ($basis:ident, $val:expr) => {
+                Self::$basis(specified::FontSizeAdjustFactor::Number(
+                    ToComputedValue::from_computed_value($val),
+                ))
+            };
+        }
+        match *computed {
+            FontSizeAdjust::None => Self::None,
+            FontSizeAdjust::ExHeight(ref val) => case!(ExHeight, val),
+            FontSizeAdjust::CapHeight(ref val) => case!(CapHeight, val),
+            FontSizeAdjust::ChWidth(ref val) => case!(ChWidth, val),
+            FontSizeAdjust::IcWidth(ref val) => case!(IcWidth, val),
+            FontSizeAdjust::IcHeight(ref val) => case!(IcHeight, val),
+        }
+    }
+}
+
 /// Use FontSettings as computed type of FontFeatureSettings.
 pub type FontFeatureSettings = FontSettings<FeatureTagValue<Integer>>;
 
@@ -807,8 +930,7 @@ where
                 .0
                 .iter()
                 .map(T::from_computed_value)
-                .collect::<Vec<_>>()
-                .into_boxed_slice(),
+                .collect()
         )
     }
 }
@@ -830,6 +952,7 @@ where
     ToShmem,
 )]
 #[repr(C)]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
 #[value_info(other_values = "normal")]
 pub struct FontLanguageOverride(pub u32);
 
@@ -873,22 +996,6 @@ impl ToCss for FontLanguageOverride {
     }
 }
 
-// FIXME(emilio): Make Gecko use the cbindgen'd fontLanguageOverride, then
-// remove this.
-#[cfg(feature = "gecko")]
-impl From<u32> for FontLanguageOverride {
-    fn from(v: u32) -> Self {
-        unsafe { Self::from_u32(v) }
-    }
-}
-
-#[cfg(feature = "gecko")]
-impl From<FontLanguageOverride> for u32 {
-    fn from(v: FontLanguageOverride) -> u32 {
-        v.0
-    }
-}
-
 impl ToComputedValue for specified::MozScriptMinSize {
     type ComputedValue = MozScriptMinSize;
 
@@ -896,8 +1003,11 @@ impl ToComputedValue for specified::MozScriptMinSize {
         // this value is used in the computation of font-size, so
         // we use the parent size
         let base_size = FontBaseSize::InheritedStyle;
+        let line_height_base = LineHeightBase::InheritedStyle;
         match self.0 {
-            NoCalcLength::FontRelative(value) => value.to_computed_value(cx, base_size),
+            NoCalcLength::FontRelative(value) => {
+                value.to_computed_value(cx, base_size, line_height_base)
+            },
             NoCalcLength::ServoCharacterWidth(value) => {
                 value.to_computed_value(base_size.resolve(cx).computed_size())
             },
@@ -948,8 +1058,8 @@ impl ToComputedValue for specified::MathDepth {
 
 /// - Use a signed 8.8 fixed-point value (representable range -128.0..128)
 ///
-/// Values of <angle> below -90 or above 90 not permitted, so we use out of
-/// range values to represent normal | oblique
+/// Values of <angle> below -90 or above 90 are not permitted, so we use an out
+/// of range value to represent `italic`.
 pub const FONT_STYLE_FRACTION_BITS: u16 = 8;
 
 /// This is an alias which is useful mostly as a cbindgen / C++ inference
@@ -958,10 +1068,9 @@ pub type FontStyleFixedPoint = FixedPoint<i16, FONT_STYLE_FRACTION_BITS>;
 
 /// The computed value of `font-style`.
 ///
-/// - Define out of range values min value (-128.0) as meaning 'normal'
-/// - Define max value (127.99609375) as 'italic'
-/// - Other values represent 'oblique <angle>'
-/// - Note that 'oblique 0deg' is distinct from 'normal' (should it be?)
+/// - Define angle of zero degrees as `normal`
+/// - Define out-of-range value 100 degrees as `italic`
+/// - Other values represent `oblique <angle>`
 ///
 /// cbindgen:derive-lt
 /// cbindgen:derive-lte
@@ -972,23 +1081,26 @@ pub type FontStyleFixedPoint = FixedPoint<i16, FONT_STYLE_FRACTION_BITS>;
     ComputeSquaredDistance,
     Copy,
     Debug,
+    Eq,
     Hash,
     MallocSizeOf,
     PartialEq,
     PartialOrd,
     ToResolvedValue,
 )]
+#[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
 #[repr(C)]
 pub struct FontStyle(FontStyleFixedPoint);
 
 impl FontStyle {
-    /// The normal keyword.
+    /// The `normal` keyword, equal to `oblique` with angle zero.
     pub const NORMAL: FontStyle = FontStyle(FontStyleFixedPoint {
-        value: 100 << FONT_STYLE_FRACTION_BITS,
+        value: 0 << FONT_STYLE_FRACTION_BITS,
     });
+
     /// The italic keyword.
     pub const ITALIC: FontStyle = FontStyle(FontStyleFixedPoint {
-        value: 101 << FONT_STYLE_FRACTION_BITS,
+        value: 100 << FONT_STYLE_FRACTION_BITS,
     });
 
     /// The default angle for `font-style: oblique`.
@@ -1017,7 +1129,6 @@ impl FontStyle {
 
     /// Returns the oblique angle for this style.
     pub fn oblique_degrees(&self) -> f32 {
-        debug_assert_ne!(*self, Self::NORMAL);
         debug_assert_ne!(*self, Self::ITALIC);
         self.0.to_float()
     }
@@ -1034,12 +1145,12 @@ impl ToCss for FontStyle {
         if *self == Self::ITALIC {
             return dest.write_str("italic");
         }
-        if *self == Self::OBLIQUE {
-            return dest.write_str("oblique");
+        dest.write_str("oblique")?;
+        if *self != Self::OBLIQUE {
+            // It's not the default oblique amount, so append the angle in degrees.
+            dest.write_char(' ')?;
+            Angle::from_degrees(self.oblique_degrees()).to_css(dest)?;
         }
-        dest.write_str("oblique ")?;
-        let angle = Angle::from_degrees(self.oblique_degrees());
-        angle.to_css(dest)?;
         Ok(())
     }
 }
@@ -1048,13 +1159,7 @@ impl ToAnimatedValue for FontStyle {
     type AnimatedValue = generics::FontStyle<Angle>;
 
     #[inline]
-    fn to_animated_value(self) -> Self::AnimatedValue {
-        if self == Self::NORMAL {
-            // This allows us to animate between normal and oblique values. Per spec,
-            // https://drafts.csswg.org/css-fonts-4/#font-style-prop:
-            //   Animation type: by computed value type; 'normal' animates as 'oblique 0deg'
-            return generics::FontStyle::Oblique(Angle::from_degrees(0.0));
-        }
+    fn to_animated_value(self, _: &crate::values::animated::Context) -> Self::AnimatedValue {
         if self == Self::ITALIC {
             return generics::FontStyle::Italic;
         }
@@ -1064,16 +1169,8 @@ impl ToAnimatedValue for FontStyle {
     #[inline]
     fn from_animated_value(animated: Self::AnimatedValue) -> Self {
         match animated {
-            generics::FontStyle::Normal => Self::NORMAL,
             generics::FontStyle::Italic => Self::ITALIC,
-            generics::FontStyle::Oblique(ref angle) => {
-                if angle.degrees() == 0.0 {
-                    // Reverse the conversion done in to_animated_value()
-                    Self::NORMAL
-                } else {
-                    Self::oblique(angle.degrees())
-                }
-            },
+            generics::FontStyle::Oblique(ref angle) => Self::oblique(angle.degrees()),
         }
     }
 }
@@ -1101,6 +1198,7 @@ pub type FontStretchFixedPoint = FixedPoint<u16, FONT_STRETCH_FRACTION_BITS>;
 #[derive(
     Clone, ComputeSquaredDistance, Copy, Debug, MallocSizeOf, PartialEq, PartialOrd, ToResolvedValue,
 )]
+#[cfg_attr(feature = "servo", derive(Deserialize, Hash, Serialize))]
 #[repr(C)]
 pub struct FontStretch(pub FontStretchFixedPoint);
 
@@ -1228,7 +1326,7 @@ impl ToAnimatedValue for FontStretch {
     type AnimatedValue = Percentage;
 
     #[inline]
-    fn to_animated_value(self) -> Self::AnimatedValue {
+    fn to_animated_value(self, _: &crate::values::animated::Context) -> Self::AnimatedValue {
         self.to_percentage()
     }
 
@@ -1237,3 +1335,45 @@ impl ToAnimatedValue for FontStretch {
         Self::from_percentage(animated.0)
     }
 }
+
+/// A computed value for the `line-height` property.
+pub type LineHeight = generics::GenericLineHeight<NonNegativeNumber, NonNegativeLength>;
+
+impl ToResolvedValue for LineHeight {
+    type ResolvedValue = Self;
+
+    fn to_resolved_value(self, context: &ResolvedContext) -> Self::ResolvedValue {
+        #[cfg(feature = "gecko")]
+        {
+            // Resolve <number> to an absolute <length> based on font size.
+            if matches!(self, Self::Normal | Self::MozBlockHeight) {
+                return self;
+            }
+            let wm = context.style.writing_mode;
+            Self::Length(
+                context
+                    .device
+                    .calc_line_height(
+                        context.style.get_font(),
+                        wm,
+                        Some(context.element_info.element),
+                    )
+                    .to_resolved_value(context),
+            )
+        }
+        #[cfg(feature = "servo")]
+        {
+            if let LineHeight::Number(num) = &self {
+                let size = context.style.get_font().clone_font_size().computed_size();
+                LineHeight::Length(NonNegativeLength::new(size.px() * num.0))
+            } else {
+                self
+            }
+        }
+    }
+
+    #[inline]
+    fn from_resolved_value(value: Self::ResolvedValue) -> Self {
+        value
+    }
+}
