@@ -10,9 +10,10 @@ use crate::values::animated::ToAnimatedZero;
 use crate::values::computed::{Angle, Integer, Length, LengthPercentage, Number, Percentage};
 use crate::values::generics::transform as generic;
 use crate::Zero;
-use euclid::{Transform3D, Vector3D};
+use euclid::default::{Transform3D, Vector3D};
 
 pub use crate::values::generics::transform::TransformStyle;
+pub use crate::values::specified::transform::TransformBox;
 
 /// A single operation in a computed CSS `transform`
 pub type TransformOperation =
@@ -24,6 +25,9 @@ pub type Transform = generic::GenericTransform<TransformOperation>;
 pub type TransformOrigin =
     generic::GenericTransformOrigin<LengthPercentage, LengthPercentage, Length>;
 
+/// The computed value of the `perspective()` transform function.
+pub type PerspectiveFunction = generic::PerspectiveFunction<Length>;
+
 /// A vector to represent the direction vector (rotate axis) for Rotate3D.
 pub type DirectionVector = Vector3D<CSSFloat>;
 
@@ -516,8 +520,8 @@ impl ToAnimatedZero for TransformOperation {
             generic::TransformOperation::Rotate(_) => {
                 Ok(generic::TransformOperation::Rotate(Angle::zero()))
             },
-            generic::TransformOperation::Perspective(ref l) => Ok(
-                generic::TransformOperation::Perspective(l.to_animated_zero()?),
+            generic::TransformOperation::Perspective(_) => Ok(
+                generic::TransformOperation::Perspective(generic::PerspectiveFunction::None),
             ),
             generic::TransformOperation::AccumulateMatrix { .. } |
             generic::TransformOperation::InterpolateMatrix { .. } => {
