@@ -4,53 +4,18 @@
 
 //! Specified types for CSS Easing functions.
 use crate::parser::{Parse, ParserContext};
-use crate::piecewise_linear::{PiecewiseLinearFunction, PiecewiseLinearFunctionBuildParameters};
+use crate::piecewise_linear::{PiecewiseLinearFunction, PiecewiseLinearFunctionBuilder};
 use crate::values::computed::easing::TimingFunction as ComputedTimingFunction;
 use crate::values::computed::{Context, ToComputedValue};
 use crate::values::generics::easing::TimingFunction as GenericTimingFunction;
 use crate::values::generics::easing::{StepPosition, TimingKeyword};
-use crate::values::specified::{Integer, Number, Percentage};
+use crate::values::specified::{AnimationName, Integer, Number, Percentage};
 use cssparser::{Delimiter, Parser, Token};
 use selectors::parser::SelectorParseErrorKind;
-use std::iter::FromIterator;
 use style_traits::{ParseError, StyleParseErrorKind};
 
-/// An entry for linear easing function.
-#[derive(Clone, Copy, Debug, MallocSizeOf, PartialEq, SpecifiedValueInfo, ToCss, ToShmem)]
-pub struct LinearStop {
-    /// Output of the function at the given point.
-    pub output: Number,
-    /// Playback progress at which this output is given.
-    #[css(skip_if = "Option::is_none")]
-    pub input: Option<Percentage>,
-}
-
-/// A list of specified linear stops.
-#[derive(Clone, Default, Debug, MallocSizeOf, PartialEq, SpecifiedValueInfo, ToCss, ToShmem)]
-#[css(comma)]
-pub struct LinearStops {
-    #[css(iterable)]
-    entries: crate::OwnedSlice<LinearStop>,
-}
-
-impl LinearStops {
-    fn new(list: crate::OwnedSlice<LinearStop>) -> Self {
-        LinearStops { entries: list }
-    }
-}
-
 /// A specified timing function.
-pub type TimingFunction = GenericTimingFunction<Integer, Number, LinearStops>;
-
-#[cfg(feature = "gecko")]
-fn linear_timing_function_enabled() -> bool {
-    static_prefs::pref!("layout.css.linear-easing-function.enabled")
-}
-
-#[cfg(feature = "servo")]
-fn linear_timing_function_enabled() -> bool {
-    false
-}
+pub type TimingFunction = GenericTimingFunction<Integer, Number, PiecewiseLinearFunction>;
 
 impl Parse for TimingFunction {
     fn parse<'i, 't>(
@@ -133,13 +98,12 @@ impl TimingFunction {
         context: &ParserContext,
         input: &mut Parser<'i, 't>,
     ) -> Result<Self, ParseError<'i>> {
-        if !linear_timing_function_enabled() {
-            return Err(input.new_custom_error(StyleParseErrorKind::ExperimentalProperty));
-        }
-        let mut result = vec![];
+        let mut builder = PiecewiseLinearFunctionBuilder::default();
+        let mut num_specified_stops = 0;
         // Closely follows `parse_comma_separated`, but can generate multiple entries for one comma-separated entry.
         loop {
             input.parse_until_before(Delimiter::Comma, |i| {
+                let builder = &mut builder;
                 let mut input_start = i.try_parse(|i| Percentage::parse(context, i)).ok();
                 let mut input_end = i.try_parse(|i| Percentage::parse(context, i)).ok();
 
@@ -149,19 +113,14 @@ impl TimingFunction {
                     input_start = i.try_parse(|i| Percentage::parse(context, i)).ok();
                     input_end = i.try_parse(|i| Percentage::parse(context, i)).ok();
                 }
-                result.push(LinearStop {
-                    output,
-                    input: input_start.into(),
-                });
+                builder.push(output.into(), input_start.map(|v| v.get()).into());
+                num_specified_stops += 1;
                 if input_end.is_some() {
                     debug_assert!(
                         input_start.is_some(),
                         "Input end valid but not input start?"
                     );
-                    result.push(LinearStop {
-                        output,
-                        input: input_end.into(),
-                    });
+                    builder.push(output.into(), input_end.map(|v| v.get()).into());
                 }
 
                 Ok(())
@@ -173,22 +132,26 @@ impl TimingFunction {
                 Ok(_) => unreachable!(),
             }
         }
-        if result.len() < 2 {
+        // By spec, specifying only a single stop makes the function invalid, even if that single entry may generate
+        // two entries.
+        if num_specified_stops < 2 {
             return Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError));
         }
 
-        Ok(GenericTimingFunction::LinearFunction(LinearStops::new(
-            crate::OwnedSlice::from(result),
-        )))
+        Ok(GenericTimingFunction::LinearFunction(builder.build()))
     }
-}
 
-impl LinearStop {
-    /// Convert this type to entries that can be used to build PiecewiseLinearFunction.
-    pub fn to_piecewise_linear_build_parameters(
-        x: &LinearStop,
-    ) -> PiecewiseLinearFunctionBuildParameters {
-        (x.output.get(), x.input.map(|x| x.get()))
+    /// Returns true if the name matches any keyword.
+    #[inline]
+    pub fn match_keywords(name: &AnimationName) -> bool {
+        if let Some(name) = name.as_atom() {
+            #[cfg(feature = "gecko")]
+            return name.with_str(|n| TimingKeyword::from_ident(n).is_ok());
+            #[cfg(feature = "servo")]
+            return TimingKeyword::from_ident(name).is_ok();
+
+        }
+        false
     }
 }
 
@@ -211,13 +174,8 @@ impl TimingFunction {
                 }
             },
             GenericTimingFunction::Keyword(keyword) => GenericTimingFunction::Keyword(*keyword),
-            GenericTimingFunction::LinearFunction(steps) => {
-                GenericTimingFunction::LinearFunction(PiecewiseLinearFunction::from_iter(
-                    steps
-                        .entries
-                        .iter()
-                        .map(|e| LinearStop::to_piecewise_linear_build_parameters(e)),
-                ))
+            GenericTimingFunction::LinearFunction(function) => {
+                GenericTimingFunction::LinearFunction(function.clone())
             },
         }
     }
@@ -240,12 +198,7 @@ impl ToComputedValue for TimingFunction {
             },
             ComputedTimingFunction::Keyword(keyword) => GenericTimingFunction::Keyword(*keyword),
             ComputedTimingFunction::LinearFunction(function) => {
-                GenericTimingFunction::LinearFunction(LinearStops {
-                    entries: crate::OwnedSlice::from_iter(function.iter().map(|e| LinearStop {
-                        output: Number::new(e.y),
-                        input: Some(Percentage::new(e.x)).into(),
-                    })),
-                })
+                GenericTimingFunction::LinearFunction(function.clone())
             },
         }
     }
