@@ -24,13 +24,13 @@
 #include "nsCOMPtr.h"
 #include "PLDHashTable.h"
 #include "nsCycleCollectionParticipant.h"
+#include "mozilla/intl/Localization.h"
 
 #include "mozilla/LinkedList.h"
 #include "mozilla/UniquePtr.h"
 
 namespace mozilla {
 namespace dom {
-class BrowserBridgeChild;
 class BrowsingContext;
 }  // namespace dom
 }  // namespace mozilla
@@ -39,12 +39,12 @@ class BrowsingContext;
  * nsDocLoader implementation...
  ****************************************************************************/
 
-#define NS_THIS_DOCLOADER_IMPL_CID                   \
-  { /* b4ec8387-98aa-4c08-93b6-6d23069c06f2 */       \
-    0xb4ec8387, 0x98aa, 0x4c08, {                    \
-      0x93, 0xb6, 0x6d, 0x23, 0x06, 0x9c, 0x06, 0xf2 \
-    }                                                \
-  }
+#define NS_THIS_DOCLOADER_IMPL_CID            \
+  {/* b4ec8387-98aa-4c08-93b6-6d23069c06f2 */ \
+   0xb4ec8387,                                \
+   0x98aa,                                    \
+   0x4c08,                                    \
+   {0x93, 0xb6, 0x6d, 0x23, 0x06, 0x9c, 0x06, 0xf2}}
 
 class nsDocLoader : public nsIDocumentLoader,
                     public nsIRequestObserver,
@@ -55,9 +55,7 @@ class nsDocLoader : public nsIDocumentLoader,
                     public nsIChannelEventSink,
                     public nsISupportsPriority {
  public:
-  using BrowserBridgeChild = mozilla::dom::BrowserBridgeChild;
-
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_THIS_DOCLOADER_IMPL_CID)
+  NS_INLINE_DECL_STATIC_IID(NS_THIS_DOCLOADER_IMPL_CID)
 
   nsDocLoader() : nsDocLoader(false) {}
 
@@ -124,40 +122,13 @@ class nsDocLoader : public nsIDocumentLoader,
 
   void SetDocumentOpenedButNotLoaded() { mDocumentOpenedButNotLoaded = true; }
 
-  bool TreatAsBackgroundLoad();
-
-  void SetFakeOnLoadDispatched() { mHasFakeOnLoadDispatched = true; };
-
-  bool HasFakeOnLoadDispatched() { return mHasFakeOnLoadDispatched; };
-
-  void ResetToFirstLoad() {
-    mHasFakeOnLoadDispatched = false;
-    mIsReadyToHandlePostMessage = false;
-    mTreatAsBackgroundLoad = false;
-  };
-
-  // Inform a parent docloader that a BrowserBridgeChild has been created for
-  // an OOP sub-document.
-  // (This is the OOP counterpart to ChildEnteringOnload below.)
-  void OOPChildLoadStarted(BrowserBridgeChild* aChild) {
-    MOZ_DIAGNOSTIC_ASSERT(!mOOPChildrenLoading.Contains(aChild));
-    mOOPChildrenLoading.AppendElement(aChild);
-  }
-
-  // Inform a parent docloader that the BrowserBridgeChild for one of its
-  // OOP sub-documents is done calling its onload handler.
-  // (This is the OOP counterpart to ChildDoneWithOnload below.)
-  void OOPChildLoadDone(BrowserBridgeChild* aChild) {
-    // aChild will not be in the list if nsDocLoader::Stop() was called, since
-    // that clears mOOPChildrenLoading.  It also dispatches the 'load' event,
-    // so we don't need to call DocLoaderIsEmpty in that case.
-    if (mOOPChildrenLoading.RemoveElement(aChild)) {
-      DocLoaderIsEmpty(true);
-    }
-  }
-
   uint32_t ChildCount() const { return mChildList.Length(); }
 
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void OOPChildrenLoadingIsEmpty() {
+    DocLoaderIsEmpty(true);
+  }
+
  protected:
   explicit nsDocLoader(bool aNotifyAboutBackgroundRequests);
   virtual ~nsDocLoader();
@@ -207,7 +178,7 @@ class nsDocLoader : public nsIDocumentLoader,
                             nsIURI* aUri, uint32_t aFlags);
 
   [[nodiscard]] bool RefreshAttempted(nsIWebProgress* aWebProgress,
-                                      nsIURI* aURI, int32_t aDelay,
+                                      nsIURI* aURI, uint32_t aDelay,
                                       bool aSameURI);
 
   // this function is overridden by the docshell, it is provided so that we
@@ -239,7 +210,9 @@ class nsDocLoader : public nsIDocumentLoader,
 
   // Inform a parent docloader that aChild is done calling its onload
   // handler.
-  void ChildDoneWithOnload(nsIDocumentLoader* aChild) {
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void ChildDoneWithOnload(
+      nsIDocumentLoader* aChild) {
     mChildrenInOnload.RemoveObject(aChild);
     DocLoaderIsEmpty(true);
   }
@@ -251,7 +224,7 @@ class nsDocLoader : public nsIDocumentLoader,
   // aFlushLayout is true.
   // @param aOverrideStatus An optional status to use when notifying listeners
   // of the completed load, instead of using the load group's status.
-  void DocLoaderIsEmpty(
+  MOZ_CAN_RUN_SCRIPT void DocLoaderIsEmpty(
       bool aFlushLayout,
       const mozilla::Maybe<nsresult>& aOverrideStatus = mozilla::Nothing());
 
@@ -349,12 +322,7 @@ class nsDocLoader : public nsIDocumentLoader,
      flushing. */
   bool mIsFlushingLayout;
 
-  bool mTreatAsBackgroundLoad;
-
  private:
-  bool mHasFakeOnLoadDispatched;
-
-  bool mIsReadyToHandlePostMessage;
   /**
    * This flag indicates that the loader is waiting for completion of
    * a document.open-triggered "document load".  This is set when
@@ -364,6 +332,11 @@ class nsDocLoader : public nsIDocumentLoader,
    */
   bool mDocumentOpenedButNotLoaded;
 
+  /**
+   * This flag indicates that the loader is loading javascipt URI.
+   */
+  bool mIsLoadingJavascriptURI = false;
+
   bool mNotifyAboutBackgroundRequests;
 
   static const PLDHashTableOps sRequestInfoHashOps;
@@ -374,10 +347,6 @@ class nsDocLoader : public nsIDocumentLoader,
   // loadgroup) unless this is empty.
   nsCOMArray<nsIDocumentLoader> mChildrenInOnload;
 
-  // The OOP counterpart to mChildrenInOnload.
-  // Not holding strong refs here since we don't actually use the BBCs.
-  nsTArray<const BrowserBridgeChild*> mOOPChildrenLoading;
-
   int64_t GetMaxTotalProgress();
 
   nsresult AddRequestInfo(nsIRequest* aRequest);
@@ -397,11 +366,15 @@ class nsDocLoader : public nsIDocumentLoader,
    * load event yet.
    */
   bool IsBlockingLoadEvent() const {
-    return mIsLoadingDocument || mDocumentOpenedButNotLoaded;
+    return mIsLoadingDocument || mDocumentOpenedButNotLoaded ||
+           mIsLoadingJavascriptURI;
   }
-};
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsDocLoader, NS_THIS_DOCLOADER_IMPL_CID)
+  RefPtr<mozilla::intl::Localization> mL10n;
+  static mozilla::Maybe<nsLiteralCString> StatusCodeToL10nId(nsresult aStatus);
+  nsresult FormatStatusMessage(nsresult aStatus, const nsAString& aHost,
+                               nsAString& aRetVal);
+};
 
 static inline nsISupports* ToSupports(nsDocLoader* aDocLoader) {
   return static_cast<nsIDocumentLoader*>(aDocLoader);
