@@ -5,12 +5,14 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsURILoader.h"
+#include "nsComponentManagerUtils.h"
+#include "nsContentSecurityUtils.h"
 #include "nsIURIContentListener.h"
 #include "nsIContentHandler.h"
 #include "nsILoadGroup.h"
 #include "nsIDocumentLoader.h"
 #include "nsIStreamListener.h"
-#include "nsIURI.h"
+#include "nsIURL.h"
 #include "nsIChannel.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
@@ -41,9 +43,13 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/Result.h"
 #include "mozilla/Unused.h"
+#include "mozilla/StaticPrefs_browser.h"
 #include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/StaticPrefs_general.h"
 #include "nsContentUtils.h"
+#include "imgLoader.h"
 
 mozilla::LazyLogModule nsURILoader::mLog("URILoader");
 
@@ -52,14 +58,6 @@ mozilla::LazyLogModule nsURILoader::mLog("URILoader");
   MOZ_LOG(nsURILoader::mLog, mozilla::LogLevel::Error, args)
 #define LOG_ENABLED() MOZ_LOG_TEST(nsURILoader::mLog, mozilla::LogLevel::Debug)
 
-static uint32_t sConvertDataLimit = 20;
-
-static bool InitPreferences() {
-  mozilla::Preferences::AddUintVarCache(
-      &sConvertDataLimit, "general.document_open_conversion_depth_limit", 20);
-  return true;
-}
-
 NS_IMPL_ADDREF(nsDocumentOpenInfo)
 NS_IMPL_RELEASE(nsDocumentOpenInfo)
 
@@ -75,14 +73,17 @@ nsDocumentOpenInfo::nsDocumentOpenInfo(nsIInterfaceRequestor* aWindowContext,
     : m_originalContext(aWindowContext),
       mFlags(aFlags),
       mURILoader(aURILoader),
-      mDataConversionDepthLimit(sConvertDataLimit) {}
+      mDataConversionDepthLimit(
+          mozilla::StaticPrefs::
+              general_document_open_conversion_depth_limit()) {}
 
 nsDocumentOpenInfo::nsDocumentOpenInfo(uint32_t aFlags,
                                        bool aAllowListenerConversions)
     : m_originalContext(nullptr),
       mFlags(aFlags),
       mURILoader(nullptr),
-      mDataConversionDepthLimit(sConvertDataLimit),
+      mDataConversionDepthLimit(
+          mozilla::StaticPrefs::general_document_open_conversion_depth_limit()),
       mAllowListenerConversions(aAllowListenerConversions) {}
 
 nsDocumentOpenInfo::~nsDocumentOpenInfo() {}
@@ -134,35 +135,26 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest* request) {
       return NS_BINDING_ABORTED;
     }
 
-    static bool sLargeAllocationHeaderEnabled = false;
-    static bool sCachedLargeAllocationPref = false;
-    if (!sCachedLargeAllocationPref) {
-      sCachedLargeAllocationPref = true;
-      mozilla::Preferences::AddBoolVarCache(
-          &sLargeAllocationHeaderEnabled, "dom.largeAllocationHeader.enabled");
-    }
+    if (!mozilla::StaticPrefs::
+            browser_http_blank_page_with_error_response_enabled()) {
+      // Bug 1325876: Show internal error page for HTTP responses with error
+      // codes (4xx, 5xx) and "Content-Length": 0 instead of blank page
+      int64_t contentLength = 0;
+      rv = httpChannel->GetContentLength(&contentLength);
 
-    if (sLargeAllocationHeaderEnabled) {
-      if (StaticPrefs::dom_largeAllocation_testing_allHttpLoads()) {
+      if (NS_SUCCEEDED(rv) && contentLength == 0) {
         nsCOMPtr<nsIURI> uri;
         rv = httpChannel->GetURI(getter_AddRefs(uri));
-        if (NS_SUCCEEDED(rv) && uri) {
-          if ((uri->SchemeIs("http") || uri->SchemeIs("https")) &&
-              nsContentUtils::AttemptLargeAllocationLoad(httpChannel)) {
-            return NS_BINDING_ABORTED;
+        // Bug 1944984: Avoid error page on view-source views of empty pages
+        if (NS_FAILED(rv) || !uri->SchemeIs("view-source")) {
+          if (responseCode >= 500) {
+            return NS_ERROR_NET_ERROR_RESPONSE;
+          }
+          if (responseCode >= 400) {
+            return NS_ERROR_NET_EMPTY_RESPONSE;
           }
         }
       }
-
-      // If we have a Large-Allocation header, let's check if we should perform
-      // a process switch.
-      nsAutoCString largeAllocationHeader;
-      rv = httpChannel->GetResponseHeader(
-          NS_LITERAL_CSTRING("Large-Allocation"), largeAllocationHeader);
-      if (NS_SUCCEEDED(rv) &&
-          nsContentUtils::AttemptLargeAllocationLoad(httpChannel)) {
-        return NS_BINDING_ABORTED;
-      }
     }
   }
 
@@ -187,7 +179,7 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest* request) {
     return NS_OK;
   }
 
-  rv = DispatchContent(request, nullptr);
+  rv = DispatchContent(request);
 
   LOG(("  After dispatch, m_targetStreamListener: 0x%p, rv: 0x%08" PRIX32,
        m_targetStreamListener.get(), static_cast<uint32_t>(rv)));
@@ -229,6 +221,7 @@ nsDocumentOpenInfo::OnDataAvailable(nsIRequest* request, nsIInputStream* inStr,
   // if we have retarged to the end stream listener, then forward the call....
   // otherwise, don't do anything
 
+  mReceivedData = true;
   nsresult rv = NS_OK;
 
   if (m_targetStreamListener)
@@ -237,10 +230,78 @@ nsDocumentOpenInfo::OnDataAvailable(nsIRequest* request, nsIInputStream* inStr,
   return rv;
 }
 
+NS_IMETHODIMP
+nsDocumentOpenInfo::OnDataFinished(nsresult aStatus) {
+  if (!m_targetStreamListener) {
+    return NS_ERROR_FAILURE;
+  }
+  nsCOMPtr<nsIThreadRetargetableStreamListener> retargetableListener =
+      do_QueryInterface(m_targetStreamListener);
+  if (retargetableListener) {
+    return retargetableListener->OnDataFinished(aStatus);
+  }
+
+  return NS_OK;
+}
+
+/* static */
+nsresult nsDocumentOpenInfo::CheckContentLengthDiscrepancy(
+    nsIRequest* request) {
+  // Blank page with content length discrepancy is allowed
+  if (mReceivedData ||
+      mozilla::StaticPrefs::
+          browser_http_blank_page_with_error_response_enabled()) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
+  if (!httpChannel) {
+    return NS_OK;
+  }
+
+  uint64_t decodedBodySize;
+  if (NS_FAILED(httpChannel->GetDecodedBodySize(&decodedBodySize)) ||
+      decodedBodySize != 0) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIURI> uri;
+  if (NS_SUCCEEDED(httpChannel->GetURI(getter_AddRefs(uri))) &&
+      uri->SchemeIs("view-source")) {
+    return NS_OK;
+  }
+
+  uint32_t responseCode = 0;
+  if (NS_SUCCEEDED(httpChannel->GetResponseStatus(&responseCode))) {
+    if (responseCode >= 500) {
+      LOG(
+          ("  Returning NS_ERROR_NET_ERROR_RESPONSE from "
+           "nsDocumentOpenInfo::CheckContentLengthDiscrepancy due to 5xx "
+           "responses with no content"));
+      return NS_ERROR_NET_ERROR_RESPONSE;
+    }
+    if (responseCode >= 400) {
+      LOG(
+          ("  Returning NS_ERROR_NET_EMPTY_RESPONSE from "
+           "nsDocumentOpenInfo::CheckContentLengthDiscrepancy due to 4xx "
+           "responses with no content"));
+      return NS_ERROR_NET_EMPTY_RESPONSE;
+    }
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest* request,
                                                 nsresult aStatus) {
   LOG(("[0x%p] nsDocumentOpenInfo::OnStopRequest", this));
 
+  // Bug 1945855 - Show error page for 4xx/5xx responses with no content,
+  // whether the Content-Length header is missing or lying
+  nsresult rv = CheckContentLengthDiscrepancy(request);
+  if (NS_FAILED(rv)) {
+    aStatus = rv;
+  }
+
   if (m_targetStreamListener) {
     nsCOMPtr<nsIStreamListener> listener(m_targetStreamListener);
 
@@ -259,8 +320,63 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest* request,
   return NS_OK;
 }
 
-nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
-                                             nsISupports* aCtxt) {
+static bool IsContentPDF(nsIChannel* aChannel, const nsACString& aContentType) {
+  bool isPDF = aContentType.LowerCaseEqualsASCII(APPLICATION_PDF);
+  if (!isPDF && (aContentType.LowerCaseEqualsASCII(APPLICATION_OCTET_STREAM) ||
+                 aContentType.IsEmpty())) {
+    nsAutoString flname;
+    aChannel->GetContentDispositionFilename(flname);
+    isPDF = StringEndsWith(flname, u".pdf"_ns);
+    if (!isPDF) {
+      nsCOMPtr<nsIURI> uri;
+      aChannel->GetURI(getter_AddRefs(uri));
+      nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+      if (url) {
+        nsAutoCString ext;
+        url->GetFileExtension(ext);
+        isPDF = ext.EqualsLiteral("pdf");
+      }
+    }
+  }
+
+  return isPDF;
+}
+
+static mozilla::Result<bool, nsresult> ShouldHandleExternally(
+    const nsACString& aMimeType) {
+  // For a PDF, check if the preference is set that forces attachments to be
+  // opened inline. If so, treat it as a non-attachment by clearing
+  // 'forceExternalHandling' again. This allows it open a PDF directly
+  // instead of downloading it first. It may still end up being handled by
+  // a helper app depending anyway on the later checks.
+  nsCOMPtr<nsIMIMEInfo> mimeInfo;
+
+  nsCOMPtr<nsIMIMEService> mimeSvc(do_GetService(NS_MIMESERVICE_CONTRACTID));
+  if (!mimeSvc) {
+    return mozilla::Err(NS_ERROR_FAILURE);
+  }
+
+  mimeSvc->GetFromTypeAndExtension(aMimeType, EmptyCString(),
+                                   getter_AddRefs(mimeInfo));
+
+  if (mimeInfo) {
+    int32_t action = nsIMIMEInfo::saveToDisk;
+    mimeInfo->GetPreferredAction(&action);
+
+    bool alwaysAsk = true;
+    mimeInfo->GetAlwaysAskBeforeHandling(&alwaysAsk);
+    return alwaysAsk || action != nsIMIMEInfo::handleInternally;
+  }
+
+  return false;
+}
+
+static bool IsSandboxed(nsIChannel* aChannel) {
+  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
+  return loadInfo->GetSandboxFlags();
+}
+
+nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
   LOG(("[0x%p] nsDocumentOpenInfo::DispatchContent for type '%s'", this,
        mContentType.get()));
 
@@ -274,7 +390,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
     return NS_ERROR_FAILURE;
   }
 
-  NS_NAMED_LITERAL_CSTRING(anyType, "*/*");
+  constexpr auto anyType = "*/*"_ns;
   if (mContentType.IsEmpty() || mContentType == anyType) {
     rv = aChannel->GetContentType(mContentType);
     if (NS_FAILED(rv)) return rv;
@@ -287,22 +403,62 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
     // Reset to application/octet-stream for now; no one other than the
     // external helper app service should see APPLICATION_GUESS_FROM_EXT.
     mContentType = APPLICATION_OCTET_STREAM;
-    aChannel->SetContentType(NS_LITERAL_CSTRING(APPLICATION_OCTET_STREAM));
+    aChannel->SetContentType(nsLiteralCString(APPLICATION_OCTET_STREAM));
   }
 
   // Check whether the data should be forced to be handled externally.  This
   // could happen because the Content-Disposition header is set so, or, in the
   // future, because the user has specified external handling for the MIME
   // type.
-  bool forceExternalHandling = false;
   uint32_t disposition;
   rv = aChannel->GetContentDisposition(&disposition);
 
-  if (NS_SUCCEEDED(rv) && disposition == nsIChannel::DISPOSITION_ATTACHMENT) {
-    forceExternalHandling = true;
-  }
+  bool forceExternalHandling =
+      NS_SUCCEEDED(rv) && disposition == nsIChannel::DISPOSITION_ATTACHMENT;
 
   LOG(("  forceExternalHandling: %s", forceExternalHandling ? "yes" : "no"));
+  LOG(("  IsSandboxed: %s", IsSandboxed(aChannel) ? "yes" : "no"));
+  LOG(("  IsContentPDF: %s",
+       IsContentPDF(aChannel, mContentType) ? "yes" : "no"));
+
+  // Ignore the Content-Disposition header if we're loading a PDF or Image
+  // subresource within an object/embed element.
+  if (forceExternalHandling && (mFlags & nsIURILoader::IS_OBJECT_EMBED) &&
+      (imgLoader::SupportImageWithMimeType(mContentType) ||
+       IsContentPDF(aChannel, mContentType))) {
+    LOG(("Handling pdf/image MIME internally for object/embed element"));
+    forceExternalHandling = false;
+  }
+
+  bool maybeForceInternalHandling =
+      forceExternalHandling &&
+      mozilla::StaticPrefs::browser_download_open_pdf_attachments_inline();
+
+  // Check if this is a PDF which should be opened internally. We also handle
+  // octet-streams that look like they might be PDFs based on their extension.
+  if (maybeForceInternalHandling && IsContentPDF(aChannel, mContentType)) {
+    // For a PDF, check if the preference is set that forces attachments to be
+    // opened inline. If so, treat it as a non-attachment by clearing
+    // 'forceExternalHandling' again. This allows it open a PDF directly
+    // instead of downloading it first. It may still end up being handled by
+    // a helper app depending anyway on the later checks.
+    auto result = ShouldHandleExternally(nsLiteralCString(APPLICATION_PDF));
+    if (result.isErr()) {
+      return result.unwrapErr();
+    }
+    forceExternalHandling = result.unwrap();
+
+    // If we're not opening the PDF externally we block it if it's sandboxed.
+    if (IsSandboxed(aChannel) && !forceExternalHandling) {
+      LOG(("Blocked sandboxed PDF"));
+      nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+      if (httpChannel) {
+        nsContentSecurityUtils::LogMessageToConsole(
+            httpChannel, "IframeSandboxBlockedDownload");
+      }
+      return NS_ERROR_CONTENT_BLOCKED;
+    }
+  }
 
   if (!forceExternalHandling) {
     //
@@ -339,32 +495,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
       }
 
       //
-      // Third step: Try to find a content listener that has not yet had
-      // the chance to register, as it is contained in a not-yet-loaded
-      // module, but which has registered a contract ID.
-      //
-      nsCOMPtr<nsICategoryManager> catman =
-          do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
-      if (catman) {
-        nsCString contractidString;
-        rv = catman->GetCategoryEntry(NS_CONTENT_LISTENER_CATEGORYMANAGER_ENTRY,
-                                      mContentType, contractidString);
-        if (NS_SUCCEEDED(rv) && !contractidString.IsEmpty()) {
-          LOG(("  Listener contractid for '%s' is '%s'", mContentType.get(),
-               contractidString.get()));
-
-          listener = do_CreateInstance(contractidString.get());
-          LOG(("  Listener from category manager: 0x%p", listener.get()));
-
-          if (listener && TryContentListener(listener, aChannel)) {
-            LOG(("  Listener from category manager likes this type"));
-            return NS_OK;
-          }
-        }
-      }
-
-      //
-      // Fourth step: try to find an nsIContentHandler for our type.
+      // Third step: Try to find an nsIContentHandler for our type.
       //
       nsAutoCString handlerContractID(NS_CONTENT_HANDLER_CONTRACTID_PREFIX);
       handlerContractID += mContentType;
@@ -402,7 +533,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
     }
 
     //
-    // Fifth step:  If no listener prefers this type, see if any stream
+    // Fourth step: If no listener prefers this type, see if any stream
     //              converters exist to transform this content type into
     //              some other.
     //
@@ -421,13 +552,6 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
   NS_ASSERTION(!m_targetStreamListener,
                "If we found a listener, why are we not using it?");
 
-  if (mFlags & nsIURILoader::DONT_RETARGET) {
-    LOG(
-        ("  External handling forced or (listener not interested and no "
-         "stream converter exists), and retargeting disallowed -> aborting"));
-    return NS_ERROR_WONT_HANDLE_CONTENT;
-  }
-
   // Before dispatching to the external helper app service, check for an HTTP
   // error page.  If we got one, we don't want to handle it with a helper app,
   // really.
@@ -436,12 +560,21 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
     bool requestSucceeded;
     rv = httpChannel->GetRequestSucceeded(&requestSucceeded);
     if (NS_FAILED(rv) || !requestSucceeded) {
-      // returning error from OnStartRequest will cancel the channel
-      return NS_ERROR_FILE_NOT_FOUND;
+      LOG(
+          ("  Returning NS_ERROR_NET_ERROR_RESPONSE from "
+           "nsDocumentOpenInfo::DispatchContent due to failed HTTP response"));
+      return NS_ERROR_NET_ERROR_RESPONSE;
     }
   }
 
-  // Sixth step:
+  if (mFlags & nsIURILoader::DONT_RETARGET) {
+    LOG(
+        ("  External handling forced or (listener not interested and no "
+         "stream converter exists), and retargeting disallowed -> aborting"));
+    return NS_ERROR_WONT_HANDLE_CONTENT;
+  }
+
+  // Fifth step:
   //
   // All attempts to dispatch this content have failed.  Just pass it off to
   // the helper app service.
@@ -459,9 +592,9 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request,
     request->SetLoadFlags(loadFlags | nsIChannel::LOAD_RETARGETED_DOCUMENT_URI |
                           nsIChannel::LOAD_TARGETED);
 
-    if (isGuessFromExt) {
+    if (isGuessFromExt || mContentType.IsEmpty()) {
       mContentType = APPLICATION_GUESS_FROM_EXT;
-      aChannel->SetContentType(NS_LITERAL_CSTRING(APPLICATION_GUESS_FROM_EXT));
+      aChannel->SetContentType(nsLiteralCString(APPLICATION_GUESS_FROM_EXT));
     }
 
     rv = TryExternalHelperApp(helperAppService, aChannel);
@@ -551,8 +684,16 @@ nsresult nsDocumentOpenInfo::ConvertData(nsIRequest* request,
 }
 
 nsresult nsDocumentOpenInfo::TryStreamConversion(nsIChannel* aChannel) {
-  NS_NAMED_LITERAL_CSTRING(anyType, "*/*");
-  nsresult rv = ConvertData(aChannel, m_contentListener, mContentType, anyType);
+  constexpr auto anyType = "*/*"_ns;
+
+  // A empty content type should be treated like the unknown content type.
+  nsCString srcContentType(mContentType);
+  if (srcContentType.IsEmpty()) {
+    srcContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
+  }
+
+  nsresult rv =
+      ConvertData(aChannel, m_contentListener, srcContentType, anyType);
   if (NS_FAILED(rv)) {
     m_targetStreamListener = nullptr;
   } else if (m_targetStreamListener) {
@@ -721,19 +862,6 @@ NS_IMETHODIMP nsURILoader::OpenURI(nsIChannel* channel, uint32_t aFlags,
     }
   }
 
-  if (aFlags & nsIURILoader::REDIRECTED_CHANNEL) {
-    // Our channel was redirected from another process, so doesn't need to
-    // be opened again. However, it does need its listener hooked up
-    // correctly.
-    if (nsCOMPtr<nsIChildChannel> childChannel = do_QueryInterface(channel)) {
-      return childChannel->CompleteRedirectSetup(loader);
-    }
-
-    // It's possible for the redirected channel to not implement
-    // nsIChildChannel and be entirely local (like srcdoc). In that case we
-    // can just open the local instance and it will work.
-  }
-
   // This method is not complete. Eventually, we should first go
   // to the content listener and ask them for a protocol handler...
   // if they don't give us one, we need to go to the registry and get
@@ -765,9 +893,6 @@ nsresult nsURILoader::OpenChannel(nsIChannel* channel, uint32_t aFlags,
     LOG(("nsURILoader::OpenChannel for %s", spec.get()));
   }
 
-  static bool once = InitPreferences();
-  mozilla::Unused << once;
-
   // we need to create a DocumentOpenInfo object which will go ahead and open
   // the url and discover the content type....
   RefPtr<nsDocumentOpenInfo> loader =
