@@ -14,6 +14,7 @@
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/SegmentedVector.h"
+#include "mozilla/Variant.h"
 #include "jsapi.h"
 #include "jsfriendapi.h"
 #include "js/TypeDecls.h"
@@ -84,17 +85,16 @@ class JSZoneParticipant : public nsCycleCollectionParticipant {
 
 class IncrementalFinalizeRunnable;
 
+enum WhichJSHolders { AllJSHolders, JSHoldersRequiredForGrayMarking };
+
 // A map from JS holders to tracer objects, where the values are stored in
 // SegmentedVector to speed up iteration.
 class JSHolderMap {
  public:
-  enum WhichHolders { AllHolders, HoldersInCollectingZones };
+  class Iter;
 
   JSHolderMap();
-
-  // Call functor |f| for each holder.
-  template <typename F>
-  void ForEach(F&& f, WhichHolders aWhich = AllHolders);
+  ~JSHolderMap() { MOZ_RELEASE_ASSERT(!mHasIterator); }
 
   bool Has(void* aHolder) const;
   nsScriptObjectTracer* Get(void* aHolder) const;
@@ -124,8 +124,7 @@ class JSHolderMap {
       mozilla::HashMap<JS::Zone*, UniquePtr<EntryVector>,
                        DefaultHasher<JS::Zone*>, InfallibleAllocPolicy>;
 
-  template <typename F>
-  void ForEach(EntryVector& aJSHolders, const F& f, JS::Zone* aZone);
+  class EntryVectorIter;
 
   bool RemoveEntry(EntryVector& aJSHolders, Entry* aEntry);
 
@@ -142,6 +141,180 @@ class JSHolderMap {
   // Currently this will only contain wrapper cache wrappers since these are the
   // only holders to pass a zone parameter through to AddJSHolder.
   EntryVectorMap mPerZoneJSHolders;
+
+  // Iterators can mutate the element vectors by removing stale elements. Allow
+  // at most one to exist at a time.
+  bool mHasIterator = false;
+};
+
+// An iterator over an EntryVector that skips over removed entries and removes
+// them from the map.
+class JSHolderMap::EntryVectorIter {
+ public:
+  EntryVectorIter(JSHolderMap& aMap, EntryVector& aVector)
+      : mHolderMap(aMap), mVector(aVector), mIter(aVector.Iter()) {
+    Settle();
+  }
+
+  const EntryVector& Vector() const { return mVector; }
+
+  bool Done() const { return mIter.Done(); }
+  const Entry& Get() const { return mIter.Get(); }
+  void Next() {
+    mIter.Next();
+    Settle();
+  }
+
+  operator const Entry*() const { return &Get(); }
+  const Entry* operator->() const { return &Get(); }
+
+ private:
+  void Settle();
+  friend class JSHolderMap::Iter;
+
+  JSHolderMap& mHolderMap;
+  EntryVector& mVector;
+  EntryVector::IterImpl mIter;
+};
+
+class JSHolderMap::Iter {
+ public:
+  explicit Iter(JSHolderMap& aMap, WhichJSHolders aWhich = AllJSHolders);
+
+  ~Iter() {
+    MOZ_RELEASE_ASSERT(mHolderMap.mHasIterator);
+    mHolderMap.mHasIterator = false;
+  }
+
+  bool Done() const { return mIter.Done(); }
+  const Entry& Get() const { return mIter.Get(); }
+  void Next() {
+    mIter.Next();
+    Settle();
+  }
+
+  // If the holders have been removed from the map while the iterator is live,
+  // then the iterator may point to a removed entry. Update the iterator to make
+  // sure it points to a valid entry or is done.
+  void UpdateForRemovals();
+
+  operator const Entry*() const { return &Get(); }
+  const Entry* operator->() const { return &Get(); }
+
+  JS::Zone* Zone() const { return mZone; }
+
+ private:
+  void Settle();
+
+  JSHolderMap& mHolderMap;
+  Vector<JS::Zone*, 1, InfallibleAllocPolicy> mZones;
+  JS::Zone* mZone = nullptr;
+  EntryVectorIter mIter;
+};
+
+struct JSHolderListEntry {
+  void* mHolder;
+  JSHolderKey* mKey;
+  nsScriptObjectTracer* mTracer;
+
+  JSHolderListEntry();
+  JSHolderListEntry(void* aHolder, JSHolderKey* aKey,
+                    nsScriptObjectTracer* aTracer);
+};
+
+// A list of JS holders and their tracer objects.
+//
+// This supports the same operations as JSHolderMap but does not use a hash
+// table to look up holders. Instead it requires that entries embed a
+// JSHolderKey, which it uses to store the location of the entry in the list.
+class JSHolderList {
+ public:
+  class Iter;
+
+  JSHolderList();
+  ~JSHolderList() { MOZ_RELEASE_ASSERT(!mHasIterator); }
+
+  bool Has(JSHolderKey* aKey) const;
+  nsScriptObjectTracer* Get(void* aHolder, JSHolderKey* aKey) const;
+  nsScriptObjectTracer* Extract(void* aHolder, JSHolderKey* aKey);
+  void Put(void* aHolder, nsScriptObjectTracer* aTracer, JSHolderKey* aKey);
+
+  size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const;
+
+ private:
+  using Entry = JSHolderListEntry;
+
+  using EntryVector = SegmentedVector<Entry, 256, InfallibleAllocPolicy>;
+
+  class EntryVectorIter;
+
+  bool RemoveEntry(EntryVector& aJSHolders, Entry* aEntry);
+
+  // A vector of holders. These are not associated with a particular zone and
+  // can contain pointers to GC things in more than one zone.
+  EntryVector mJSHolders;
+
+  // Iterators can mutate the element vectors by removing stale elements. Allow
+  // at most one to exist at a time.
+  bool mHasIterator = false;
+};
+
+// An iterator over an EntryVector that skips over removed entries and removes
+// them from the list.
+class JSHolderList::EntryVectorIter {
+ public:
+  EntryVectorIter(JSHolderList& aList, EntryVector& aVector)
+      : mHolderList(aList), mVector(aVector), mIter(aVector.Iter()) {
+    Settle();
+  }
+
+  const EntryVector& Vector() const { return mVector; }
+
+  bool Done() const { return mIter.Done(); }
+  const Entry& Get() const { return mIter.Get(); }
+  void Next() {
+    mIter.Next();
+    Settle();
+  }
+
+  operator const Entry*() const { return &Get(); }
+  const Entry* operator->() const { return &Get(); }
+
+ private:
+  void Settle();
+  friend class JSHolderList::Iter;
+
+  JSHolderList& mHolderList;
+  EntryVector& mVector;
+  EntryVector::IterImpl mIter;
+};
+
+class JSHolderList::Iter {
+ public:
+  explicit Iter(JSHolderList& aList, WhichJSHolders aWhich = AllJSHolders);
+
+  ~Iter() {
+    MOZ_RELEASE_ASSERT(mHolderList.mHasIterator);
+    mHolderList.mHasIterator = false;
+  }
+
+  bool Done() const { return mIter.Done(); }
+  const Entry& Get() const { return mIter.Get(); }
+  void Next() { mIter.Next(); }
+
+  // If the holders have been removed from the list while the iterator is live,
+  // then the iterator may point to a removed entry. Update the iterator to make
+  // sure it points to a valid entry or is done.
+  void UpdateForRemovals();
+
+  operator const Entry*() const { return &Get(); }
+  const Entry* operator->() const { return &Get(); }
+
+  JS::Zone* Zone() const { return nullptr; }
+
+ private:
+  JSHolderList& mHolderList;
+  EntryVectorIter mIter;
 };
 
 class CycleCollectedJSRuntime {
@@ -168,6 +341,9 @@ class CycleCollectedJSRuntime {
 
   CycleCollectedJSContext* GetContext() { return mContext; }
 
+  void TraceNativeBlackRoots(JSTracer* aTracer);
+  virtual void TraceAdditionalNativeBlackRoots(JSTracer* aTracer) {}
+
  private:
   void DescribeGCThing(bool aIsMarked, JS::GCCellPtr aThing,
                        nsCycleCollectionTraversalCallback& aCb) const;
@@ -201,15 +377,24 @@ class CycleCollectedJSRuntime {
 
   void TraverseNativeRoots(nsCycleCollectionNoteRootCallback& aCb);
 
+  template <typename ContainerT>
+  void TraverseJSHolders(ContainerT& aHolders,
+                         nsCycleCollectionNoteRootCallback& aCb);
+
   static void TraceBlackJS(JSTracer* aTracer, void* aData);
-  static void TraceGrayJS(JSTracer* aTracer, void* aData);
+
+  // Trace gray JS roots until budget is exceeded and return whether we
+  // finished.
+  static bool TraceGrayJS(JSTracer* aTracer, JS::SliceBudget& budget,
+                          void* aData);
+
   static void GCCallback(JSContext* aContext, JSGCStatus aStatus,
                          JS::GCReason aReason, void* aData);
   static void GCSliceCallback(JSContext* aContext, JS::GCProgress aProgress,
                               const JS::GCDescription& aDesc);
   static void GCNurseryCollectionCallback(JSContext* aContext,
                                           JS::GCNurseryProgress aProgress,
-                                          JS::GCReason aReason);
+                                          JS::GCReason aReason, void* data);
   static void OutOfMemoryCallback(JSContext* aContext, void* aData);
 
   static bool ContextCallback(JSContext* aCx, unsigned aOperation, void* aData);
@@ -217,16 +402,30 @@ class CycleCollectedJSRuntime {
   static void* BeforeWaitCallback(uint8_t* aMemory);
   static void AfterWaitCallback(void* aCookie);
 
-  virtual void TraceNativeBlackRoots(JSTracer* aTracer){};
-  void TraceNativeGrayRoots(JSTracer* aTracer,
-                            JSHolderMap::WhichHolders aWhich);
+#ifdef NS_BUILD_REFCNT_LOGGING
+  void TraceAllNativeGrayRoots(JSTracer* aTracer);
+#endif
+
+  bool TraceNativeGrayRoots(JSTracer* aTracer, WhichJSHolders aWhich,
+                            JS::SliceBudget& aBudget);
+  template <typename IterT>
+  bool TraceJSHolders(JSTracer* aTracer, IterT& aIter,
+                      JS::SliceBudget& aBudget);
 
  public:
-  void FinalizeDeferredThings(
-      CycleCollectedJSContext::DeferredFinalizeType aType);
+  enum DeferredFinalizeType {
+    // Never finalize immediately, because it would be unsafe.
+    FinalizeLater,
+    // Finalize later if we can, but it is okay to do it immediately.
+    FinalizeIncrementally,
+    // Finalize immediately, for shutdown or testing purposes.
+    FinalizeNow,
+  };
+
+  void FinalizeDeferredThings(DeferredFinalizeType aType);
 
   virtual void PrepareForForgetSkippable() = 0;
-  virtual void BeginCycleCollectionCallback() = 0;
+  virtual void BeginCycleCollectionCallback(mozilla::CCReason aReason) = 0;
   virtual void EndCycleCollectionCallback(CycleCollectorResults& aResults) = 0;
   virtual void DispatchDeferredDeletion(bool aContinuation,
                                         bool aPurge = false) = 0;
@@ -265,6 +464,10 @@ class CycleCollectedJSRuntime {
 
   const char* OOMStateToString(const OOMState aOomState) const;
 
+  // Returns true if OOM was reported and a new successful GC cycle hasn't
+  // occurred since.
+  bool OOMReported();
+
   void SetLargeAllocationFailure(OOMState aNewState);
 
   void AnnotateAndSetOutOfMemory(OOMState* aStatePtr, OOMState aNewState);
@@ -289,20 +492,24 @@ class CycleCollectedJSRuntime {
 
   void RunIdleTimeGCTask() {
     if (HasPendingIdleGCTask()) {
-      JS::RunIdleTimeGCTask(Runtime());
+      JS::MaybeRunNurseryCollection(Runtime(),
+                                    JS::GCReason::EAGER_NURSERY_COLLECTION);
       ClearPendingIdleGCTask();
     }
   }
 
   bool IsIdleGCTaskNeeded() {
     return !HasPendingIdleGCTask() && Runtime() &&
-           JS::IsIdleGCTaskNeeded(Runtime());
+           JS::WantEagerMinorGC(Runtime()) != JS::GCReason::NO_REASON;
   }
 
  public:
   void AddJSHolder(void* aHolder, nsScriptObjectTracer* aTracer,
                    JS::Zone* aZone);
+  void AddJSHolderWithKey(void* aHolder, nsScriptObjectTracer* aTracer,
+                          JSHolderKey* aKey);
   void RemoveJSHolder(void* aHolder);
+  void RemoveJSHolderWithKey(void* aHolder, JSHolderKey* aKey);
 #ifdef DEBUG
   void AssertNoObjectsToTrace(void* aPossibleJSHolder);
 #endif
@@ -315,15 +522,14 @@ class CycleCollectedJSRuntime {
   void FixWeakMappingGrayBits() const;
   void CheckGrayBits() const;
   bool AreGCGrayBitsValid() const;
-  void GarbageCollect(JS::GCReason aReason) const;
+  void GarbageCollect(JS::GCOptions options, JS::GCReason aReason) const;
 
   // This needs to be an nsWrapperCache, not a JSObject, because we need to know
   // when our object gets moved.  But we can't trace it (and hence update our
   // storage), because we do not want to keep it alive.  nsWrapperCache handles
   // this for us via its "object moved" handling.
   void NurseryWrapperAdded(nsWrapperCache* aCache);
-  void NurseryWrapperPreserved(JSObject* aWrapper);
-  void JSObjectsTenured();
+  void JSObjectsTenured(JS::GCContext* aGCContext);
 
   void DeferredFinalize(DeferredFinalizeAppendFunction aAppendFunc,
                         DeferredFinalizeFunction aFunc, void* aThing);
@@ -336,7 +542,7 @@ class CycleCollectedJSRuntime {
     mZonesWaitingForGC.Insert(aZone);
   }
 
-  static void OnZoneDestroyed(JSFreeOp* aFop, JS::Zone* aZone);
+  static void OnZoneDestroyed(JS::GCContext* aGcx, JS::Zone* aZone);
 
   // Prepare any zones for GC that have been passed to AddZoneWaitingForGC()
   // since the last GC or since the last call to PrepareWaitingZonesForGC(),
@@ -367,14 +573,17 @@ class CycleCollectedJSRuntime {
   bool mHasPendingIdleGCTask;
 
   JS::GCSliceCallback mPrevGCSliceCallback;
-  JS::GCNurseryCollectionCallback mPrevGCNurseryCollectionCallback;
 
   mozilla::TimeStamp mLatestNurseryCollectionStart;
 
-  JSHolderMap mJSHolders;
+  JSHolderMap mJSHolderMap;
+  JSHolderList mJSHolderList;
 
-  typedef nsTHashMap<nsFuncPtrHashKey<DeferredFinalizeFunction>, void*>
-      DeferredFinalizerTable;
+  // Holds state for incremental tracing of gray roots between calls to
+  // TraceNativeGrayRoots.
+  Variant<Nothing, JSHolderMap::Iter, JSHolderList::Iter> mTraceState;
+
+  using DeferredFinalizerTable = nsTHashMap<DeferredFinalizeFunction, void*>;
   DeferredFinalizerTable mDeferredFinalizerTable;
 
   RefPtr<IncrementalFinalizeRunnable> mFinalizeRunnable;
@@ -383,16 +592,14 @@ class CycleCollectedJSRuntime {
   OOMState mLargeAllocationFailureState;
 
   static const size_t kSegmentSize = 512;
-  SegmentedVector<nsWrapperCache*, kSegmentSize, InfallibleAllocPolicy>
-      mNurseryObjects;
-  SegmentedVector<JS::PersistentRooted<JSObject*>, kSegmentSize,
-                  InfallibleAllocPolicy>
-      mPreservedNurseryObjects;
+  using NurseryObjectsVector =
+      SegmentedVector<nsWrapperCache*, kSegmentSize, InfallibleAllocPolicy>;
+  NurseryObjectsVector mNurseryObjects;
 
   nsTHashSet<JS::Zone*> mZonesWaitingForGC;
 
   struct EnvironmentPreparer : public js::ScriptEnvironmentPreparer {
-    void invoke(JS::HandleObject global, Closure& closure) override;
+    void invoke(JS::Handle<JSObject*> global, Closure& closure) override;
   };
   EnvironmentPreparer mEnvironmentPreparer;
 
@@ -405,7 +612,8 @@ class CycleCollectedJSRuntime {
   // Built on nightly only to avoid any possible performance impact on release
 
   struct ErrorInterceptor final : public JSErrorInterceptor {
-    virtual void interceptError(JSContext* cx, JS::HandleValue exn) override;
+    virtual void interceptError(JSContext* cx,
+                                JS::Handle<JS::Value> exn) override;
     void Shutdown(JSRuntime* rt);
 
     // Copy of the details of the exception.
@@ -413,7 +621,7 @@ class CycleCollectedJSRuntime {
     // complicated garbage-collection scenarios, e.g. a JSContext being killed
     // while we still hold onto an exception thrown from it.
     struct ErrorDetails {
-      nsString mFilename;
+      nsCString mFilename;
       nsString mMessage;
       nsString mStack;
       JSExnType mType;
