@@ -10,8 +10,8 @@
 /*
  * Having problems?
  *
- * See the User Manual at:
- *   http://www.mozilla.org/projects/xpcom/nsCOMPtr.html
+ * See the documentation at:
+ *   https://firefox-source-docs.mozilla.org/xpcom/refptr.html
  *
  *
  * nsCOMPtr
@@ -21,15 +21,14 @@
  */
 
 #include <type_traits>
-#include <utility>
 
 #include "mozilla/AlreadyAddRefed.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/RefPtr.h"
 #include "nsCycleCollectionNoteChild.h"
-#include "nsDebug.h"  // for |NS_ASSERTION|
-#include "nsISupportsUtils.h"  // for |nsresult|, |NS_ADDREF|, |NS_GET_TEMPLATE_IID| et al
+#include "nsDebug.h"           // for |NS_ASSERTION|
+#include "nsISupportsUtils.h"  // for |nsresult|, |NS_ADDREF|, |NS_GET_IID| et al
 
 /*
  * WARNING: This file defines several macros for internal use only. These
@@ -48,11 +47,8 @@
 #  pragma warning(disable : 4514)
 #endif
 
-#define NSCAP_FEATURE_USE_BASE
-
 #ifdef DEBUG
 #  define NSCAP_FEATURE_TEST_DONTQUERY_CASES
-#  undef NSCAP_FEATURE_USE_BASE
 #endif
 
 #ifdef __GNUC__
@@ -67,10 +63,6 @@
 #  define NS_MAY_ALIAS_PTR(t) t*
 #endif
 
-#if defined(NSCAP_DISABLE_DEBUG_PTR_TYPES)
-#  define NSCAP_FEATURE_USE_BASE
-#endif
-
 /*
  * The following three macros (NSCAP_ADDREF, NSCAP_RELEASE, and
  * NSCAP_LOG_ASSIGNMENT) allow external clients the ability to add logging or
@@ -83,11 +75,15 @@
  */
 
 #ifndef NSCAP_ADDREF
-#  define NSCAP_ADDREF(this, ptr) (ptr)->AddRef()
+#  define NSCAP_ADDREF(this, ptr) \
+    mozilla::RefPtrTraits<        \
+        typename std::remove_reference<decltype(*ptr)>::type>::AddRef(ptr)
 #endif
 
 #ifndef NSCAP_RELEASE
-#  define NSCAP_RELEASE(this, ptr) (ptr)->Release()
+#  define NSCAP_RELEASE(this, ptr) \
+    mozilla::RefPtrTraits<         \
+        typename std::remove_reference<decltype(*ptr)>::type>::Release(ptr)
 #endif
 
 // Clients can define |NSCAP_LOG_ASSIGNMENT| to perform logging.
@@ -158,7 +154,6 @@ class MOZ_STACK_CLASS nsQueryInterfaceISupports {
   nsISupports* MOZ_OWNING_REF mRawPtr;
 };
 
-#ifndef NSCAP_FEATURE_USE_BASE
 template <typename T>
 class MOZ_STACK_CLASS nsQueryInterface final
     : public nsQueryInterfaceISupports {
@@ -170,7 +165,6 @@ class MOZ_STACK_CLASS nsQueryInterface final
     return nsQueryInterfaceISupports::operator()(aIID, aAnswer);
   }
 };
-#endif  // #ifndef NSCAP_FEATURE_USE_BASE
 
 class MOZ_STACK_CLASS nsQueryInterfaceISupportsWithError {
  public:
@@ -184,7 +178,6 @@ class MOZ_STACK_CLASS nsQueryInterfaceISupportsWithError {
   nsresult* mErrorPtr;
 };
 
-#ifndef NSCAP_FEATURE_USE_BASE
 template <typename T>
 class MOZ_STACK_CLASS nsQueryInterfaceWithError final
     : public nsQueryInterfaceISupportsWithError {
@@ -196,7 +189,6 @@ class MOZ_STACK_CLASS nsQueryInterfaceWithError final
     return nsQueryInterfaceISupportsWithError::operator()(aIID, aAnswer);
   }
 };
-#endif  // #ifndef NSCAP_FEATURE_USE_BASE
 
 namespace mozilla {
 // PointedToType<> is needed so that do_QueryInterface() will work with a
@@ -206,20 +198,6 @@ template <class T>
 using PointedToType = std::remove_pointer_t<decltype(&*std::declval<T>())>;
 }  // namespace mozilla
 
-#ifdef NSCAP_FEATURE_USE_BASE
-template <class T>
-inline nsQueryInterfaceISupports do_QueryInterface(T aPtr) {
-  return nsQueryInterfaceISupports(
-      ToSupports(static_cast<mozilla::PointedToType<T>*>(aPtr)));
-}
-
-template <class T>
-inline nsQueryInterfaceISupportsWithError do_QueryInterface(T aPtr,
-                                                            nsresult* aError) {
-  return nsQueryInterfaceISupportsWithError(
-      ToSupports(static_cast<mozilla::PointedToType<T>*>(aPtr)), aError);
-}
-#else
 template <class T>
 inline nsQueryInterface<mozilla::PointedToType<T>> do_QueryInterface(T aPtr) {
   return nsQueryInterface<mozilla::PointedToType<T>>(aPtr);
@@ -231,8 +209,6 @@ inline nsQueryInterfaceWithError<mozilla::PointedToType<T>> do_QueryInterface(
   return nsQueryInterfaceWithError<mozilla::PointedToType<T>>(aRawPtr, aError);
 }
 
-#endif  // ! #ifdef NSCAP_FEATURE_USE_BASE
-
 template <class T>
 inline void do_QueryInterface(already_AddRefed<T>&) {
   // This signature exists solely to _stop_ you from doing the bad thing.
@@ -310,87 +286,18 @@ class MOZ_STACK_CLASS nsQueryReferent final {
   nsresult* mErrorPtr;
 };
 
-/**
- * Factors implementation for all template versions of nsCOMPtr.
- *
- * Here's the way people normally do things like this:
- *
- *   template<class T> class Foo { ... };
- *   template<> class Foo<void*> { ... };
- *   template<class T> class Foo<T*> : private Foo<void*> { ... };
- */
-class nsCOMPtr_base {
- public:
-  explicit nsCOMPtr_base(nsISupports* aRawPtr = nullptr) : mRawPtr(aRawPtr) {}
-
-  NS_CONSTRUCTOR_FASTCALL ~nsCOMPtr_base() {
-    NSCAP_LOG_RELEASE(this, mRawPtr);
-    if (mRawPtr) {
-      NSCAP_RELEASE(this, mRawPtr);
-    }
-  }
-
-  void NS_FASTCALL assign_with_AddRef(nsISupports*);
-  void NS_FASTCALL assign_from_qi(const nsQueryInterfaceISupports,
-                                  const nsIID&);
-  void NS_FASTCALL assign_from_qi_with_error(
-      const nsQueryInterfaceISupportsWithError&, const nsIID&);
-  void NS_FASTCALL assign_from_gs_cid(const nsGetServiceByCID, const nsIID&);
-  void NS_FASTCALL assign_from_gs_cid_with_error(
-      const nsGetServiceByCIDWithError&, const nsIID&);
-  void NS_FASTCALL assign_from_gs_contractid(const nsGetServiceByContractID,
-                                             const nsIID&);
-  void NS_FASTCALL assign_from_gs_contractid_with_error(
-      const nsGetServiceByContractIDWithError&, const nsIID&);
-  void NS_FASTCALL assign_from_query_referent(const nsQueryReferent&,
-                                              const nsIID&);
-  void NS_FASTCALL assign_from_helper(const nsCOMPtr_helper&, const nsIID&);
-  void** NS_FASTCALL begin_assignment();
-
- protected:
-  NS_MAY_ALIAS_PTR(nsISupports) MOZ_OWNING_REF mRawPtr;
-
-  void assign_assuming_AddRef(nsISupports* aNewPtr) {
-    // |AddRef()|ing the new value (before entering this function) before
-    // |Release()|ing the old lets us safely ignore the self-assignment case.
-    // We must, however, be careful only to |Release()| _after_ doing the
-    // assignment, in case the |Release()| leads to our _own_ destruction,
-    // which would, in turn, cause an incorrect second |Release()| of our old
-    // pointer. Thank <waterson@netscape.com> for discovering this.
-    nsISupports* oldPtr = mRawPtr;
-    mRawPtr = aNewPtr;
-    NSCAP_LOG_ASSIGNMENT(this, aNewPtr);
-    NSCAP_LOG_RELEASE(this, oldPtr);
-    if (oldPtr) {
-      NSCAP_RELEASE(this, oldPtr);
-    }
-  }
-};
-
 // template<class T> class nsGetterAddRefs;
 
 // Helper for assert_validity method
 template <class T>
-char (&TestForIID(decltype(&NS_GET_TEMPLATE_IID(T))))[2];
+char (&TestForIID(decltype(&NS_GET_IID(T))))[2];
 template <class T>
 char TestForIID(...);
 
 template <class T>
-class MOZ_IS_REFPTR nsCOMPtr final
-#ifdef NSCAP_FEATURE_USE_BASE
-    : private nsCOMPtr_base
-#endif
-{
+class MOZ_IS_REFPTR nsCOMPtr final {
  private:
-#ifdef NSCAP_FEATURE_USE_BASE
-#  define NSCAP_CTOR_BASE(x) nsCOMPtr_base(ToSupports(x))
-  void assign_assuming_AddRef(T* aNewPtr) {
-    nsCOMPtr_base::assign_assuming_AddRef(ToSupports(aNewPtr));
-  }
-#else
-#  define NSCAP_CTOR_BASE(x) mRawPtr(x)
-
-  void assign_with_AddRef(nsISupports*);
+  void assign_with_AddRef(T*);
   template <typename U>
   void assign_from_qi(const nsQueryInterface<U>, const nsIID&);
   template <typename U>
@@ -418,40 +325,43 @@ class MOZ_IS_REFPTR nsCOMPtr final
 
  private:
   T* MOZ_OWNING_REF mRawPtr;
-#endif
 
   void assert_validity() {
     static_assert(1 < sizeof(TestForIID<T>(nullptr)),
                   "nsCOMPtr only works "
                   "for types with IIDs.  Either use RefPtr; add an IID to "
-                  "your type with NS_DECLARE_STATIC_IID_ACCESSOR/"
-                  "NS_DEFINE_STATIC_IID_ACCESSOR; or make the nsCOMPtr point "
+                  "your type with NS_INLINE_DECL_STATIC_IID/ ;"
+                  "or make the nsCOMPtr point "
                   "to a base class with an IID.");
   }
 
  public:
   typedef T element_type;
 
-#ifndef NSCAP_FEATURE_USE_BASE
   ~nsCOMPtr() {
     NSCAP_LOG_RELEASE(this, mRawPtr);
     if (mRawPtr) {
       NSCAP_RELEASE(this, mRawPtr);
     }
   }
-#endif
 
 #ifdef NSCAP_FEATURE_TEST_DONTQUERY_CASES
   void Assert_NoQueryNeeded() {
-    if (mRawPtr) {
-      // This can't be defined in terms of do_QueryInterface because
-      // that bans casts from a class to itself.
-      void* out = nullptr;
-      mRawPtr->QueryInterface(NS_GET_TEMPLATE_IID(T), &out);
-      T* query_result = static_cast<T*>(out);
-      MOZ_ASSERT(query_result == mRawPtr, "QueryInterface needed");
-      NS_RELEASE(query_result);
+    if (!mRawPtr) {
+      return;
     }
+    if constexpr (std::is_same_v<T, nsISupports>) {
+      // FIXME: nsCOMPtr<nsISupports> never asserted this, and it currently
+      // fails...
+      return;
+    }
+    // This can't be defined in terms of do_QueryInterface because
+    // that bans casts from a class to itself.
+    void* out = nullptr;
+    mRawPtr->QueryInterface(NS_GET_IID(T), &out);
+    T* query_result = static_cast<T*>(out);
+    MOZ_ASSERT(query_result == mRawPtr, "QueryInterface needed");
+    NS_RELEASE(query_result);
   }
 
 #  define NSCAP_ASSERT_NO_QUERY_NEEDED() Assert_NoQueryNeeded();
@@ -461,17 +371,17 @@ class MOZ_IS_REFPTR nsCOMPtr final
 
   // Constructors
 
-  nsCOMPtr() : NSCAP_CTOR_BASE(nullptr) {
+  nsCOMPtr() : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
   }
 
-  MOZ_IMPLICIT nsCOMPtr(decltype(nullptr)) : NSCAP_CTOR_BASE(nullptr) {
+  MOZ_IMPLICIT nsCOMPtr(decltype(nullptr)) : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
   }
 
-  nsCOMPtr(const nsCOMPtr<T>& aSmartPtr) : NSCAP_CTOR_BASE(aSmartPtr.mRawPtr) {
+  nsCOMPtr(const nsCOMPtr<T>& aSmartPtr) : mRawPtr(aSmartPtr.mRawPtr) {
     assert_validity();
     if (mRawPtr) {
       NSCAP_ADDREF(this, mRawPtr);
@@ -481,7 +391,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
 
   template <class U>
   MOZ_IMPLICIT nsCOMPtr(const nsCOMPtr<U>& aSmartPtr)
-      : NSCAP_CTOR_BASE(aSmartPtr.get()) {
+      : mRawPtr(aSmartPtr.get()) {
     // Make sure that U actually inherits from T
     static_assert(std::is_base_of<T, U>::value, "U should be a subclass of T");
     assert_validity();
@@ -491,7 +401,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
     NSCAP_LOG_ASSIGNMENT(this, aSmartPtr.get());
   }
 
-  nsCOMPtr(nsCOMPtr<T>&& aSmartPtr) : NSCAP_CTOR_BASE(aSmartPtr.mRawPtr) {
+  nsCOMPtr(nsCOMPtr<T>&& aSmartPtr) : mRawPtr(aSmartPtr.mRawPtr) {
     assert_validity();
     aSmartPtr.mRawPtr = nullptr;
     NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
@@ -499,7 +409,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
 
   template <class U>
   MOZ_IMPLICIT nsCOMPtr(nsCOMPtr<U>&& aSmartPtr)
-      : NSCAP_CTOR_BASE(aSmartPtr.forget().template downcast<T>().take()) {
+      : mRawPtr(aSmartPtr.forget().template downcast<T>().take()) {
     // Make sure that U actually inherits from T
     static_assert(std::is_base_of<T, U>::value, "U should be a subclass of T");
     assert_validity();
@@ -507,7 +417,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
     NSCAP_ASSERT_NO_QUERY_NEEDED();
   }
 
-  MOZ_IMPLICIT nsCOMPtr(T* aRawPtr) : NSCAP_CTOR_BASE(aRawPtr) {
+  MOZ_IMPLICIT nsCOMPtr(T* aRawPtr) : mRawPtr(aRawPtr) {
     assert_validity();
     if (mRawPtr) {
       NSCAP_ADDREF(this, mRawPtr);
@@ -517,7 +427,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   }
 
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<T>& aSmartPtr)
-      : NSCAP_CTOR_BASE(aSmartPtr.take()) {
+      : mRawPtr(aSmartPtr.take()) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
     NSCAP_ASSERT_NO_QUERY_NEEDED();
@@ -525,7 +435,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
 
   // Construct from |otherComPtr.forget()|.
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<T>&& aSmartPtr)
-      : NSCAP_CTOR_BASE(aSmartPtr.take()) {
+      : mRawPtr(aSmartPtr.take()) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
     NSCAP_ASSERT_NO_QUERY_NEEDED();
@@ -534,8 +444,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   // Construct from |std::move(otherRefPtr)|.
   template <typename U>
   MOZ_IMPLICIT nsCOMPtr(RefPtr<U>&& aSmartPtr)
-      : NSCAP_CTOR_BASE(
-            static_cast<already_AddRefed<T>>(aSmartPtr.forget()).take()) {
+      : mRawPtr(static_cast<already_AddRefed<T>>(aSmartPtr.forget()).take()) {
     assert_validity();
     // Make sure that U actually inherits from T
     static_assert(std::is_base_of<T, U>::value, "U is not a subclass of T");
@@ -546,7 +455,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   // Construct from |already_AddRefed|.
   template <typename U>
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<U>& aSmartPtr)
-      : NSCAP_CTOR_BASE(static_cast<T*>(aSmartPtr.take())) {
+      : mRawPtr(static_cast<T*>(aSmartPtr.take())) {
     assert_validity();
     // But make sure that U actually inherits from T.
     static_assert(std::is_base_of<T, U>::value, "U is not a subclass of T");
@@ -557,7 +466,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   // Construct from |otherComPtr.forget()|.
   template <typename U>
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<U>&& aSmartPtr)
-      : NSCAP_CTOR_BASE(static_cast<T*>(aSmartPtr.take())) {
+      : mRawPtr(static_cast<T*>(aSmartPtr.take())) {
     assert_validity();
     // But make sure that U actually inherits from T.
     static_assert(std::is_base_of<T, U>::value, "U is not a subclass of T");
@@ -566,81 +475,85 @@ class MOZ_IS_REFPTR nsCOMPtr final
   }
 
   // Construct from |do_QueryInterface(expr)|.
-#ifdef NSCAP_FEATURE_USE_BASE
-  MOZ_IMPLICIT nsCOMPtr(const nsQueryInterfaceISupports aQI)
-#else
   template <typename U>
-  MOZ_IMPLICIT nsCOMPtr(const nsQueryInterface<U> aQI)
-#endif  // ! #ifdef NSCAP_FEATURE_USE_BASE
-      : NSCAP_CTOR_BASE(nullptr) {
+  MOZ_IMPLICIT nsCOMPtr(const nsQueryInterface<U> aQI) : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_qi(aQI, NS_GET_TEMPLATE_IID(T));
+    assign_from_qi(aQI, NS_GET_IID(T));
   }
 
   // Construct from |do_QueryInterface(expr, &rv)|.
-#ifdef NSCAP_FEATURE_USE_BASE
-  MOZ_IMPLICIT nsCOMPtr(const nsQueryInterfaceISupportsWithError& aQI)
-#else
   template <typename U>
   MOZ_IMPLICIT nsCOMPtr(const nsQueryInterfaceWithError<U>& aQI)
-#endif  // ! #ifdef NSCAP_FEATURE_USE_BASE
-      : NSCAP_CTOR_BASE(nullptr) {
+      : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_qi_with_error(aQI, NS_GET_TEMPLATE_IID(T));
+    assign_from_qi_with_error(aQI, NS_GET_IID(T));
   }
 
   // Construct from |do_GetService(cid_expr)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCID aGS)
-      : NSCAP_CTOR_BASE(nullptr) {
+  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCID aGS) : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_cid(aGS, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_cid(aGS, NS_GET_IID(T));
   }
 
   // Construct from |do_GetService(cid_expr, &rv)|.
   MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCIDWithError& aGS)
-      : NSCAP_CTOR_BASE(nullptr) {
+      : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_cid_with_error(aGS, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_cid_with_error(aGS, NS_GET_IID(T));
   }
 
   // Construct from |do_GetService(contractid_expr)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractID aGS)
-      : NSCAP_CTOR_BASE(nullptr) {
+  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractID aGS) : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_contractid(aGS, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_contractid(aGS, NS_GET_IID(T));
   }
 
   // Construct from |do_GetService(contractid_expr, &rv)|.
   MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractIDWithError& aGS)
-      : NSCAP_CTOR_BASE(nullptr) {
+      : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_contractid_with_error(aGS, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_contractid_with_error(aGS, NS_GET_IID(T));
   }
 
   // Construct from |do_QueryReferent(ptr)|
   MOZ_IMPLICIT nsCOMPtr(const nsQueryReferent& aQueryReferent)
-      : NSCAP_CTOR_BASE(nullptr) {
+      : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_query_referent(aQueryReferent, NS_GET_TEMPLATE_IID(T));
+    assign_from_query_referent(aQueryReferent, NS_GET_IID(T));
   }
 
   // And finally, anything else we might need to construct from can exploit the
   // nsCOMPtr_helper facility.
-  MOZ_IMPLICIT nsCOMPtr(const nsCOMPtr_helper& aHelper)
-      : NSCAP_CTOR_BASE(nullptr) {
+  MOZ_IMPLICIT nsCOMPtr(const nsCOMPtr_helper& aHelper) : mRawPtr(nullptr) {
     assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_helper(aHelper, NS_GET_TEMPLATE_IID(T));
+    assign_from_helper(aHelper, NS_GET_IID(T));
     NSCAP_ASSERT_NO_QUERY_NEEDED();
   }
 
+  // construct from |mozilla::NotNull|.
+  template <typename I,
+            typename = std::enable_if_t<!std::is_same_v<I, nsCOMPtr<T>> &&
+                                        std::is_convertible_v<I, nsCOMPtr<T>>>>
+  MOZ_IMPLICIT nsCOMPtr(const mozilla::NotNull<I>& aSmartPtr)
+      : mRawPtr(nsCOMPtr<T>(aSmartPtr.get()).forget().take()) {}
+
+  // construct from |mozilla::MovingNotNull|.
+  template <typename I,
+            typename = std::enable_if_t<!std::is_same_v<I, nsCOMPtr<T>> &&
+                                        std::is_convertible_v<I, nsCOMPtr<T>>>>
+  MOZ_IMPLICIT nsCOMPtr(mozilla::MovingNotNull<I>&& aSmartPtr)
+      : mRawPtr(
+            nsCOMPtr<T>(std::move(aSmartPtr).unwrapBasePtr()).forget().take()) {
+  }
+
   // Defined in OwningNonNull.h
   template <class U>
   MOZ_IMPLICIT nsCOMPtr(const mozilla::OwningNonNull<U>& aOther);
@@ -648,7 +561,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   // Assignment operators
 
   nsCOMPtr<T>& operator=(const nsCOMPtr<T>& aRhs) {
-    assign_with_AddRef(ToSupports(aRhs.mRawPtr));
+    assign_with_AddRef(aRhs.mRawPtr);
     return *this;
   }
 
@@ -656,7 +569,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   nsCOMPtr<T>& operator=(const nsCOMPtr<U>& aRhs) {
     // Make sure that U actually inherits from T
     static_assert(std::is_base_of<T, U>::value, "U should be a subclass of T");
-    assign_with_AddRef(ToSupports(static_cast<T*>(aRhs.get())));
+    assign_with_AddRef(aRhs.get());
     return *this;
   }
 
@@ -675,7 +588,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
   }
 
   nsCOMPtr<T>& operator=(T* aRhs) {
-    assign_with_AddRef(ToSupports(aRhs));
+    assign_with_AddRef(aRhs);
     NSCAP_ASSERT_NO_QUERY_NEEDED();
     return *this;
   }
@@ -716,78 +629,81 @@ class MOZ_IS_REFPTR nsCOMPtr final
   }
 
   // Assign from |do_QueryInterface(expr)|.
-#ifdef NSCAP_FEATURE_USE_BASE
-  nsCOMPtr<T>& operator=(const nsQueryInterfaceISupports aRhs)
-#else
   template <typename U>
-  nsCOMPtr<T>& operator=(const nsQueryInterface<U> aRhs)
-#endif  // ! #ifdef NSCAP_FEATURE_USE_BASE
-  {
-    assign_from_qi(aRhs, NS_GET_TEMPLATE_IID(T));
+  nsCOMPtr<T>& operator=(const nsQueryInterface<U> aRhs) {
+    assign_from_qi(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // Assign from |do_QueryInterface(expr, &rv)|.
-#ifdef NSCAP_FEATURE_USE_BASE
-  nsCOMPtr<T>& operator=(const nsQueryInterfaceISupportsWithError& aRhs)
-#else
   template <typename U>
-  nsCOMPtr<T>& operator=(const nsQueryInterfaceWithError<U>& aRhs)
-#endif  // ! #ifdef NSCAP_FEATURE_USE_BASE
-  {
-    assign_from_qi_with_error(aRhs, NS_GET_TEMPLATE_IID(T));
+  nsCOMPtr<T>& operator=(const nsQueryInterfaceWithError<U>& aRhs) {
+    assign_from_qi_with_error(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // Assign from |do_GetService(cid_expr)|.
   nsCOMPtr<T>& operator=(const nsGetServiceByCID aRhs) {
-    assign_from_gs_cid(aRhs, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_cid(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // Assign from |do_GetService(cid_expr, &rv)|.
   nsCOMPtr<T>& operator=(const nsGetServiceByCIDWithError& aRhs) {
-    assign_from_gs_cid_with_error(aRhs, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_cid_with_error(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // Assign from |do_GetService(contractid_expr)|.
   nsCOMPtr<T>& operator=(const nsGetServiceByContractID aRhs) {
-    assign_from_gs_contractid(aRhs, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_contractid(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // Assign from |do_GetService(contractid_expr, &rv)|.
   nsCOMPtr<T>& operator=(const nsGetServiceByContractIDWithError& aRhs) {
-    assign_from_gs_contractid_with_error(aRhs, NS_GET_TEMPLATE_IID(T));
+    assign_from_gs_contractid_with_error(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // Assign from |do_QueryReferent(ptr)|.
   nsCOMPtr<T>& operator=(const nsQueryReferent& aRhs) {
-    assign_from_query_referent(aRhs, NS_GET_TEMPLATE_IID(T));
+    assign_from_query_referent(aRhs, NS_GET_IID(T));
     return *this;
   }
 
   // And finally, anything else we might need to assign from can exploit the
   // nsCOMPtr_helper facility.
   nsCOMPtr<T>& operator=(const nsCOMPtr_helper& aRhs) {
-    assign_from_helper(aRhs, NS_GET_TEMPLATE_IID(T));
+    assign_from_helper(aRhs, NS_GET_IID(T));
     NSCAP_ASSERT_NO_QUERY_NEEDED();
     return *this;
   }
 
+  // Assign from |mozilla::NotNull|.
+  template <typename I,
+            typename = std::enable_if_t<std::is_convertible_v<I, nsCOMPtr<T>>>>
+  nsCOMPtr<T>& operator=(const mozilla::NotNull<I>& aSmartPtr) {
+    assign_assuming_AddRef(nsCOMPtr<T>(aSmartPtr.get()).forget().take());
+    return *this;
+  }
+
+  // Assign from |mozilla::MovingNotNull|.
+  template <typename I,
+            typename = std::enable_if_t<std::is_convertible_v<I, nsCOMPtr<T>>>>
+  nsCOMPtr<T>& operator=(mozilla::MovingNotNull<I>&& aSmartPtr) {
+    assign_assuming_AddRef(
+        nsCOMPtr<T>(std::move(aSmartPtr).unwrapBasePtr()).forget().take());
+    return *this;
+  }
+
   // Defined in OwningNonNull.h
   template <class U>
   nsCOMPtr<T>& operator=(const mozilla::OwningNonNull<U>& aOther);
 
   // Exchange ownership with |aRhs|; can save a pair of refcount operations.
   void swap(nsCOMPtr<T>& aRhs) {
-#ifdef NSCAP_FEATURE_USE_BASE
-    nsISupports* temp = aRhs.mRawPtr;
-#else
     T* temp = aRhs.mRawPtr;
-#endif
     NSCAP_LOG_ASSIGNMENT(&aRhs, mRawPtr);
     NSCAP_LOG_ASSIGNMENT(this, temp);
     NSCAP_LOG_RELEASE(this, mRawPtr);
@@ -800,11 +716,7 @@ class MOZ_IS_REFPTR nsCOMPtr final
 
   // Exchange ownership with |aRhs|; can save a pair of refcount operations.
   void swap(T*& aRhs) {
-#ifdef NSCAP_FEATURE_USE_BASE
-    nsISupports* temp = ToSupports(aRhs);
-#else
     T* temp = aRhs;
-#endif
     NSCAP_LOG_ASSIGNMENT(this, temp);
     NSCAP_LOG_RELEASE(this, mRawPtr);
     aRhs = reinterpret_cast<T*>(mRawPtr);
@@ -881,271 +793,6 @@ class MOZ_IS_REFPTR nsCOMPtr final
   }
 };
 
-/*
- * Specializing nsCOMPtr for nsISupports allows us to use nsCOMPtr<nsISupports>
- * the same way people use nsISupports* and void*, i.e., as a `catch-all'
- * pointing to any valid [XP]COM interface. Otherwise, an nsCOMPtr<nsISupports>
- * would only be able to point to the single [XP]COM-correct nsISupports
- * instance within an object; extra querying ensues. Clients need to be able to
- * pass around arbitrary interface pointers, without hassles, through
- * intermediary code that doesn't know the exact type.
- */
-template <>
-class MOZ_IS_REFPTR nsCOMPtr<nsISupports> : private nsCOMPtr_base {
- public:
-  typedef nsISupports element_type;
-
-  // Constructors
-
-  nsCOMPtr() : nsCOMPtr_base(nullptr) { NSCAP_LOG_ASSIGNMENT(this, nullptr); }
-
-  MOZ_IMPLICIT nsCOMPtr(decltype(nullptr)) : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-  }
-
-  nsCOMPtr(const nsCOMPtr<nsISupports>& aSmartPtr)
-      : nsCOMPtr_base(aSmartPtr.mRawPtr) {
-    if (mRawPtr) {
-      NSCAP_ADDREF(this, mRawPtr);
-    }
-    NSCAP_LOG_ASSIGNMENT(this, aSmartPtr.mRawPtr);
-  }
-
-  MOZ_IMPLICIT nsCOMPtr(nsISupports* aRawPtr) : nsCOMPtr_base(aRawPtr) {
-    if (mRawPtr) {
-      NSCAP_ADDREF(this, mRawPtr);
-    }
-    NSCAP_LOG_ASSIGNMENT(this, aRawPtr);
-  }
-
-  // Construct from |already_AddRefed|.
-  MOZ_IMPLICIT nsCOMPtr(already_AddRefed<nsISupports>& aSmartPtr)
-      : nsCOMPtr_base(aSmartPtr.take()) {
-    NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
-  }
-
-  // Construct from |otherComPtr.forget()|.
-  MOZ_IMPLICIT nsCOMPtr(already_AddRefed<nsISupports>&& aSmartPtr)
-      : nsCOMPtr_base(aSmartPtr.take()) {
-    NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
-  }
-
-  // Construct from |do_QueryInterface(expr)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsQueryInterfaceISupports aQI)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_qi(aQI, NS_GET_IID(nsISupports));
-  }
-
-  // Construct from |do_QueryInterface(expr, &rv)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsQueryInterfaceISupportsWithError& aQI)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_qi_with_error(aQI, NS_GET_IID(nsISupports));
-  }
-
-  // Construct from |do_GetService(cid_expr)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCID aGS) : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_cid(aGS, NS_GET_IID(nsISupports));
-  }
-
-  // Construct from |do_GetService(cid_expr, &rv)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCIDWithError& aGS)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_cid_with_error(aGS, NS_GET_IID(nsISupports));
-  }
-
-  // Construct from |do_GetService(contractid_expr)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractID aGS)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_contractid(aGS, NS_GET_IID(nsISupports));
-  }
-
-  // Construct from |do_GetService(contractid_expr, &rv)|.
-  MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractIDWithError& aGS)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_gs_contractid_with_error(aGS, NS_GET_IID(nsISupports));
-  }
-
-  // Construct from |do_QueryReferent(ptr)|
-  MOZ_IMPLICIT nsCOMPtr(const nsQueryReferent& aQueryReferent)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_query_referent(aQueryReferent,
-                               NS_GET_TEMPLATE_IID(nsISupports));
-  }
-
-  // And finally, anything else we might need to construct from can exploit
-  // the |nsCOMPtr_helper| facility
-  MOZ_IMPLICIT nsCOMPtr(const nsCOMPtr_helper& aHelper)
-      : nsCOMPtr_base(nullptr) {
-    NSCAP_LOG_ASSIGNMENT(this, nullptr);
-    assign_from_helper(aHelper, NS_GET_IID(nsISupports));
-  }
-
-  // Assignment operators
-
-  nsCOMPtr<nsISupports>& operator=(const nsCOMPtr<nsISupports>& aRhs) {
-    assign_with_AddRef(aRhs.mRawPtr);
-    return *this;
-  }
-
-  nsCOMPtr<nsISupports>& operator=(nsISupports* aRhs) {
-    assign_with_AddRef(aRhs);
-    return *this;
-  }
-
-  nsCOMPtr<nsISupports>& operator=(decltype(nullptr)) {
-    assign_assuming_AddRef(nullptr);
-    return *this;
-  }
-
-  // Assign from |already_AddRefed|.
-  nsCOMPtr<nsISupports>& operator=(already_AddRefed<nsISupports>& aRhs) {
-    assign_assuming_AddRef(aRhs.take());
-    return *this;
-  }
-
-  // Assign from |otherComPtr.forget()|.
-  nsCOMPtr<nsISupports>& operator=(already_AddRefed<nsISupports>&& aRhs) {
-    assign_assuming_AddRef(aRhs.take());
-    return *this;
-  }
-
-  // Assign from |do_QueryInterface(expr)|.
-  nsCOMPtr<nsISupports>& operator=(const nsQueryInterfaceISupports aRhs) {
-    assign_from_qi(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Assign from |do_QueryInterface(expr, &rv)|.
-  nsCOMPtr<nsISupports>& operator=(
-      const nsQueryInterfaceISupportsWithError& aRhs) {
-    assign_from_qi_with_error(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Assign from |do_GetService(cid_expr)|.
-  nsCOMPtr<nsISupports>& operator=(const nsGetServiceByCID aRhs) {
-    assign_from_gs_cid(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Assign from |do_GetService(cid_expr, &rv)|.
-  nsCOMPtr<nsISupports>& operator=(const nsGetServiceByCIDWithError& aRhs) {
-    assign_from_gs_cid_with_error(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Assign from |do_GetService(contractid_expr)|.
-  nsCOMPtr<nsISupports>& operator=(const nsGetServiceByContractID aRhs) {
-    assign_from_gs_contractid(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Assign from |do_GetService(contractid_expr, &rv)|.
-  nsCOMPtr<nsISupports>& operator=(
-      const nsGetServiceByContractIDWithError& aRhs) {
-    assign_from_gs_contractid_with_error(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Assign from |do_QueryReferent(ptr)|.
-  nsCOMPtr<nsISupports>& operator=(const nsQueryReferent& aRhs) {
-    assign_from_query_referent(aRhs, NS_GET_TEMPLATE_IID(nsISupports));
-    return *this;
-  }
-
-  // And finally, anything else we might need to assign from can exploit the
-  // nsCOMPtr_helper facility
-  nsCOMPtr<nsISupports>& operator=(const nsCOMPtr_helper& aRhs) {
-    assign_from_helper(aRhs, NS_GET_IID(nsISupports));
-    return *this;
-  }
-
-  // Exchange ownership with |aRhs|; can save a pair of refcount operations.
-  void swap(nsCOMPtr<nsISupports>& aRhs) {
-    nsISupports* temp = aRhs.mRawPtr;
-    NSCAP_LOG_ASSIGNMENT(&aRhs, mRawPtr);
-    NSCAP_LOG_ASSIGNMENT(this, temp);
-    NSCAP_LOG_RELEASE(this, mRawPtr);
-    NSCAP_LOG_RELEASE(&aRhs, temp);
-    aRhs.mRawPtr = mRawPtr;
-    mRawPtr = temp;
-  }
-
-  // Exchange ownership with |aRhs|; can save a pair of refcount operations.
-  void swap(nsISupports*& aRhs) {
-    nsISupports* temp = aRhs;
-    NSCAP_LOG_ASSIGNMENT(this, temp);
-    NSCAP_LOG_RELEASE(this, mRawPtr);
-    aRhs = mRawPtr;
-    mRawPtr = temp;
-  }
-
-  // Return the value of mRawPtr and null out mRawPtr. Useful for
-  // already_AddRefed return values.
-  already_AddRefed<nsISupports> forget() {
-    nsISupports* temp = nullptr;
-    swap(temp);
-    return already_AddRefed<nsISupports>(temp);
-  }
-
-  // Set the target of aRhs to the value of mRawPtr and null out mRawPtr.
-  // Useful to avoid unnecessary AddRef/Release pairs with "out"
-  // parameters.
-  void forget(nsISupports** aRhs) {
-    NS_ASSERTION(aRhs, "Null pointer passed to forget!");
-    *aRhs = nullptr;
-    swap(*aRhs);
-  }
-
-  // Other pointer operators
-
-  // Prefer the implicit conversion provided automatically by
-  // |operator nsISupports*() const|. Use |get()| to resolve ambiguity or to
-  // get a castable pointer.
-  nsISupports* get() const { return reinterpret_cast<nsISupports*>(mRawPtr); }
-
-  // Makes an nsCOMPtr act like its underlying raw pointer type whenever it is
-  // used in a context where a raw pointer is expected. It is this operator
-  // that makes an nsCOMPtr substitutable for a raw pointer.
-  //
-  // Prefer the implicit use of this operator to calling |get()|, except where
-  // necessary to resolve ambiguity/
-  operator nsISupports*() const { return get(); }
-
-  nsISupports* operator->() const MOZ_NO_ADDREF_RELEASE_ON_RETURN {
-    MOZ_ASSERT(mRawPtr != nullptr,
-               "You can't dereference a NULL nsCOMPtr with operator->().");
-    return get();
-  }
-
-  // These are not intended to be used by clients. See |address_of| below.
-  nsCOMPtr<nsISupports>* get_address() { return this; }
-  const nsCOMPtr<nsISupports>* get_address() const { return this; }
-
- public:
-  nsISupports& operator*() const {
-    MOZ_ASSERT(mRawPtr != nullptr,
-               "You can't dereference a NULL nsCOMPtr with operator*().");
-    return *get();
-  }
-
-  nsISupports** StartAssignment() {
-#ifndef NSCAP_FEATURE_INLINE_STARTASSIGNMENT
-    return reinterpret_cast<nsISupports**>(begin_assignment());
-#else
-    assign_assuming_AddRef(nullptr);
-    return reinterpret_cast<nsISupports**>(&mRawPtr);
-#endif
-  }
-};
-
 template <typename T>
 inline void ImplCycleCollectionUnlink(nsCOMPtr<T>& aField) {
   aField = nullptr;
@@ -1158,21 +805,23 @@ inline void ImplCycleCollectionTraverse(
   CycleCollectionNoteChild(aCallback, aField.get(), aName, aFlags);
 }
 
-#ifndef NSCAP_FEATURE_USE_BASE
 template <class T>
-void nsCOMPtr<T>::assign_with_AddRef(nsISupports* aRawPtr) {
+void nsCOMPtr<T>::assign_with_AddRef(T* aRawPtr) {
   if (aRawPtr) {
     NSCAP_ADDREF(this, aRawPtr);
   }
-  assign_assuming_AddRef(reinterpret_cast<T*>(aRawPtr));
+  assign_assuming_AddRef(aRawPtr);
 }
 
 template <class T>
 template <typename U>
 void nsCOMPtr<T>::assign_from_qi(const nsQueryInterface<U> aQI,
                                  const nsIID& aIID) {
+  // Allow QIing to nsISupports from nsISupports as a special-case, since
+  // SameCOMIdentity uses it.
   static_assert(
-      !(std::is_same_v<T, U> || std::is_base_of<T, U>::value),
+      std::is_same_v<T, nsISupports> ||
+          !(std::is_same_v<T, U> || std::is_base_of<T, U>::value),
       "don't use do_QueryInterface for compile-time-determinable casts");
   void* newRawPtr;
   if (NS_FAILED(aQI(aIID, &newRawPtr))) {
@@ -1265,7 +914,6 @@ void** nsCOMPtr<T>::begin_assignment() {
   result.mT = &mRawPtr;
   return result.mVoid;
 }
-#endif
 
 template <class T>
 inline nsCOMPtr<T>* address_of(nsCOMPtr<T>& aPtr) {
@@ -1451,7 +1099,7 @@ inline nsresult CallQueryInterface(nsCOMPtr<SourceType>& aSourcePtr,
 template <class T>
 RefPtr<T>::RefPtr(const nsQueryReferent& aQueryReferent) {
   void* newRawPtr;
-  if (NS_FAILED(aQueryReferent(NS_GET_TEMPLATE_IID(T), &newRawPtr))) {
+  if (NS_FAILED(aQueryReferent(NS_GET_IID(T), &newRawPtr))) {
     newRawPtr = nullptr;
   }
   mRawPtr = static_cast<T*>(newRawPtr);
@@ -1460,7 +1108,7 @@ RefPtr<T>::RefPtr(const nsQueryReferent& aQueryReferent) {
 template <class T>
 RefPtr<T>::RefPtr(const nsCOMPtr_helper& aHelper) {
   void* newRawPtr;
-  if (NS_FAILED(aHelper(NS_GET_TEMPLATE_IID(T), &newRawPtr))) {
+  if (NS_FAILED(aHelper(NS_GET_IID(T), &newRawPtr))) {
     newRawPtr = nullptr;
   }
   mRawPtr = static_cast<T*>(newRawPtr);
@@ -1469,7 +1117,7 @@ RefPtr<T>::RefPtr(const nsCOMPtr_helper& aHelper) {
 template <class T>
 RefPtr<T>& RefPtr<T>::operator=(const nsQueryReferent& aQueryReferent) {
   void* newRawPtr;
-  if (NS_FAILED(aQueryReferent(NS_GET_TEMPLATE_IID(T), &newRawPtr))) {
+  if (NS_FAILED(aQueryReferent(NS_GET_IID(T), &newRawPtr))) {
     newRawPtr = nullptr;
   }
   assign_assuming_AddRef(static_cast<T*>(newRawPtr));
@@ -1479,7 +1127,7 @@ RefPtr<T>& RefPtr<T>::operator=(const nsQueryReferent& aQueryReferent) {
 template <class T>
 RefPtr<T>& RefPtr<T>::operator=(const nsCOMPtr_helper& aHelper) {
   void* newRawPtr;
-  if (NS_FAILED(aHelper(NS_GET_TEMPLATE_IID(T), &newRawPtr))) {
+  if (NS_FAILED(aHelper(NS_GET_IID(T), &newRawPtr))) {
     newRawPtr = nullptr;
   }
   assign_assuming_AddRef(static_cast<T*>(newRawPtr));
