@@ -12,6 +12,7 @@
 
 #include "nsXPCOM.h"
 #include "mozilla/Assertions.h"
+#include "mozilla/glue/Debug.h"
 #include "mozilla/DbgMacro.h"
 #include "mozilla/Likely.h"
 #include <stdarg.h>
@@ -162,13 +163,13 @@ inline void MOZ_PretendNoReturn() MOZ_PRETEND_NORETURN_FOR_STATIC_ANALYSIS {}
 
 #  ifdef XGILL_PLUGIN
 
-#    define STATIC_PRECONDITION(COND) __attribute__((precondition(#    COND)))
+#    define STATIC_PRECONDITION(COND) __attribute__((precondition(#COND)))
 #    define STATIC_PRECONDITION_ASSUME(COND) \
       __attribute__((precondition_assume(#COND)))
-#    define STATIC_POSTCONDITION(COND) __attribute__((postcondition(#    COND)))
+#    define STATIC_POSTCONDITION(COND) __attribute__((postcondition(#COND)))
 #    define STATIC_POSTCONDITION_ASSUME(COND) \
       __attribute__((postcondition_assume(#COND)))
-#    define STATIC_INVARIANT(COND) __attribute__((invariant(#    COND)))
+#    define STATIC_INVARIANT(COND) __attribute__((invariant(#COND)))
 #    define STATIC_INVARIANT_ASSUME(COND) \
       __attribute__((invariant_assume(#COND)))
 
@@ -313,9 +314,6 @@ inline void MOZ_PretendNoReturn() MOZ_PRETEND_NORETURN_FOR_STATIC_ANALYSIS {}
 
 #define NS_ENSURE_STATE(state) NS_ENSURE_TRUE(state, NS_ERROR_UNEXPECTED)
 
-#define NS_ENSURE_NO_AGGREGATION(outer) \
-  NS_ENSURE_FALSE(outer, NS_ERROR_NO_AGGREGATION)
-
 /*****************************************************************************/
 
 #if (defined(DEBUG) || (defined(NIGHTLY_BUILD) && !defined(MOZ_PROFILING))) && \
@@ -329,59 +327,4 @@ void NS_ABORT_OOM(size_t aSize);
 inline void NS_ABORT_OOM(size_t) { MOZ_CRASH(); }
 #endif
 
-/* When compiling the XPCOM Glue on Windows, we pretend that it's going to
- * be linked with a static CRT (-MT) even when it's not. This means that we
- * cannot link to data exports from the CRT, only function exports. So,
- * instead of referencing "stderr" directly, use fdopen.
- */
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-/**
- * printf_stderr(...) is much like fprintf(stderr, ...), except that:
- *  - on Android and Firefox OS, *instead* of printing to stderr, it
- *    prints to logcat.  (Newlines in the string lead to multiple lines
- *    of logcat, but each function call implicitly completes a line even
- *    if the string does not end with a newline.)
- *  - on Windows, if a debugger is present, it calls OutputDebugString
- *    in *addition* to writing to stderr
- */
-void printf_stderr(const char* aFmt, ...) MOZ_FORMAT_PRINTF(1, 2);
-
-/**
- * Same as printf_stderr, but taking va_list instead of varargs
- */
-void vprintf_stderr(const char* aFmt, va_list aArgs) MOZ_FORMAT_PRINTF(1, 0);
-
-/**
- * fprintf_stderr is like fprintf, except that if its file argument
- * is stderr, it invokes printf_stderr instead.
- *
- * This is useful for general debugging code that logs information to a
- * file, but that you would like to be useful on Android and Firefox OS.
- * If you use fprintf_stderr instead of fprintf in such debugging code,
- * then callers can pass stderr to get logging that works on Android and
- * Firefox OS (and also the other side-effects of using printf_stderr).
- *
- * Code that is structured this way needs to be careful not to split a
- * line of output across multiple calls to fprintf_stderr, since doing
- * so will cause it to appear in multiple lines in logcat output.
- * (Producing multiple lines at once is fine.)
- */
-void fprintf_stderr(FILE* aFile, const char* aFmt, ...) MOZ_FORMAT_PRINTF(2, 3);
-
-/*
- * print_stderr and fprint_stderr are like printf_stderr and fprintf_stderr,
- * except they deal with Android logcat line length limitations. They do this
- * by printing individual lines out of the provided stringstream using separate
- * calls to logcat.
- */
-void print_stderr(std::stringstream& aStr);
-void fprint_stderr(FILE* aFile, std::stringstream& aStr);
-
-#ifdef __cplusplus
-}
-#endif
-
 #endif /* nsDebug_h___ */
