@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <type_traits>
 #include <utility>
 
 #include "mozilla/HashFunctions.h"
@@ -52,9 +53,7 @@ inline uint32_t HashString(const nsACString& aStr) {
  * nsFloatHashKey
  * IntPtrHashKey
  * nsPtrHashKey
- * nsClearingPtrHashKey
  * nsVoidPtrHashKey
- * nsClearingVoidPtrHashKey
  * nsISupportsHashKey
  * nsIDHashKey
  * nsDepCharHashKey
@@ -100,39 +99,78 @@ class nsStringHashKey : public PLDHashEntryHdr {
   nsString mStr;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsStringHashKey& aField, const char* aName, uint32_t aFlags = 0) {}
+
 #ifdef MOZILLA_INTERNAL_API
 
+namespace mozilla::detail {
+
+template <class CharT, bool Unicode = true>
+struct comparatorTraits {};
+
+template <>
+struct comparatorTraits<char, false> {
+  static int caseInsensitiveCompare(const char* aLhs, const char* aRhs,
+                                    size_t aLhsLength, size_t aRhsLength) {
+    return nsCaseInsensitiveCStringComparator(aLhs, aRhs, aLhsLength,
+                                              aRhsLength);
+  };
+};
+
+template <>
+struct comparatorTraits<char, true> {
+  static int caseInsensitiveCompare(const char* aLhs, const char* aRhs,
+                                    size_t aLhsLength, size_t aRhsLength) {
+    return nsCaseInsensitiveUTF8StringComparator(aLhs, aRhs, aLhsLength,
+                                                 aRhsLength);
+  };
+};
+
+template <>
+struct comparatorTraits<char16_t, true> {
+  static int caseInsensitiveCompare(const char16_t* aLhs, const char16_t* aRhs,
+                                    size_t aLhsLength, size_t aRhsLength) {
+    return nsCaseInsensitiveStringComparator(aLhs, aRhs, aLhsLength,
+                                             aRhsLength);
+  };
+};
+
+}  // namespace mozilla::detail
+
 /**
- * hashkey wrapper using nsAString KeyType
- *
- * This is internal-API only because nsCaseInsensitiveStringComparator is
+ * This is internal-API only because nsCaseInsensitive{C}StringComparator is
  * internal-only.
  *
  * @see nsTHashtable::EntryType for specification
  */
-class nsStringCaseInsensitiveHashKey : public PLDHashEntryHdr {
- public:
-  typedef const nsAString& KeyType;
-  typedef const nsAString* KeyTypePointer;
 
-  explicit nsStringCaseInsensitiveHashKey(KeyTypePointer aStr) : mStr(*aStr) {
+template <typename T, bool Unicode>
+class nsTStringCaseInsensitiveHashKey : public PLDHashEntryHdr {
+ public:
+  typedef const nsTSubstring<T>& KeyType;
+  typedef const nsTSubstring<T>* KeyTypePointer;
+
+  explicit nsTStringCaseInsensitiveHashKey(KeyTypePointer aStr) : mStr(*aStr) {
     // take it easy just deal HashKey
   }
 
-  nsStringCaseInsensitiveHashKey(const nsStringCaseInsensitiveHashKey&) =
+  nsTStringCaseInsensitiveHashKey(const nsTStringCaseInsensitiveHashKey&) =
       delete;
-  nsStringCaseInsensitiveHashKey(nsStringCaseInsensitiveHashKey&& aToMove)
+  nsTStringCaseInsensitiveHashKey(nsTStringCaseInsensitiveHashKey&& aToMove)
       : PLDHashEntryHdr(std::move(aToMove)), mStr(std::move(aToMove.mStr)) {}
-  ~nsStringCaseInsensitiveHashKey() = default;
+  ~nsTStringCaseInsensitiveHashKey() = default;
 
   KeyType GetKey() const { return mStr; }
   bool KeyEquals(const KeyTypePointer aKey) const {
-    return mStr.Equals(*aKey, nsCaseInsensitiveStringComparator);
+    using comparator = typename mozilla::detail::comparatorTraits<T, Unicode>;
+    return mStr.Equals(*aKey, comparator::caseInsensitiveCompare);
   }
 
   static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
   static PLDHashNumber HashKey(const KeyTypePointer aKey) {
-    nsAutoString tmKey(*aKey);
+    nsTAutoString<T> tmKey(*aKey);
     ToLowerCase(tmKey);
     return mozilla::HashString(tmKey);
   }
@@ -144,10 +182,23 @@ class nsStringCaseInsensitiveHashKey : public PLDHashEntryHdr {
   }
 
  private:
-  const nsString mStr;
+  const nsTString<T> mStr;
 };
 
-#endif
+template <class T, bool Unicode>
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsTStringCaseInsensitiveHashKey<T, Unicode>& aField,
+    const char* aName, uint32_t aFlags = 0) {}
+
+using nsStringCaseInsensitiveHashKey =
+    nsTStringCaseInsensitiveHashKey<char16_t, true>;
+using nsCStringASCIICaseInsensitiveHashKey =
+    nsTStringCaseInsensitiveHashKey<char, false>;
+using nsCStringUTF8CaseInsensitiveHashKey =
+    nsTStringCaseInsensitiveHashKey<char, true>;
+
+#endif  // MOZILLA_INTERNAL_API
 
 /**
  * hashkey wrapper using nsACString KeyType
@@ -185,60 +236,60 @@ class nsCStringHashKey : public PLDHashEntryHdr {
   const nsCString mStr;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsCStringHashKey& aField, const char* aName, uint32_t aFlags = 0) {}
+
 /**
- * hashkey wrapper using uint32_t KeyType
+ * hashkey wrapper using integral or enum KeyTypes
  *
  * @see nsTHashtable::EntryType for specification
  */
-class nsUint32HashKey : public PLDHashEntryHdr {
+template <typename T,
+          std::enable_if_t<std::is_integral_v<T> || std::is_enum_v<T>, int> = 0>
+class nsIntegralHashKey : public PLDHashEntryHdr {
  public:
-  typedef const uint32_t& KeyType;
-  typedef const uint32_t* KeyTypePointer;
+  using KeyType = const T&;
+  using KeyTypePointer = const T*;
 
-  explicit nsUint32HashKey(KeyTypePointer aKey) : mValue(*aKey) {}
-  nsUint32HashKey(nsUint32HashKey&& aOther)
-      : PLDHashEntryHdr(std::move(aOther)), mValue(std::move(aOther.mValue)) {}
-  ~nsUint32HashKey() = default;
-
-  KeyType GetKey() const { return mValue; }
-  bool KeyEquals(KeyTypePointer aKey) const { return *aKey == mValue; }
-
-  static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
-  static PLDHashNumber HashKey(KeyTypePointer aKey) { return *aKey; }
-  enum { ALLOW_MEMMOVE = true };
-
- private:
-  const uint32_t mValue;
-};
-
-/**
- * hashkey wrapper using uint64_t KeyType
- *
- * @see nsTHashtable::EntryType for specification
- */
-class nsUint64HashKey : public PLDHashEntryHdr {
- public:
-  typedef const uint64_t& KeyType;
-  typedef const uint64_t* KeyTypePointer;
-
-  explicit nsUint64HashKey(KeyTypePointer aKey) : mValue(*aKey) {}
-  nsUint64HashKey(nsUint64HashKey&& aOther)
-      : PLDHashEntryHdr(std::move(aOther)), mValue(std::move(aOther.mValue)) {}
-  ~nsUint64HashKey() = default;
+  explicit nsIntegralHashKey(KeyTypePointer aKey) : mValue(*aKey) {}
+  nsIntegralHashKey(nsIntegralHashKey&& aOther) noexcept
+      : PLDHashEntryHdr(std::move(aOther)), mValue(aOther.mValue) {}
+  ~nsIntegralHashKey() = default;
 
   KeyType GetKey() const { return mValue; }
   bool KeyEquals(KeyTypePointer aKey) const { return *aKey == mValue; }
 
   static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
   static PLDHashNumber HashKey(KeyTypePointer aKey) {
-    return PLDHashNumber(*aKey);
+    return mozilla::HashGeneric(*aKey);
   }
   enum { ALLOW_MEMMOVE = true };
 
  private:
-  const uint64_t mValue;
+  const T mValue;
 };
 
+template <typename T>
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsIntegralHashKey<T>& aField, const char* aName,
+    uint32_t aFlags = 0) {}
+
+/**
+ * hashkey wrapper using uint32_t KeyType
+ *
+ * @see nsTHashtable::EntryType for specification
+ */
+using nsUint32HashKey = nsIntegralHashKey<uint32_t>;
+
+/**
+ * hashkey wrapper using uint64_t KeyType
+ *
+ * @see nsTHashtable::EntryType for specification
+ */
+using nsUint64HashKey = nsIntegralHashKey<uint64_t>;
+
 /**
  * hashkey wrapper using float KeyType
  *
@@ -267,33 +318,16 @@ class nsFloatHashKey : public PLDHashEntryHdr {
   const float mValue;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback, const nsFloatHashKey& aField,
+    const char* aName, uint32_t aFlags = 0) {}
+
 /**
  * hashkey wrapper using intptr_t KeyType
  *
  * @see nsTHashtable::EntryType for specification
  */
-class IntPtrHashKey : public PLDHashEntryHdr {
- public:
-  typedef const intptr_t& KeyType;
-  typedef const intptr_t* KeyTypePointer;
-
-  explicit IntPtrHashKey(KeyTypePointer aKey) : mValue(*aKey) {}
-  IntPtrHashKey(IntPtrHashKey&& aOther)
-      : PLDHashEntryHdr(std::move(aOther)), mValue(aOther.mValue) {}
-  ~IntPtrHashKey() = default;
-
-  KeyType GetKey() const { return mValue; }
-  bool KeyEquals(KeyTypePointer aKey) const { return *aKey == mValue; }
-
-  static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
-  static PLDHashNumber HashKey(KeyTypePointer aKey) {
-    return mozilla::HashGeneric(*aKey);
-  }
-  enum { ALLOW_MEMMOVE = true };
-
- private:
-  const intptr_t mValue;
-};
+using IntPtrHashKey = nsIntegralHashKey<intptr_t>;
 
 /**
  * hashkey wrapper using nsISupports* KeyType
@@ -302,14 +336,12 @@ class IntPtrHashKey : public PLDHashEntryHdr {
  */
 class nsISupportsHashKey : public PLDHashEntryHdr {
  public:
-  typedef nsISupports* KeyType;
-  typedef const nsISupports* KeyTypePointer;
+  using KeyType = nsISupports*;
+  using KeyTypePointer = const nsISupports*;
 
   explicit nsISupportsHashKey(const nsISupports* aKey)
       : mSupports(const_cast<nsISupports*>(aKey)) {}
-  nsISupportsHashKey(nsISupportsHashKey&& aOther)
-      : PLDHashEntryHdr(std::move(aOther)),
-        mSupports(std::move(aOther.mSupports)) {}
+  nsISupportsHashKey(nsISupportsHashKey&& aOther) = default;
   ~nsISupportsHashKey() = default;
 
   KeyType GetKey() const { return mSupports; }
@@ -317,7 +349,7 @@ class nsISupportsHashKey : public PLDHashEntryHdr {
 
   static KeyTypePointer KeyToPointer(KeyType aKey) { return aKey; }
   static PLDHashNumber HashKey(KeyTypePointer aKey) {
-    return NS_PTR_TO_UINT32(aKey) >> 2;
+    return mozilla::HashGeneric(aKey);
   }
   enum { ALLOW_MEMMOVE = true };
 
@@ -325,6 +357,13 @@ class nsISupportsHashKey : public PLDHashEntryHdr {
   nsCOMPtr<nsISupports> mSupports;
 };
 
+template <class T>
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsISupportsHashKey& aField, const char* aName, uint32_t aFlags = 0) {
+  CycleCollectionNoteChild(aCallback, aField.GetKey(), aName, aFlags);
+}
+
 /**
  * hashkey wrapper using refcounted * KeyType
  *
@@ -333,12 +372,11 @@ class nsISupportsHashKey : public PLDHashEntryHdr {
 template <class T>
 class nsRefPtrHashKey : public PLDHashEntryHdr {
  public:
-  typedef T* KeyType;
-  typedef const T* KeyTypePointer;
+  using KeyType = T*;
+  using KeyTypePointer = const T*;
 
   explicit nsRefPtrHashKey(const T* aKey) : mKey(const_cast<T*>(aKey)) {}
-  nsRefPtrHashKey(nsRefPtrHashKey&& aOther)
-      : PLDHashEntryHdr(std::move(aOther)), mKey(std::move(aOther.mKey)) {}
+  nsRefPtrHashKey(nsRefPtrHashKey&& aOther) = default;
   ~nsRefPtrHashKey() = default;
 
   KeyType GetKey() const { return mKey; }
@@ -346,7 +384,7 @@ class nsRefPtrHashKey : public PLDHashEntryHdr {
 
   static KeyTypePointer KeyToPointer(KeyType aKey) { return aKey; }
   static PLDHashNumber HashKey(KeyTypePointer aKey) {
-    return NS_PTR_TO_UINT32(aKey) >> 2;
+    return mozilla::HashGeneric(aKey);
   }
   enum { ALLOW_MEMMOVE = true };
 
@@ -356,30 +394,11 @@ class nsRefPtrHashKey : public PLDHashEntryHdr {
 
 template <class T>
 inline void ImplCycleCollectionTraverse(
-    nsCycleCollectionTraversalCallback& aCallback, nsRefPtrHashKey<T>& aField,
-    const char* aName, uint32_t aFlags = 0) {
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsRefPtrHashKey<T>& aField, const char* aName, uint32_t aFlags = 0) {
   CycleCollectionNoteChild(aCallback, aField.GetKey(), aName, aFlags);
 }
 
-/**
- * hashkey wrapper using T* KeyType that sets key to nullptr upon
- * destruction. Relevant only in cases where a memory pointer-scanner
- * like valgrind might get confused about stale references.
- *
- * @see nsTHashtable::EntryType for specification
- */
-
-template <class T>
-class nsClearingPtrHashKey : public nsPtrHashKey<T> {
- public:
-  explicit nsClearingPtrHashKey(const T* aKey) : nsPtrHashKey<T>(aKey) {}
-  nsClearingPtrHashKey(nsClearingPtrHashKey&& aToMove)
-      : nsPtrHashKey<T>(std::move(aToMove)) {}
-  ~nsClearingPtrHashKey() { nsPtrHashKey<T>::mKey = nullptr; }
-};
-
-typedef nsClearingPtrHashKey<const void> nsClearingVoidPtrHashKey;
-
 /**
  * hashkey wrapper using a function pointer KeyType
  *
@@ -400,7 +419,7 @@ class nsFuncPtrHashKey : public PLDHashEntryHdr {
 
   static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
   static PLDHashNumber HashKey(KeyTypePointer aKey) {
-    return NS_PTR_TO_UINT32(*aKey) >> 2;
+    return mozilla::HashGeneric(*aKey);
   }
   enum { ALLOW_MEMMOVE = true };
 
@@ -408,6 +427,12 @@ class nsFuncPtrHashKey : public PLDHashEntryHdr {
   T mKey;
 };
 
+template <class T>
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsFuncPtrHashKey<T>& aField, const char* aName, uint32_t aFlags = 0) {
+}
+
 /**
  * hashkey wrapper using nsID KeyType
  *
@@ -438,6 +463,10 @@ class nsIDHashKey : public PLDHashEntryHdr {
   nsID mID;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback, const nsIDHashKey& aField,
+    const char* aName, uint32_t aFlags = 0) {}
+
 /**
  * hashkey wrapper using nsID* KeyType
  *
@@ -468,6 +497,10 @@ class nsIDPointerHashKey : public PLDHashEntryHdr {
   const nsID* mID;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsIDPointerHashKey& aField, const char* aName, uint32_t aFlags = 0) {}
+
 /**
  * hashkey wrapper for "dependent" const char*; this class does not "own"
  * its string pointer.
@@ -501,6 +534,10 @@ class nsDepCharHashKey : public PLDHashEntryHdr {
   const char* mKey;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsDepCharHashKey& aField, const char* aName, uint32_t aFlags = 0) {}
+
 /**
  * hashkey wrapper for const char*; at construction, this class duplicates
  * a string pointed to by the pointer so that it doesn't matter whether or not
@@ -543,6 +580,10 @@ class nsCharPtrHashKey : public PLDHashEntryHdr {
   const char* mKey;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsCharPtrHashKey& aField, const char* aName, uint32_t aFlags = 0) {}
+
 /**
  * hashkey wrapper for const char16_t*; at construction, this class duplicates
  * a string pointed to by the pointer so that it doesn't matter whether or not
@@ -584,6 +625,11 @@ class nsUnicharPtrHashKey : public PLDHashEntryHdr {
   const char16_t* mKey;
 };
 
+inline void ImplCycleCollectionTraverse(
+    nsCycleCollectionTraversalCallback& aCallback,
+    const nsUnicharPtrHashKey& aField, const char* aName, uint32_t aFlags = 0) {
+}
+
 namespace mozilla {
 
 template <typename T>
@@ -621,4 +667,15 @@ class nsGenericHashKey : public PLDHashEntryHdr {
   T mKey;
 };
 
+// If you have a key that can be memmoved, but a value that can't, you
+// can use this class to force ALLOW_MEMMOVE to be false. Ideally, nsTHashMap
+// would somehow consider both key and value, but until then, there is this
+// workaround.
+template <class Key>
+class NoMemMoveKey : public Key {
+ public:
+  explicit NoMemMoveKey(typename Key::KeyTypePointer aKey) : Key(aKey) {}
+  enum { ALLOW_MEMMOVE = false };
+};
+
 #endif  // nsTHashKeys_h__
