@@ -4,7 +4,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include <string.h>
 #include "nsTArray.h"
 #include "nsXPCOM.h"
 #include "nsCycleCollectionNoteChild.h"
@@ -22,14 +21,3 @@ bool IsTwiceTheRequiredBytesRepresentableAsUint32(size_t aCapacity,
   using mozilla::CheckedUint32;
   return ((CheckedUint32(aCapacity) * aElemSize) * 2).isValid();
 }
-
-MOZ_NORETURN MOZ_COLD void InvalidArrayIndex_CRASH(size_t aIndex,
-                                                   size_t aLength) {
-  MOZ_CRASH_UNSAFE_PRINTF(
-      "ElementAt(aIndex = %" PRIu64 ", aLength = %" PRIu64 ")",
-      static_cast<uint64_t>(aIndex), static_cast<uint64_t>(aLength));
-}
-
-void ::detail::SetCycleCollectionArrayFlag(uint32_t& aFlags) {
-  aFlags |= CycleCollectionEdgeNameArrayFlag;
-}
