@@ -94,6 +94,10 @@ FileDescriptorFile::GetNativeLeafName(nsACString& aLeafName) {
   return mFile->GetNativeLeafName(aLeafName);
 }
 
+NS_IMETHODIMP FileDescriptorFile::GetDisplayName(nsAString& aLeafName) {
+  return mFile->GetDisplayName(aLeafName);
+}
+
 nsresult FileDescriptorFile::GetTarget(nsAString& aRetVal) {
   return mFile->GetTarget(aRetVal);
 }
@@ -160,6 +164,11 @@ FileDescriptorFile::SetNativeLeafName(const nsACString& aLeafName) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+FileDescriptorFile::HostPath(JSContext* aCx, dom::Promise** aPromise) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 nsresult FileDescriptorFile::InitWithPath(const nsAString& aPath) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -270,7 +279,9 @@ FileDescriptorFile::RenameToNative(nsIFile* aNewParentDir,
 }
 
 NS_IMETHODIMP
-FileDescriptorFile::Remove(bool aRecursive) { return NS_ERROR_NOT_IMPLEMENTED; }
+FileDescriptorFile::Remove(bool aRecursive, uint32_t* aRemoveCount) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
 
 NS_IMETHODIMP
 FileDescriptorFile::GetPermissions(uint32_t* aPermissions) {
@@ -292,6 +303,26 @@ FileDescriptorFile::SetPermissionsOfLink(uint32_t aPermissions) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+FileDescriptorFile::GetLastAccessedTime(PRTime* aLastAccessedTime) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+FileDescriptorFile::SetLastAccessedTime(PRTime aLastAccessedTime) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+FileDescriptorFile::GetLastAccessedTimeOfLink(PRTime* aLastAccessedTime) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+FileDescriptorFile::SetLastAccessedTimeOfLink(PRTime aLastAccessedTime) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP
 FileDescriptorFile::GetLastModifiedTime(PRTime* aLastModTime) {
   return NS_ERROR_NOT_IMPLEMENTED;
@@ -401,6 +432,11 @@ FileDescriptorFile::GetDiskSpaceAvailable(int64_t* aDiskSpaceAvailable) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+FileDescriptorFile::GetDiskCapacity(int64_t* aDiskCapacity) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP
 FileDescriptorFile::Reveal() { return NS_ERROR_NOT_IMPLEMENTED; }
 
