@@ -152,7 +152,7 @@ nsresult NS_OpenAnonymousTemporaryFile(PRFileDesc** aOutFileDesc) {
 // idle observer and its timer on shutdown. Note: the observer and idle
 // services hold references to instances of this object, and those references
 // are what keep this object alive.
-class nsAnonTempFileRemover final : public nsIObserver {
+class nsAnonTempFileRemover final : public nsIObserver, public nsINamed {
  public:
   NS_DECL_ISUPPORTS
 
@@ -212,6 +212,11 @@ class nsAnonTempFileRemover final : public nsIObserver {
     return NS_OK;
   }
 
+  NS_IMETHODIMP GetName(nsACString& aName) {
+    aName.AssignLiteral("nsAnonTempFileRemover");
+    return NS_OK;
+  }
+
   nsresult RegisterIdleObserver() {
     // Add this as an idle observer. When we've been idle for
     // TEMP_FILE_IDLE_TIME_S seconds, we'll get a notification, and we'll then
@@ -241,7 +246,7 @@ class nsAnonTempFileRemover final : public nsIObserver {
   nsCOMPtr<nsITimer> mTimer;
 };
 
-NS_IMPL_ISUPPORTS(nsAnonTempFileRemover, nsIObserver)
+NS_IMPL_ISUPPORTS(nsAnonTempFileRemover, nsIObserver, nsINamed)
 
 nsresult CreateAnonTempFileRemover() {
   // Create a temp file remover. If Init() succeeds, the temp file remover is
