@@ -12,14 +12,9 @@
 #define _nsLocalFileUNIX_H_
 
 #include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
 
 #include "nscore.h"
 #include "nsString.h"
-#include "nsReadableUtils.h"
-#include "nsIClassInfoImpl.h"
-#include "mozilla/Attributes.h"
 #ifdef MOZ_WIDGET_COCOA
 #  include "nsILocalFileMac.h"
 #endif
@@ -27,11 +22,6 @@
 // stat64 and lstat64 are deprecated on OS X. Normal stat and lstat are
 // 64-bit by default on OS X 10.6+.
 #if defined(HAVE_STAT64) && defined(HAVE_LSTAT64) && !defined(XP_DARWIN)
-#  if defined(AIX)
-#    if defined STAT
-#      undef STAT
-#    endif
-#  endif
 #  define STAT stat64
 #  define LSTAT lstat64
 #  define HAVE_STATS64 1
@@ -40,6 +30,10 @@
 #  define LSTAT lstat
 #endif
 
+#if defined(HAVE_SYS_QUOTA_H) && defined(HAVE_LINUX_QUOTA_H)
+#  define USE_LINUX_QUOTACTL
+#endif
+
 class nsLocalFile final
 #ifdef MOZ_WIDGET_COCOA
     : public nsILocalFileMac
@@ -51,9 +45,8 @@ class nsLocalFile final
   NS_DEFINE_STATIC_CID_ACCESSOR(NS_LOCAL_FILE_CID)
 
   nsLocalFile();
-  explicit nsLocalFile(const nsACString& aFilePath);
 
-  static nsresult nsLocalFileConstructor(nsISupports* aOuter, const nsIID& aIID,
+  static nsresult nsLocalFileConstructor(const nsIID& aIID,
                                          void** aInstancePtr);
 
   NS_DECL_THREADSAFE_ISUPPORTS
@@ -86,9 +79,20 @@ class nsLocalFile final
   nsresult CreateAndKeepOpen(uint32_t aType, int aFlags, uint32_t aPermissions,
                              bool aSkipAncestors, PRFileDesc** aResult);
 
-  nsresult SetLastModifiedTimeImpl(PRTime aLastModTime, bool aFollowLinks);
-  nsresult GetLastModifiedTimeImpl(PRTime* aLastModTime, bool aFollowLinks);
+  enum class TimeField { AccessedTime, ModifiedTime };
+  nsresult SetTimeImpl(PRTime aTime, TimeField aTimeField, bool aFollowLinks);
+  nsresult GetTimeImpl(PRTime* aTime, TimeField aTimeField, bool aFollowLinks);
+
   nsresult GetCreationTimeImpl(PRTime* aCreationTime, bool aFollowLinks);
+
+#if defined(USE_LINUX_QUOTACTL)
+  template <typename StatInfoFunc, typename QuotaInfoFunc>
+  nsresult GetDiskInfo(StatInfoFunc&& aStatInfoFunc,
+                       QuotaInfoFunc&& aQuotaInfoFunc, int64_t* aResult);
+#else
+  template <typename StatInfoFunc>
+  nsresult GetDiskInfo(StatInfoFunc&& aStatInfoFunc, int64_t* aResult);
+#endif
 };
 
 #endif /* _nsLocalFileUNIX_H_ */
