@@ -36,14 +36,14 @@ PrepareAndDispatch(nsXPTCStubBase* self, uint32_t methodIndex,
   if (! info)
       return NS_ERROR_UNEXPECTED;
 
-  paramCount = info->GetParamCount();
+  paramCount = info->ParamCount();
 
   const uint8_t indexOfJSContext = info->IndexOfJSContext();
 
   for(i = 0; i < paramCount; ++i)
   {
     int isfloat = 0;
-    const nsXPTParamInfo& param = info->GetParam(i);
+    const nsXPTParamInfo& param = info->Param(i);
     const nsXPTType& type = param.GetType();
     nsXPTCMiniVariant* dp = &paramBuffer[i];
 
