@@ -36,6 +36,8 @@ struct nsXPTMethodInfo;
 struct nsXPTConstantInfo;
 struct nsXPTDOMObjectInfo;
 
+enum class nsXPTInterface : uint16_t;
+
 // Internal helper methods.
 namespace xpt {
 namespace detail {
@@ -69,6 +71,10 @@ struct nsXPTInterfaceInfo {
     return xpt::detail::InterfaceByName(aName);
   }
 
+  static const nsXPTInterfaceInfo* Get(nsXPTInterface aID) {
+    return ByIndex(uint16_t(aID));
+  }
+
   // These are only needed for Components_interfaces's enumerator.
   static const nsXPTInterfaceInfo* ByIndex(uint16_t aIndex) {
     // NOTE: We add 1 here, as the internal index 0 is reserved for null.
@@ -101,7 +107,7 @@ struct nsXPTInterfaceInfo {
   const nsXPTMethodInfo& Method(uint16_t aIndex) const;
 
   nsresult GetMethodInfo(uint16_t aIndex, const nsXPTMethodInfo** aInfo) const;
-  nsresult GetConstant(uint16_t aIndex, JS::MutableHandleValue constant,
+  nsresult GetConstant(uint16_t aIndex, JS::MutableHandle<JS::Value> constant,
                        char** aName) const;
 
   ////////////////////////////////////////////////////////////////
@@ -471,15 +477,6 @@ struct nsXPTMethodInfo {
 
   bool GetId(JSContext* aCx, jsid& aId) const;
 
-  /////////////////////////////////////////////
-  // nsXPTMethodInfo backwards compatibility //
-  /////////////////////////////////////////////
-
-  const char* GetName() const { return Name(); }
-
-  uint8_t GetParamCount() const { return ParamCount(); }
-  const nsXPTParamInfo& GetParam(uint8_t aIndex) const { return Param(aIndex); }
-
   ////////////////////////////////////////////////////////////////
   // Ensure these fields are in the same order as xptcodegen.py //
   ////////////////////////////////////////////////////////////////
@@ -500,6 +497,17 @@ struct nsXPTMethodInfo {
 // The fields in nsXPTMethodInfo were carefully ordered to minimize size.
 static_assert(sizeof(nsXPTMethodInfo) == 8, "wrong size");
 
+// This number is chosen to be no larger than the maximum number of parameters
+// any XPIDL-defined function needs; there is a static assert in the generated
+// code from xptcodegen.py to verify that decision.  It is therefore also the
+// maximum number of stack allocated nsXPTCMiniVariant structures for argument
+// passing purposes in PrepareAndDispatch implementations.
+#if defined(MOZ_THUNDERBIRD) || defined(MOZ_SUITE)
+#  define PARAM_BUFFER_COUNT 18
+#else
+#  define PARAM_BUFFER_COUNT 14
+#endif
+
 /**
  * A nsXPTConstantInfo is used to describe a single interface constant.
  */
@@ -533,11 +541,13 @@ static_assert(sizeof(nsXPTConstantInfo) == 8, "wrong size");
  * This object will not live in rodata as it contains relocations.
  */
 struct nsXPTDOMObjectInfo {
-  nsresult Unwrap(JS::HandleValue aHandle, void** aObj, JSContext* aCx) const {
+  nsresult Unwrap(JS::Handle<JS::Value> aHandle, void** aObj,
+                  JSContext* aCx) const {
     return mUnwrap(aHandle, aObj, aCx);
   }
 
-  bool Wrap(JSContext* aCx, void* aObj, JS::MutableHandleValue aHandle) const {
+  bool Wrap(JSContext* aCx, void* aObj,
+            JS::MutableHandle<JS::Value> aHandle) const {
     return mWrap(aCx, aObj, aHandle);
   }
 
@@ -547,8 +557,10 @@ struct nsXPTDOMObjectInfo {
   // Ensure these fields are in the same order as xptcodegen.py //
   ////////////////////////////////////////////////////////////////
 
-  nsresult (*mUnwrap)(JS::HandleValue aHandle, void** aObj, JSContext* aCx);
-  bool (*mWrap)(JSContext* aCx, void* aObj, JS::MutableHandleValue aHandle);
+  nsresult (*mUnwrap)(JS::Handle<JS::Value> aHandle, void** aObj,
+                      JSContext* aCx);
+  bool (*mWrap)(JSContext* aCx, void* aObj,
+                JS::MutableHandle<JS::Value> aHandle);
   void (*mCleanup)(void* aObj);
 };
 
