@@ -5,9 +5,11 @@
 #include <type_traits>
 
 #include "nsComponentManagerUtils.h"
+#include "nsIThread.h"
 #include "nsThreadUtils.h"
 #include "mozilla/IdleTaskRunner.h"
 #include "mozilla/RefCounted.h"
+#include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/UniquePtr.h"
 
 #include "gtest/gtest.h"
@@ -431,7 +433,9 @@ static void TestRunnableFactory(bool aNamed) {
 }
 
 TEST(ThreadUtils, NewRunnableFunction)
-{ TestRunnableFactory<BasicRunnableFactory>(/*aNamed*/ false); }
+{
+  TestRunnableFactory<BasicRunnableFactory>(/*aNamed*/ false);
+}
 
 TEST(ThreadUtils, NewNamedRunnableFunction)
 {
@@ -448,7 +452,9 @@ TEST(ThreadUtils, NewNamedRunnableFunction)
 }
 
 TEST(ThreadUtils, NewCancelableRunnableFunction)
-{ TestRunnableFactory<CancelableRunnableFactory>(/*aNamed*/ false); }
+{
+  TestRunnableFactory<CancelableRunnableFactory>(/*aNamed*/ false);
+}
 
 TEST(ThreadUtils, NewNamedCancelableRunnableFunction)
 {
@@ -477,6 +483,21 @@ TEST(ThreadUtils, NewNamedCancelableRunnableFunction)
     EXPECT_EQ(foo->refCount(), 1u);
     EXPECT_FALSE(ran);
   }
+
+  // Test no-op after cancelation.
+  {
+    auto foo = MakeRefPtr<TestRefCounted>();
+    bool ran = false;
+
+    RefPtr<CancelableRunnable> func =
+        NS_NewCancelableRunnableFunction("unused", [foo, &ran] { ran = true; });
+
+    EXPECT_EQ(foo->refCount(), 2u);
+    func->Cancel();
+    func->Run();
+
+    EXPECT_FALSE(ran);
+  }
 }
 
 static void TestNewRunnableMethod(bool aNamed) {
@@ -495,6 +516,15 @@ static void TestNewRunnableMethod(bool aNamed) {
     // Read only string. Dereferencing in runnable method to check this works.
     char* message = (char*)"Test message";
 
+    {
+      auto bar = MakeRefPtr<nsBar>();
+
+      NS_DispatchToMainThread(
+          aNamed ? NewRunnableMethod("unused", std::move(bar), &nsBar::DoBar1)
+                 : NewRunnableMethod("nsBar::DoBar1", std::move(bar),
+                                     &nsBar::DoBar1));
+    }
+
     NS_DispatchToMainThread(
         aNamed ? NewRunnableMethod("unused", bar, &nsBar::DoBar1)
                : NewRunnableMethod("nsBar::DoBar1", bar, &nsBar::DoBar1));
@@ -569,7 +599,9 @@ static void TestNewRunnableMethod(bool aNamed) {
 }
 
 TEST(ThreadUtils, RunnableMethod)
-{ TestNewRunnableMethod(/* aNamed */ false); }
+{
+  TestNewRunnableMethod(/* aNamed */ false);
+}
 
 TEST(ThreadUtils, NamedRunnableMethod)
 {
@@ -621,7 +653,7 @@ class IdleObject final {
  public:
   NS_INLINE_DECL_REFCOUNTING(IdleObject)
   IdleObject() {
-    for (uint32_t index = 0; index < ArrayLength(mRunnableExecuted); ++index) {
+    for (uint32_t index = 0; index < std::size(mRunnableExecuted); ++index) {
       mRunnableExecuted[index] = false;
       mSetIdleDeadlineCalled = false;
     }
@@ -635,7 +667,7 @@ class IdleObject final {
       << aKey << ": Method" << index << " should've executed";
     }
 
-    for (; index < ArrayLength(mRunnableExecuted); ++index) {
+    for (; index < std::size(mRunnableExecuted); ++index) {
       ASSERT_FALSE(mRunnableExecuted[index])
       << aKey << ": Method" << index << " shouldn't have executed";
     }
@@ -715,6 +747,9 @@ class IdleObject final {
   ~IdleObject() = default;
 };
 
+// Disable test due to frequent failures
+#if 0
+// because test fails on multiple platforms
 TEST(ThreadUtils, IdleRunnableMethod)
 {
   {
@@ -761,6 +796,7 @@ TEST(ThreadUtils, IdleRunnableMethod)
     ASSERT_TRUE(idleInheritedSetDeadline->mSetDeadlineCalled);
   }
 }
+#endif
 
 TEST(ThreadUtils, IdleTaskRunner)
 {
@@ -773,7 +809,8 @@ TEST(ThreadUtils, IdleTaskRunner)
         cnt1++;
         return true;
       },
-      "runner1", 10, 3, true, nullptr);
+      "runner1", 0, TimeDuration::FromMilliseconds(10),
+      TimeDuration::FromMilliseconds(3), true, nullptr);
 
   // Non-repeating but callback always return false so it's still repeating.
   int cnt2 = 0;
@@ -782,7 +819,8 @@ TEST(ThreadUtils, IdleTaskRunner)
         cnt2++;
         return false;
       },
-      "runner2", 10, 3, false, nullptr);
+      "runner2", 0, TimeDuration::FromMilliseconds(10),
+      TimeDuration::FromMilliseconds(3), false, nullptr);
 
   // Repeating until cnt3 >= 2 by returning 'true' in MayStopProcessing
   // callback. The strategy is to stop repeating as early as possible so that we
@@ -793,7 +831,8 @@ TEST(ThreadUtils, IdleTaskRunner)
         cnt3++;
         return true;
       },
-      "runner3", 10, 3, true, [&cnt3] { return cnt3 >= 2; });
+      "runner3", 0, TimeDuration::FromMilliseconds(10),
+      TimeDuration::FromMilliseconds(3), true, [&cnt3] { return cnt3 >= 2; });
 
   // Non-repeating can callback return true so the callback will
   // be only run once.
@@ -803,32 +842,39 @@ TEST(ThreadUtils, IdleTaskRunner)
         cnt4++;
         return true;
       },
-      "runner4", 10, 3, false, nullptr);
+      "runner4", 0, TimeDuration::FromMilliseconds(10),
+      TimeDuration::FromMilliseconds(3), false, nullptr);
 
   // Firstly we wait until the two repeating tasks reach their limits.
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() { return cnt1 >= 100; }));
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() { return cnt2 >= 100; }));
+  MOZ_ALWAYS_TRUE(
+      SpinEventLoopUntil("xpcom:TEST(ThreadUtils, IdleTaskRunner) cnt1"_ns,
+                         [&]() { return cnt1 >= 100; }));
+  MOZ_ALWAYS_TRUE(
+      SpinEventLoopUntil("xpcom:TEST(ThreadUtils, IdleTaskRunner) cnt2"_ns,
+                         [&]() { return cnt2 >= 100; }));
 
   // At any point ==> 0 <= cnt3 <= 2 since MayStopProcessing() would return
   // true when cnt3 >= 2.
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() {
-    if (cnt3 > 2) {
-      EXPECT_TRUE(false) << "MaybeContinueProcess() doesn't work.";
-      return true;  // Stop on failure.
-    }
-    return cnt3 == 2;  // Stop finish if we have reached its max value.
-  }));
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "xpcom:TEST(ThreadUtils, IdleTaskRunner) cnt3"_ns, [&]() {
+        if (cnt3 > 2) {
+          EXPECT_TRUE(false) << "MaybeContinueProcess() doesn't work.";
+          return true;  // Stop on failure.
+        }
+        return cnt3 == 2;  // Stop finish if we have reached its max value.
+      }));
 
   // At any point ==> 0 <= cnt4 <= 1 since this is a non-repeating
   // idle runner.
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() {
-    // At any point: 0 <= cnt4 <= 1
-    if (cnt4 > 1) {
-      EXPECT_TRUE(false) << "The 'mRepeating' flag doesn't work.";
-      return true;  // Stop on failure.
-    }
-    return cnt4 == 1;
-  }));
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "xpcom:TEST(ThreadUtils, IdleTaskRunner) cnt4"_ns, [&]() {
+        // At any point: 0 <= cnt4 <= 1
+        if (cnt4 > 1) {
+          EXPECT_TRUE(false) << "The 'mRepeating' flag doesn't work.";
+          return true;  // Stop on failure.
+        }
+        return cnt4 == 1;
+      }));
 
   // The repeating timers require an explicit Cancel() call.
   runner1->Cancel();
@@ -836,131 +882,93 @@ TEST(ThreadUtils, IdleTaskRunner)
 }
 
 // {9e70a320-be02-11d1-8031-006008159b5a}
-#define NS_IFOO_IID                                  \
-  {                                                  \
-    0x9e70a320, 0xbe02, 0x11d1, {                    \
-      0x80, 0x31, 0x00, 0x60, 0x08, 0x15, 0x9b, 0x5a \
-    }                                                \
-  }
+#define NS_IFOO_IID \
+  {0x9e70a320, 0xbe02, 0x11d1, {0x80, 0x31, 0x00, 0x60, 0x08, 0x15, 0x9b, 0x5a}}
 
 TEST(ThreadUtils, TypeTraits)
 {
-  static_assert(!mozilla::IsRefcountedSmartPointer<int>::value,
-                "IsRefcountedSmartPointer<int> should be false");
-  static_assert(mozilla::IsRefcountedSmartPointer<RefPtr<int>>::value,
-                "IsRefcountedSmartPointer<RefPtr<...>> should be true");
-  static_assert(mozilla::IsRefcountedSmartPointer<const RefPtr<int>>::value,
-                "IsRefcountedSmartPointer<const RefPtr<...>> should be true");
+  static_assert(std::is_same_v<int, mozilla::RemoveSmartPointer<int>>,
+                "RemoveSmartPointer<int> should be int");
+  static_assert(std::is_same_v<int*, mozilla::RemoveSmartPointer<int*>>,
+                "RemoveSmartPointer<int*> should be int*");
+  static_assert(std::is_same_v<UniquePtr<int>,
+                               mozilla::RemoveSmartPointer<UniquePtr<int>>>,
+                "RemoveSmartPointer<UniquePtr<int>> should be UniquePtr<int>");
+  static_assert(std::is_same_v<int, mozilla::RemoveSmartPointer<RefPtr<int>>>,
+                "RemoveSmartPointer<RefPtr<int>> should be int");
   static_assert(
-      mozilla::IsRefcountedSmartPointer<volatile RefPtr<int>>::value,
-      "IsRefcountedSmartPointer<volatile RefPtr<...>> should be true");
+      std::is_same_v<int, mozilla::RemoveSmartPointer<const RefPtr<int>>>,
+      "RemoveSmartPointer<const RefPtr<int>> should be int");
   static_assert(
-      mozilla::IsRefcountedSmartPointer<const volatile RefPtr<int>>::value,
-      "IsRefcountedSmartPointer<const volatile RefPtr<...>> should be true");
-  static_assert(mozilla::IsRefcountedSmartPointer<nsCOMPtr<int>>::value,
-                "IsRefcountedSmartPointer<nsCOMPtr<...>> should be true");
-  static_assert(mozilla::IsRefcountedSmartPointer<const nsCOMPtr<int>>::value,
-                "IsRefcountedSmartPointer<const nsCOMPtr<...>> should be true");
+      std::is_same_v<int, mozilla::RemoveSmartPointer<volatile RefPtr<int>>>,
+      "RemoveSmartPointer<volatile RefPtr<int>> should be int");
   static_assert(
-      mozilla::IsRefcountedSmartPointer<volatile nsCOMPtr<int>>::value,
-      "IsRefcountedSmartPointer<volatile nsCOMPtr<...>> should be true");
+      std::is_same_v<int,
+                     mozilla::RemoveSmartPointer<const volatile RefPtr<int>>>,
+      "RemoveSmartPointer<const volatile RefPtr<int>> should be int");
+  static_assert(std::is_same_v<int, mozilla::RemoveSmartPointer<nsCOMPtr<int>>>,
+                "RemoveSmartPointer<nsCOMPtr<int>> should be int");
   static_assert(
-      mozilla::IsRefcountedSmartPointer<const volatile nsCOMPtr<int>>::value,
-      "IsRefcountedSmartPointer<const volatile nsCOMPtr<...>> should be true");
+      std::is_same_v<int, mozilla::RemoveSmartPointer<const nsCOMPtr<int>>>,
+      "RemoveSmartPointer<const nsCOMPtr<int>> should be int");
+  static_assert(
+      std::is_same_v<int, mozilla::RemoveSmartPointer<volatile nsCOMPtr<int>>>,
+      "RemoveSmartPointer<volatile nsCOMPtr<int>> should be int");
+  static_assert(
+      std::is_same_v<int,
+                     mozilla::RemoveSmartPointer<const volatile nsCOMPtr<int>>>,
+      "RemoveSmartPointer<const volatile nsCOMPtr<int>> should be int");
 
-  static_assert(std::is_same_v<int, mozilla::RemoveSmartPointer<int>::Type>,
-                "RemoveSmartPointer<int>::Type should be int");
-  static_assert(std::is_same_v<int*, mozilla::RemoveSmartPointer<int*>::Type>,
-                "RemoveSmartPointer<int*>::Type should be int*");
+  static_assert(std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<int>>,
+                "RemoveRawOrSmartPointer<int> should be int");
   static_assert(
       std::is_same_v<UniquePtr<int>,
-                     mozilla::RemoveSmartPointer<UniquePtr<int>>::Type>,
-      "RemoveSmartPointer<UniquePtr<int>>::Type should be UniquePtr<int>");
+                     mozilla::RemoveRawOrSmartPointer<UniquePtr<int>>>,
+      "RemoveRawOrSmartPointer<UniquePtr<int>> should be UniquePtr<int>");
+  static_assert(std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<int*>>,
+                "RemoveRawOrSmartPointer<int*> should be int");
   static_assert(
-      std::is_same_v<int, mozilla::RemoveSmartPointer<RefPtr<int>>::Type>,
-      "RemoveSmartPointer<RefPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int, mozilla::RemoveSmartPointer<const RefPtr<int>>::Type>,
-      "RemoveSmartPointer<const RefPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int,
-                     mozilla::RemoveSmartPointer<volatile RefPtr<int>>::Type>,
-      "RemoveSmartPointer<volatile RefPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<
-          int, mozilla::RemoveSmartPointer<const volatile RefPtr<int>>::Type>,
-      "RemoveSmartPointer<const volatile RefPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int, mozilla::RemoveSmartPointer<nsCOMPtr<int>>::Type>,
-      "RemoveSmartPointer<nsCOMPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int,
-                     mozilla::RemoveSmartPointer<const nsCOMPtr<int>>::Type>,
-      "RemoveSmartPointer<const nsCOMPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int,
-                     mozilla::RemoveSmartPointer<volatile nsCOMPtr<int>>::Type>,
-      "RemoveSmartPointer<volatile nsCOMPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<
-          int, mozilla::RemoveSmartPointer<const volatile nsCOMPtr<int>>::Type>,
-      "RemoveSmartPointer<const volatile nsCOMPtr<int>>::Type should be int");
-
-  static_assert(
-      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<int>::Type>,
-      "RemoveRawOrSmartPointer<int>::Type should be int");
-  static_assert(
-      std::is_same_v<UniquePtr<int>,
-                     mozilla::RemoveRawOrSmartPointer<UniquePtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<UniquePtr<int>>::Type should be UniquePtr<int>");
-  static_assert(
-      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<int*>::Type>,
-      "RemoveRawOrSmartPointer<int*>::Type should be int");
-  static_assert(
-      std::is_same_v<const int,
-                     mozilla::RemoveRawOrSmartPointer<const int*>::Type>,
-      "RemoveRawOrSmartPointer<const int*>::Type should be const int");
+      std::is_same_v<const int, mozilla::RemoveRawOrSmartPointer<const int*>>,
+      "RemoveRawOrSmartPointer<const int*> should be const int");
   static_assert(
       std::is_same_v<volatile int,
-                     mozilla::RemoveRawOrSmartPointer<volatile int*>::Type>,
-      "RemoveRawOrSmartPointer<volatile int*>::Type should be volatile int");
+                     mozilla::RemoveRawOrSmartPointer<volatile int*>>,
+      "RemoveRawOrSmartPointer<volatile int*> should be volatile int");
   static_assert(
-      std::is_same_v<const volatile int, mozilla::RemoveRawOrSmartPointer<
-                                             const volatile int*>::Type>,
-      "RemoveRawOrSmartPointer<const volatile int*>::Type should be const "
+      std::is_same_v<const volatile int,
+                     mozilla::RemoveRawOrSmartPointer<const volatile int*>>,
+      "RemoveRawOrSmartPointer<const volatile int*> should be const "
       "volatile int");
   static_assert(
-      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<RefPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<RefPtr<int>>::Type should be int");
+      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<RefPtr<int>>>,
+      "RemoveRawOrSmartPointer<RefPtr<int>> should be int");
+  static_assert(
+      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<const RefPtr<int>>>,
+      "RemoveRawOrSmartPointer<const RefPtr<int>> should be int");
   static_assert(
       std::is_same_v<int,
-                     mozilla::RemoveRawOrSmartPointer<const RefPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<const RefPtr<int>>::Type should be int");
+                     mozilla::RemoveRawOrSmartPointer<volatile RefPtr<int>>>,
+      "RemoveRawOrSmartPointer<volatile RefPtr<int>> should be int");
   static_assert(
       std::is_same_v<
-          int, mozilla::RemoveRawOrSmartPointer<volatile RefPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<volatile RefPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<
-                              const volatile RefPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<const volatile RefPtr<int>>::Type should be "
+          int, mozilla::RemoveRawOrSmartPointer<const volatile RefPtr<int>>>,
+      "RemoveRawOrSmartPointer<const volatile RefPtr<int>> should be "
       "int");
+  static_assert(
+      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<nsCOMPtr<int>>>,
+      "RemoveRawOrSmartPointer<nsCOMPtr<int>> should be int");
   static_assert(
       std::is_same_v<int,
-                     mozilla::RemoveRawOrSmartPointer<nsCOMPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<nsCOMPtr<int>>::Type should be int");
+                     mozilla::RemoveRawOrSmartPointer<const nsCOMPtr<int>>>,
+      "RemoveRawOrSmartPointer<const nsCOMPtr<int>> should be int");
+  static_assert(
+      std::is_same_v<int,
+                     mozilla::RemoveRawOrSmartPointer<volatile nsCOMPtr<int>>>,
+      "RemoveRawOrSmartPointer<volatile nsCOMPtr<int>> should be int");
   static_assert(
       std::is_same_v<
-          int, mozilla::RemoveRawOrSmartPointer<const nsCOMPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<const nsCOMPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<
-          int, mozilla::RemoveRawOrSmartPointer<volatile nsCOMPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<volatile nsCOMPtr<int>>::Type should be int");
-  static_assert(
-      std::is_same_v<int, mozilla::RemoveRawOrSmartPointer<
-                              const volatile nsCOMPtr<int>>::Type>,
-      "RemoveRawOrSmartPointer<const volatile nsCOMPtr<int>>::Type should be "
+          int, mozilla::RemoveRawOrSmartPointer<const volatile nsCOMPtr<int>>>,
+      "RemoveRawOrSmartPointer<const volatile nsCOMPtr<int>> should be "
       "int");
 }
 
@@ -1043,17 +1051,16 @@ struct Spy {
 };
 
 struct ISpyWithISupports : public nsISupports {
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IFOO_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_IFOO_IID)
   NS_IMETHOD_(nsrefcnt) RefCnt() = 0;
   NS_IMETHOD_(int32_t) ID() = 0;
 };
-NS_DEFINE_STATIC_IID_ACCESSOR(ISpyWithISupports, NS_IFOO_IID)
 struct SpyWithISupports : public ISpyWithISupports, public Spy {
  private:
   virtual ~SpyWithISupports() = default;
 
  public:
-  explicit SpyWithISupports(int aID) : Spy(aID){};
+  explicit SpyWithISupports(int aID) : Spy(aID) {};
   NS_DECL_ISUPPORTS
   NS_IMETHOD_(nsrefcnt) RefCnt() override { return mRefCnt; }
   NS_IMETHOD_(int32_t) ID() override { return mID; }
@@ -1062,14 +1069,12 @@ NS_IMPL_ISUPPORTS(SpyWithISupports, ISpyWithISupports)
 
 class IThreadUtilsObject : public nsISupports {
  public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IFOO_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_IFOO_IID)
 
   NS_IMETHOD_(nsrefcnt) RefCnt() = 0;
   NS_IMETHOD_(int32_t) ID() = 0;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(IThreadUtilsObject, NS_IFOO_IID)
-
 struct ThreadUtilsObjectNonRefCountedBase {
   virtual void MethodFromNonRefCountedBase() {}
 };
@@ -1248,15 +1253,9 @@ TEST(ThreadUtils, main)
 
   static_assert(!IsParameterStorageClass<int>::value,
                 "'int' should not be recognized as Storage Class");
-  static_assert(
-      IsParameterStorageClass<StoreCopyPassByValue<int>>::value,
-      "StoreCopyPassByValue<int> should be recognized as Storage Class");
   static_assert(
       IsParameterStorageClass<StoreCopyPassByConstLRef<int>>::value,
       "StoreCopyPassByConstLRef<int> should be recognized as Storage Class");
-  static_assert(
-      IsParameterStorageClass<StoreCopyPassByLRef<int>>::value,
-      "StoreCopyPassByLRef<int> should be recognized as Storage Class");
   static_assert(
       IsParameterStorageClass<StoreCopyPassByRRef<int>>::value,
       "StoreCopyPassByRRef<int> should be recognized as Storage Class");
@@ -1275,12 +1274,6 @@ TEST(ThreadUtils, main)
   static_assert(
       IsParameterStorageClass<StoreConstPtrPassByConstPtr<int>>::value,
       "StoreConstPtrPassByConstPtr<int> should be recognized as Storage Class");
-  static_assert(
-      IsParameterStorageClass<StoreCopyPassByConstPtr<int>>::value,
-      "StoreCopyPassByConstPtr<int> should be recognized as Storage Class");
-  static_assert(
-      IsParameterStorageClass<StoreCopyPassByPtr<int>>::value,
-      "StoreCopyPassByPtr<int> should be recognized as Storage Class");
 
   RefPtr<ThreadUtilsObject> rpt(new ThreadUtilsObject);
   int count = 0;
@@ -1319,11 +1312,6 @@ TEST(ThreadUtils, main)
                                StoreCopyPassByConstLRef<int>>,
                 "detail::ParameterStorage<int>::Type should be "
                 "StoreCopyPassByConstLRef<int>");
-  static_assert(std::is_same_v<
-                    ::detail::ParameterStorage<StoreCopyPassByValue<int>>::Type,
-                    StoreCopyPassByValue<int>>,
-                "detail::ParameterStorage<StoreCopyPassByValue<int>>::Type "
-                "should be StoreCopyPassByValue<int>");
 
   r1 = NewRunnableMethod<int>("TestThreadUtils::ThreadUtilsObject::Test1i", rpt,
                               &ThreadUtilsObject::Test1i, 12);
@@ -1440,37 +1428,6 @@ TEST(ThreadUtils, main)
     EXPECT_EQ(i, rpt->mA0);
   }
 
-  // Raw pointer to copy.
-  static_assert(std::is_same_v<StoreCopyPassByPtr<int>::stored_type, int>,
-                "StoreCopyPassByPtr<int>::stored_type should be int");
-  static_assert(std::is_same_v<StoreCopyPassByPtr<int>::passed_type, int*>,
-                "StoreCopyPassByPtr<int>::passed_type should be int*");
-  {
-    int i = 1202;
-    r1 = NewRunnableMethod<StoreCopyPassByPtr<int>>(
-        "TestThreadUtils::ThreadUtilsObject::Test1pi", rpt,
-        &ThreadUtilsObject::Test1pi, i);
-    r1->Run();
-    EXPECT_EQ(count += 2, rpt->mCount);
-    EXPECT_EQ(i, rpt->mA0);
-  }
-
-  // Raw pointer to const copy.
-  static_assert(std::is_same_v<StoreCopyPassByConstPtr<int>::stored_type, int>,
-                "StoreCopyPassByConstPtr<int>::stored_type should be int");
-  static_assert(
-      std::is_same_v<StoreCopyPassByConstPtr<int>::passed_type, const int*>,
-      "StoreCopyPassByConstPtr<int>::passed_type should be const int*");
-  {
-    int i = 1203;
-    r1 = NewRunnableMethod<StoreCopyPassByConstPtr<int>>(
-        "TestThreadUtils::ThreadUtilsObject::Test1pci", rpt,
-        &ThreadUtilsObject::Test1pci, i);
-    r1->Run();
-    EXPECT_EQ(count += 2, rpt->mCount);
-    EXPECT_EQ(i, rpt->mA0);
-  }
-
   // nsRefPtr to pointer.
   static_assert(
       std::is_same_v<::detail::ParameterStorage<
@@ -1496,7 +1453,7 @@ TEST(ThreadUtils, main)
   // (more nsRefPtr tests below)
 
   // nsRefPtr for ref-countable classes that do not derive from ISupports.
-  static_assert(::detail::HasRefCountMethods<ThreadUtilsRefCountedFinal>::value,
+  static_assert(::detail::HasRefCountMethods<ThreadUtilsRefCountedFinal>,
                 "ThreadUtilsRefCountedFinal has AddRef() and Release()");
   static_assert(
       std::is_same_v<
@@ -1504,7 +1461,7 @@ TEST(ThreadUtils, main)
           StoreRefPtrPassByPtr<ThreadUtilsRefCountedFinal>>,
       "ParameterStorage<ThreadUtilsRefCountedFinal*>::Type should be "
       "StoreRefPtrPassByPtr<ThreadUtilsRefCountedFinal>");
-  static_assert(::detail::HasRefCountMethods<ThreadUtilsRefCountedBase>::value,
+  static_assert(::detail::HasRefCountMethods<ThreadUtilsRefCountedBase>,
                 "ThreadUtilsRefCountedBase has AddRef() and Release()");
   static_assert(
       std::is_same_v<
@@ -1512,9 +1469,8 @@ TEST(ThreadUtils, main)
           StoreRefPtrPassByPtr<ThreadUtilsRefCountedBase>>,
       "ParameterStorage<ThreadUtilsRefCountedBase*>::Type should be "
       "StoreRefPtrPassByPtr<ThreadUtilsRefCountedBase>");
-  static_assert(
-      ::detail::HasRefCountMethods<ThreadUtilsRefCountedDerived>::value,
-      "ThreadUtilsRefCountedDerived has AddRef() and Release()");
+  static_assert(::detail::HasRefCountMethods<ThreadUtilsRefCountedDerived>,
+                "ThreadUtilsRefCountedDerived has AddRef() and Release()");
   static_assert(
       std::is_same_v<
           ::detail::ParameterStorage<ThreadUtilsRefCountedDerived*>::Type,
@@ -1522,7 +1478,7 @@ TEST(ThreadUtils, main)
       "ParameterStorage<ThreadUtilsRefCountedDerived*>::Type should be "
       "StoreRefPtrPassByPtr<ThreadUtilsRefCountedDerived>");
 
-  static_assert(!::detail::HasRefCountMethods<ThreadUtilsNonRefCounted>::value,
+  static_assert(!::detail::HasRefCountMethods<ThreadUtilsNonRefCounted>,
                 "ThreadUtilsNonRefCounted doesn't have AddRef() and Release()");
   static_assert(!std::is_same_v<
                     ::detail::ParameterStorage<ThreadUtilsNonRefCounted*>::Type,
@@ -1696,126 +1652,6 @@ TEST(ThreadUtils, main)
 
   // Verify copy/move assumptions.
 
-  Spy::ClearAll();
-  if (gDebug) {
-    printf("%d - Test: Store copy from lvalue, pass by value\n", __LINE__);
-  }
-  {  // Block around nsCOMPtr lifetime.
-    nsCOMPtr<nsIRunnable> r2;
-    {  // Block around Spy lifetime.
-      if (gDebug) {
-        printf("%d - Spy s(10)\n", __LINE__);
-      }
-      Spy s(10);
-      EXPECT_EQ(1, gConstructions);
-      EXPECT_EQ(1, gAlive);
-      if (gDebug) {
-        printf(
-            "%d - r2 = "
-            "NewRunnableMethod<StoreCopyPassByValue<Spy>>(&TestByValue, s)\n",
-            __LINE__);
-      }
-      r2 = NewRunnableMethod<StoreCopyPassByValue<Spy>>(
-          "TestThreadUtils::ThreadUtilsObject::TestByValue", rpt,
-          &ThreadUtilsObject::TestByValue, s);
-      EXPECT_EQ(2, gAlive);
-      EXPECT_LE(1, gCopyConstructions);  // At least 1 copy-construction.
-      Spy::ClearActions();
-      if (gDebug) {
-        printf("%d - End block with Spy s(10)\n", __LINE__);
-      }
-    }
-    EXPECT_EQ(1, gDestructions);
-    EXPECT_EQ(1, gAlive);
-    Spy::ClearActions();
-    if (gDebug) {
-      printf("%d - Run()\n", __LINE__);
-    }
-    r2->Run();
-    EXPECT_LE(1, gCopyConstructions);  // Another copy-construction in call.
-    EXPECT_EQ(10, rpt->mSpy.mID);
-    EXPECT_LE(1, gDestructions);
-    EXPECT_EQ(1, gAlive);
-    Spy::ClearActions();
-    if (gDebug) {
-      printf("%d - End block with r\n", __LINE__);
-    }
-  }
-  if (gDebug) {
-    printf("%d - After end block with r\n", __LINE__);
-  }
-  EXPECT_EQ(1, gDestructions);
-  EXPECT_EQ(0, gAlive);
-
-  Spy::ClearAll();
-  if (gDebug) {
-    printf("%d - Test: Store copy from prvalue, pass by value\n", __LINE__);
-  }
-  {
-    if (gDebug) {
-      printf(
-          "%d - r3 = "
-          "NewRunnableMethod<StoreCopyPassByValue<Spy>>(&TestByValue, "
-          "Spy(11))\n",
-          __LINE__);
-    }
-    nsCOMPtr<nsIRunnable> r3 = NewRunnableMethod<StoreCopyPassByValue<Spy>>(
-        "TestThreadUtils::ThreadUtilsObject::TestByValue", rpt,
-        &ThreadUtilsObject::TestByValue, Spy(11));
-    EXPECT_EQ(1, gAlive);
-    EXPECT_EQ(1, gConstructions);
-    EXPECT_LE(1, gMoveConstructions);
-    Spy::ClearActions();
-    if (gDebug) {
-      printf("%d - Run()\n", __LINE__);
-    }
-    r3->Run();
-    EXPECT_LE(1, gCopyConstructions);  // Another copy-construction in call.
-    EXPECT_EQ(11, rpt->mSpy.mID);
-    EXPECT_LE(1, gDestructions);
-    EXPECT_EQ(1, gAlive);
-    Spy::ClearActions();
-    if (gDebug) {
-      printf("%d - End block with r\n", __LINE__);
-    }
-  }
-  if (gDebug) {
-    printf("%d - After end block with r\n", __LINE__);
-  }
-  EXPECT_EQ(1, gDestructions);
-  EXPECT_EQ(0, gAlive);
-
-  Spy::ClearAll();
-  {  // Store copy from xvalue, pass by value.
-    nsCOMPtr<nsIRunnable> r4;
-    {
-      Spy s(12);
-      EXPECT_EQ(1, gConstructions);
-      EXPECT_EQ(1, gAlive);
-      Spy::ClearActions();
-      r4 = NewRunnableMethod<StoreCopyPassByValue<Spy>>(
-          "TestThreadUtils::ThreadUtilsObject::TestByValue", rpt,
-          &ThreadUtilsObject::TestByValue, std::move(s));
-      EXPECT_LE(1, gMoveConstructions);
-      EXPECT_EQ(1, gAlive);
-      EXPECT_EQ(1, gZombies);
-      Spy::ClearActions();
-    }
-    EXPECT_EQ(1, gDestructions);
-    EXPECT_EQ(1, gAlive);
-    EXPECT_EQ(0, gZombies);
-    Spy::ClearActions();
-    r4->Run();
-    EXPECT_LE(1, gCopyConstructions);  // Another copy-construction in call.
-    EXPECT_EQ(12, rpt->mSpy.mID);
-    EXPECT_LE(1, gDestructions);
-    EXPECT_EQ(1, gAlive);
-    Spy::ClearActions();
-  }
-  EXPECT_EQ(1, gDestructions);
-  EXPECT_EQ(0, gAlive);
-  // Won't test xvalues anymore, prvalues are enough to verify all rvalues.
-
   Spy::ClearAll();
   if (gDebug) {
     printf("%d - Test: Store copy from lvalue, pass by const lvalue ref\n",
