@@ -7,22 +7,26 @@
 #ifndef TaskQueue_h_
 #define TaskQueue_h_
 
-#include <queue>
-
 #include "mozilla/AbstractThread.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/Monitor.h"
 #include "mozilla/MozPromise.h"
+#include "mozilla/Queue.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/TaskDispatcher.h"
-#include "nsIDelayedRunnableObserver.h"
+#include "mozilla/ThreadSafeWeakPtr.h"
 #include "nsIDirectTaskDispatcher.h"
 #include "nsThreadUtils.h"
 
+#define MOZILLA_TASKQUEUE_IID \
+  {0xb5181e3a, 0x39cf, 0x4d32, {0x81, 0x4a, 0xea, 0x86, 0x94, 0x16, 0x95, 0xd1}}
+
 namespace mozilla {
 
 typedef MozPromise<bool, bool, false> ShutdownPromise;
 
+class TaskQueueTrackerEntry;
+
 // Abstracts executing runnables in order on an arbitrary event target. The
 // runnables dispatched to the TaskQueue will be executed in the order in which
 // they're received, and are guaranteed to not be executed concurrently.
@@ -48,20 +52,20 @@ typedef MozPromise<bool, bool, false> ShutdownPromise;
 // A TaskQueue does not require explicit shutdown, however it provides a
 // BeginShutdown() method that places TaskQueue in a shut down state and returns
 // a promise that gets resolved once all pending tasks have completed
-class TaskQueue : public AbstractThread,
-                  public nsIDirectTaskDispatcher,
-                  public nsIDelayedRunnableObserver {
+class TaskQueue final : public AbstractThread,
+                        public nsIDirectTaskDispatcher,
+                        public SupportsThreadSafeWeakPtr<TaskQueue> {
   class EventTargetWrapper;
 
  public:
-  explicit TaskQueue(already_AddRefed<nsIEventTarget> aTarget,
-                     bool aSupportsTailDispatch = false);
-
-  TaskQueue(already_AddRefed<nsIEventTarget> aTarget, const char* aName,
-            bool aSupportsTailDispatch = false);
-
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSIDIRECTTASKDISPATCHER
+  MOZ_DECLARE_REFCOUNTED_TYPENAME(TaskQueue)
+  NS_INLINE_DECL_STATIC_IID(MOZILLA_TASKQUEUE_IID)
+
+  static RefPtr<TaskQueue> Create(already_AddRefed<nsIEventTarget> aTarget,
+                                  const char* aName,
+                                  bool aSupportsTailDispatch = false);
 
   TaskDispatcher& TailDispatcher() override;
 
@@ -96,18 +100,11 @@ class TaskQueue : public AbstractThread,
   // So we can access nsIEventTarget::Dispatch(nsIRunnable*, uint32_t aFlags)
   using nsIEventTarget::Dispatch;
 
-  // nsIDelayedRunnableObserver
-  void OnDelayedRunnableCreated(DelayedRunnable* aRunnable) override;
-  void OnDelayedRunnableScheduled(DelayedRunnable* aRunnable) override;
-  void OnDelayedRunnableRan(DelayedRunnable* aRunnable) override;
+  NS_IMETHOD RegisterShutdownTask(nsITargetShutdownTask* aTask) override;
+  NS_IMETHOD UnregisterShutdownTask(nsITargetShutdownTask* aTask) override;
 
   using CancelPromise = MozPromise<bool, bool, false>;
 
-  // Dispatches a task to cancel any pending DelayedRunnables. Idempotent. Only
-  // dispatches the task on the first call. Creating DelayedRunnables after this
-  // is called will result in assertion failures.
-  RefPtr<CancelPromise> CancelDelayedRunnables();
-
   // Puts the queue in a shutdown state and returns immediately. The queue will
   // remain alive at least until all the events are drained, because the Runners
   // hold a strong reference to the task queue, and one of them is always held
@@ -130,7 +127,12 @@ class TaskQueue : public AbstractThread,
   bool IsCurrentThreadIn() const override;
   using nsISerialEventTarget::IsOnCurrentThread;
 
- protected:
+ private:
+  friend class SupportsThreadSafeWeakPtr<TaskQueue>;
+
+  TaskQueue(already_AddRefed<nsIEventTarget> aTarget, const char* aName,
+            bool aSupportsTailDispatch);
+
   virtual ~TaskQueue();
 
   // Blocks until all task finish executing. Called internally by methods
@@ -141,23 +143,16 @@ class TaskQueue : public AbstractThread,
   nsresult DispatchLocked(nsCOMPtr<nsIRunnable>& aRunnable, uint32_t aFlags,
                           DispatchReason aReason = NormalDispatch);
 
-  RefPtr<CancelPromise> CancelDelayedRunnablesLocked();
+  void MaybeResolveShutdown();
 
-  // Cancels any scheduled DelayedRunnables on this TaskQueue.
-  void CancelDelayedRunnablesImpl();
+  nsCOMPtr<nsIEventTarget> mTarget MOZ_GUARDED_BY(mQueueMonitor);
 
-  void MaybeResolveShutdown() {
-    mQueueMonitor.AssertCurrentThreadOwns();
-    if (mIsShutdown && !mIsRunning) {
-      mShutdownPromise.ResolveIfExists(true, __func__);
-      mTarget = nullptr;
-    }
-  }
+  // Handle for this TaskQueue being registered with our target if it implements
+  // TaskQueueTracker.
+  UniquePtr<TaskQueueTrackerEntry> mTrackerEntry MOZ_GUARDED_BY(mQueueMonitor);
 
-  nsCOMPtr<nsIEventTarget> mTarget;
-
-  // Monitor that protects the queue, mIsRunning and
-  // mDelayedRunnablesCancelPromise;
+  // Monitor that protects the queue, mIsRunning, mIsShutdown and
+  // mShutdownTasks;
   Monitor mQueueMonitor;
 
   typedef struct TaskStruct {
@@ -166,19 +161,11 @@ class TaskQueue : public AbstractThread,
   } TaskStruct;
 
   // Queue of tasks to run.
-  std::queue<TaskStruct> mTasks;
+  Queue<TaskStruct> mTasks MOZ_GUARDED_BY(mQueueMonitor);
 
-  // DelayedRunnables (from DelayedDispatch) that are managed by their
-  // respective timers, but have not yet run. Accessed only on this
-  // TaskQueue.
-  nsTArray<RefPtr<DelayedRunnable>> mScheduledDelayedRunnables;
-
-  // Manages resolving mDelayedRunnablesCancelPromise.
-  MozPromiseHolder<CancelPromise> mDelayedRunnablesCancelHolder;
-
-  // Set once the task to cancel all pending DelayedRunnables has been
-  // dispatched. Guarded by mQueueMonitor.
-  RefPtr<CancelPromise> mDelayedRunnablesCancelPromise;
+  // List of tasks to run during shutdown.
+  nsTArray<nsCOMPtr<nsITargetShutdownTask>> mShutdownTasks
+      MOZ_GUARDED_BY(mQueueMonitor);
 
   // The thread currently running the task queue. We store a reference
   // to this so that IsCurrentThreadIn() can tell if the current thread
@@ -230,11 +217,12 @@ class TaskQueue : public AbstractThread,
 
   // True if we've dispatched an event to the target to execute events from
   // the queue.
-  bool mIsRunning;
+  bool mIsRunning MOZ_GUARDED_BY(mQueueMonitor);
 
   // True if we've started our shutdown process.
-  bool mIsShutdown;
-  MozPromiseHolder<ShutdownPromise> mShutdownPromise;
+  bool mIsShutdown MOZ_GUARDED_BY(mQueueMonitor);
+  MozPromiseHolder<ShutdownPromise> mShutdownPromise
+      MOZ_GUARDED_BY(mQueueMonitor);
 
   // The name of this TaskQueue. Useful when debugging dispatch failures.
   const char* const mName;
@@ -252,6 +240,35 @@ class TaskQueue : public AbstractThread,
   };
 };
 
+#define MOZILLA_TASKQUEUETRACKER_IID \
+  {0x765c4b56, 0xd5f6, 0x4a9f, {0x91, 0xcf, 0x51, 0x47, 0xb3, 0xc1, 0x7e, 0xa6}}
+
+// XPCOM "interface" which may be implemented by nsIEventTarget implementations
+// which want to keep track of what TaskQueue instances are currently targeting
+// them. This may be used to asynchronously shutdown TaskQueues targeting a
+// threadpool or other event target before the threadpool goes away.
+//
+// This explicitly TaskQueue-aware tracker is used instead of
+// `nsITargetShutdownTask` as the operations required to shut down a TaskQueue
+// are asynchronous, which is not a requirement of that interface.
+class TaskQueueTracker : public nsISupports {
+ public:
+  NS_INLINE_DECL_STATIC_IID(MOZILLA_TASKQUEUETRACKER_IID)
+
+  // Get a strong reference to every TaskQueue currently tracked by this
+  // TaskQueueTracker. May be called from any thraed.
+  nsTArray<RefPtr<TaskQueue>> GetAllTrackedTaskQueues();
+
+ protected:
+  virtual ~TaskQueueTracker();
+
+ private:
+  friend class TaskQueueTrackerEntry;
+
+  Mutex mMutex{"TaskQueueTracker"};
+  LinkedList<TaskQueueTrackerEntry> mEntries MOZ_GUARDED_BY(mMutex);
+};
+
 }  // namespace mozilla
 
 #endif  // TaskQueue_h_
